@@ -30,23 +30,9 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  */
+
 #include "../main/dd.h"
 
-#ifndef lint
-char copyright[] =
-"@(#) Copyright (c) 1985, 1988, 1990 Regents of the University of California.\n\
- All rights reserved.\n";
-#endif /* not lint */
-
-#ifndef lint
-/*static char sccsid[] = "from: @(#)ftpd.c	5.40 (Berkeley) 7/2/91";*/
-static char rcsid[] = "$Id: ddftpd.c,v 1.5.2.1 2001/01/29 00:32:18 bbs Exp $";
-#endif /* not lint */
-
-
-/*
- * FTP server.
- */
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
@@ -58,10 +44,11 @@ static char rcsid[] = "$Id: ddftpd.c,v 1.5.2.1 2001/01/29 00:32:18 bbs Exp $";
 #include <netinet/ip.h>
 
 #define	FTP_NAMES
-#include <arpa/ftp.h>
+#include <ftp.h>
 #include <arpa/inet.h>
 #include <arpa/telnet.h>
 
+#include <stdarg.h>
 #include <stdio.h>
 #include <grp.h>
 #include <signal.h>
@@ -71,12 +58,14 @@ static char rcsid[] = "$Id: ddftpd.c,v 1.5.2.1 2001/01/29 00:32:18 bbs Exp $";
 #ifdef SHADOW_PWD
 #include <shadow.h>
 #endif
+#ifdef HAVE_UTMPX_H
+#include <utmpx.h>
+#endif
 #include <pwd.h>
 #include <setjmp.h>
 #include <netdb.h>
 #include <errno.h>
 #include <syslog.h>
-#include <varargs.h>
 #include <unistd.h>
 #include <ctype.h>
 #include <stdlib.h>
@@ -85,6 +74,8 @@ static char rcsid[] = "$Id: ddftpd.c,v 1.5.2.1 2001/01/29 00:32:18 bbs Exp $";
 
 #include <global.h>
 #include <md5.h>
+#include <ddftp.h>
+#include <ddcommon.h>
 
 #define MD_CTX MD5_CTX
 #define MDInit MD5Init
@@ -127,7 +118,7 @@ int	stru;			/* avoid C keyword */
 int	mode;
 int	usedefault = 1;		/* for data transfers */
 int	pdata = -1;		/* for passive mode */
-int     ulconf=0;
+int     ulconf = 0;
 int	transflag;
 off_t	file_size;
 off_t	byte_count;
@@ -167,35 +158,56 @@ struct DayDream_Conference *confs=0;
 
 int mydir=0;
 char myroot[1024];
-char * fgetsnolf(char *buf, int n, FILE *fh);
+static void end_login(void);
+static char *fgetsnolf(char *buf, int n, FILE *fh);
+static int findfile(const char *file, char *de, struct DayDream_Conference *tc);
+static void _splitpath(char *file, char *path, char *name);
+static int cmppasswds(char *passwd, unsigned char *thepw);
+static int dupecheck(const char *name);
+static int receive_data(FILE *instr, FILE *outstr);
+static void send_data(FILE *instr, FILE *outstr, off_t blksize);
+static int checkpath(char *pa);
+
+static struct sockaddr_in passive_ip;
+static int forced_passive_ip = 0;
 
 struct ftpinfo minfo;
 char *infoname=0;
+static char *daydream_path = NULL;
 
-getddconfig()
+int getddconfig(void)
 {
 	char buf[1024];
 	int fd;
 	struct stat fib;
 	unsigned char *s;
 	
-	sprintf(buf,"%sdata/daydream.dat",datadir);
-	fd=open(buf,O_RDONLY);
-	read(fd,&mcfg,sizeof(struct DayDream_MainConfig));
+	snprintf(buf, sizeof(buf), "%sdata/daydream.dat", datadir);
+	if ((fd = open(buf, O_RDONLY)) == -1)
+		return -1;
+	if (read(fd, &mcfg, sizeof(struct DayDream_MainConfig)) !=
+		sizeof(struct DayDream_MainConfig))
+		return -1;
 	close(fd);
 
-	sprintf(buf,"%sdata/conferences.dat",datadir);
-	fd=open(buf,O_RDONLY);
-	
-	fstat(fd,&fib);
-	
-	confs=(struct DayDream_Conference *)malloc(fib.st_size+2);
-	read(fd,confs,fib.st_size);
+	snprintf(buf, sizeof(buf), "%sdata/conferences.dat", datadir);
+	if ((fd = open(buf,O_RDONLY)) == -1)
+		return -1;
+	if (fstat(fd,&fib) == -1)
+		return -1;
+	if (fib.st_size % sizeof(struct DayDream_Conference))
+		return -1;
+	confs = (struct DayDream_Conference *) malloc(fib.st_size + 2);
+	if (confs == NULL)
+		return -1;
+	if (read(fd, confs, fib.st_size) != fib.st_size)
+		return -1;
 	close(fd);
 	s=(unsigned char *)confs;
 	s[fib.st_size]=255;
 
 	memset(&minfo,0,sizeof(struct ftpinfo));
+	return 0;
 }
 
 void showfile(const char *foo)
@@ -215,14 +227,23 @@ void showfile(const char *foo)
 		fclose(fi);
 	}
 }
-main(argc, argv, envp)
-	int argc;
-	char *argv[];
-	char **envp;
+
+static int doinitgroups(uid_t uid, gid_t gid)
+{
+	struct passwd *pwd;
+
+	if ((pwd = getpwuid(uid)) == NULL)
+		return -1;
+	return initgroups(pwd->pw_name, gid);
+}
+
+int main(int argc, char *argv[], char *envp[])
 {
 	int addrlen, on = 1, tos;
 	char *cp;
 
+	init_common_library(argc, argv);
+
 	/*
 	 * LOG_NDELAY sets up the logging connection immediately,
 	 * necessary for anonymous ftp's that chroot and can't do it later.
@@ -300,9 +321,34 @@ main(argc, argv, envp)
 			goto nextopt;
 		    }
 		case 'D':
-			strcpy(datadir,++cp);
-			getddconfig();
-			setenv("DAYDREAM",datadir,1);
+			/* FIXME: buffer overflow */
+			strlcpy(datadir, ++cp, sizeof(datadir));
+			if (getddconfig() == -1) {
+				fprintf(stderr, "ftpd: cannot load config\n");
+				exit(1);
+			}
+			setenv("DAYDREAM", datadir, 1);
+			goto nextopt;
+		case 'P':
+			if (inet_pton(AF_INET, ++cp, 
+				&passive_ip.sin_addr) <= 0) {
+				fputs("ftpd: invalid forced passive IP\n",
+					stderr);
+				exit(1);
+			}
+			forced_passive_ip = 1;
+			goto nextopt;
+		case 'p':
+			if (getenv("PATH") == NULL)
+				setenv("PATH", strdup(++cp), 1);
+			else {
+				cp++;
+				daydream_path = (char *) malloc(strlen(cp) +
+					strlen(getenv("PATH")) + 2);
+				sprintf(daydream_path, "%s:%s", 
+					getenv("PATH"), cp);
+				setenv("PATH", daydream_path, 1);
+			}
 			goto nextopt;
 			
 		default:
@@ -317,6 +363,12 @@ nextopt:
 		fprintf(stderr,"No config loaded!\n");
 		exit(1);
 	}
+
+	if (doinitgroups(mcfg.CFG_BBSUID, mcfg.CFG_BBSGID) == -1) {
+		syslog(LOG_ERR, "initgroups() failed");
+		exit(1);
+	}
+
 	(void) freopen(_PATH_DEVNULL, "w", stderr);
 	(void) signal(SIGPIPE, lostconn);
 	(void) signal(SIGCHLD, SIG_IGN);
@@ -351,6 +403,42 @@ nextopt:
 	/* NOTREACHED */
 }
 
+#ifndef HAVE_SETPROCTITLE
+/*
+ * clobber argv so ps will show what we're doing.
+ * (stolen from sendmail)
+ * warning, since this is usually started from inetd.conf, it
+ * often doesn't have much of an environment or arglist to overwrite.
+ */
+
+/*VARARGS1*/
+void setproctitle(fmt, a, b, c)
+char *fmt;
+{
+	register char *p, *bp, ch;
+	register int i;
+	char buf[BUFSIZ];
+
+	(void) sprintf(buf, fmt, a, b, c);
+
+	/* make ps print our process name */
+	p = Argv[0];
+	*p++ = '-';
+
+	i = strlen(buf);
+	if (i > LastArgv - p - 2) {
+		i = LastArgv - p - 2;
+		buf[i] = '\0';
+	}
+	bp = buf;
+	while ((ch = *bp++) != 0)
+		if (ch != '\n' && ch != '\r')
+			*p++ = ch;
+	while (p < LastArgv)
+		*p++ = ' ';
+}
+#endif /* HAVE_SETPROCTITLE */
+
 void
 lostconn()
 {
@@ -361,39 +449,14 @@ lostconn()
 
 static char ttyline[20];
 
-/*
- * Helper function for sgetpwnam().
- */
-char *
-sgetsave(s)
-	char *s;
-{
-	char *new = malloc((unsigned) strlen(s) + 1);
-
-	if (new == NULL) {
-		perror_reply(421, "Local resource failure: malloc");
-		dologout(1);
-		/* NOTREACHED */
-	}
-	(void) strcpy(new, s);
-	return (new);
-}
-
-int strupr (char *strh)
-{
-	        while (*strh)
-	{
-		                *strh++=toupper(*strh);
-	}
-}
-
-char * filepart(char *s)
+static char *filepart(char *s)
 {
 	char *t;
-	t=&s[strlen(s)];
 
-	while(s!=t) {
-		if (*t=='/') return t+1;
+	t = s + strlen(s); 
+	while(s != t) {
+		if (*t == '/') 
+			return t + 1;
 		t--;
 	}
 	return t;
@@ -404,8 +467,7 @@ char * filepart(char *s)
  * the data returned must not be clobbered by any other command
  * (e.g., globbing).
  */
-
-struct userbase *ddgetpwnam(char *name)
+static struct userbase *ddgetpwnam(char *name)
 {
 	int fd;
 	char buf[1024];
@@ -423,47 +485,11 @@ struct userbase *ddgetpwnam(char *name)
 			}
 		}
 		close(fd);
-	}
+	} else 
+		syslog(LOG_ERR, "cannot open userbase \"%sdata/userbase.dat\" (%m)", datadir);
 	return 0;
 }
 
-/*
-struct passwd *
-sgetpwnam(name)
-	char *name;
-{
-	static struct passwd save;
-	register struct passwd *p;
-	char *sgetsave();
-#ifdef SHADOW_PWD
-	struct spwd *sp;
-#endif
-
-	if ((p = getpwnam(name)) == NULL)
-		return (p);
-#ifdef SHADOW_PWD
-	sp = getspnam(name);
-	if (sp)
-		p->pw_passwd = sp->sp_pwdp;
-	endspent();
-#endif
-	if (save.pw_name) {
-		free(save.pw_name);
-		free(save.pw_passwd);
-		free(save.pw_gecos);
-		free(save.pw_dir);
-		free(save.pw_shell);
-	}
-	save = *p;
-	save.pw_name = sgetsave(p->pw_name);
-
-	save.pw_passwd = sgetsave(p->pw_passwd);
-	save.pw_gecos = sgetsave(p->pw_gecos);
-	save.pw_dir = sgetsave(p->pw_dir);
-	save.pw_shell = sgetsave(p->pw_shell);
-	return (&save);
-}
-*/
 int login_attempts;		/* number of failed login attempts */
 int askpasswd;			/* had user command, ask for passwd */
 
@@ -478,11 +504,8 @@ int askpasswd;			/* had user command, ask for passwd */
  * shell as returned by getusershell().  Disallow anyone mentioned in the file
  * _PATH_FTPUSERS to allow people such as root and uucp to be avoided.
  */
-user(name)
-	char *name;
+void user(char *name)
 {
-	register char *cp;
-	char *shell;
 	char *getusershell();
 
 	if (logged_in) {
@@ -510,18 +533,12 @@ user(name)
 		sleep((unsigned) login_attempts);
 }
 
-int ispid(pid_t pid)
+static int ispid(pid_t pid)
 {
-	char ibuf[1024];
-	struct stat st;
-	
-	sprintf(ibuf,"/proc/%d",pid);
-
-	if (stat(ibuf,&st)==-1) return 0; 
-	return 1;
+	return kill(pid, 0) != -1;
 }
 
-int checkconfaccess(int confn)
+static int checkconfaccess(int confn)
 {
         int newcn=confn-1;
 
@@ -534,7 +551,6 @@ int checkconfaccess(int confn)
         return 0;
 }
 
-
 int setulconf(int cn)
 {
 	struct DayDream_Conference *tconf=confs;
@@ -547,7 +563,8 @@ int setulconf(int cn)
 			ulconf=cn;
 			return 1;
 		}
-                (struct DayDream_Conference *)tbase=tconf+1;
+                /* FIXME: Very suspicious code */
+		tbase = (struct DayDream_MsgBase *) tconf + 1;
 		bcnt=tconf->CONF_MSGBASES;
 		while(bcnt) {
 			tbase++;
@@ -567,8 +584,8 @@ char *getddinfo()
 	char *res=0;
 	int freeh=0;
 	for (i=0;i<mcfg.CFG_MAXFTPUSERS;i++) {
-		sprintf(nam,"%sftpinfo%d.dat",DDTMP,i+1);
-		infd=open(nam,O_RDONLY);
+		sprintf(nam, "%s/ftpinfo%d.dat", DDTMP, i+1);
+		infd = open(nam, O_RDONLY);
 		if (infd<0) {
 		kamake:
 			freeh=i+1;
@@ -583,7 +600,7 @@ char *getddinfo()
 		}
 	}
 	if (res) {
-		sprintf(nam,"%sftpinfo%d.dat",DDTMP,freeh);
+		sprintf(nam, "%s/ftpinfo%d.dat", DDTMP, freeh);
 		infd=open(nam,O_WRONLY|O_CREAT|O_TRUNC,0644);
 		minfo.pid=getpid();
 		minfo.userid=pw->user_account_id;
@@ -594,20 +611,19 @@ char *getddinfo()
 	return res;
 }
 
-int updateinfo()
+static void updateinfo(void)
 {
 	int infd=open(infoname,O_WRONLY|O_CREAT|O_TRUNC,0644);
 	minfo.pid=getpid();
 	minfo.userid=pw->user_account_id;
-	if (infd<0) return 0;
+	if (infd<0) return;
 	write(infd,&minfo,sizeof(struct ftpinfo));
 	close(infd);
 }
 /*
  * Check if a user is in the file _PATH_FTPUSERS
  */
-checkuser(name)
-	char *name;
+int checkuser(char *name)
 {
 	register FILE *fd;
 	register char *p;
@@ -615,7 +631,7 @@ checkuser(name)
 
 	if ((fd = fopen(_PATH_FTPUSERS, "r")) != NULL) {
 		while (fgets(line, sizeof(line), fd) != NULL)
-			if ((p = index(line, '\n')) != NULL) {
+			if ((p = strchr(line, '\n')) != NULL) {
 				*p = '\0';
 				if (line[0] == '#')
 					continue;
@@ -627,25 +643,50 @@ checkuser(name)
 	return (0);
 }
 
+static void update_wtmp(void)
+{
+#ifdef HAVE_UPDWTMPX
+	struct utmpx ut;
+	ut.ut_user[0] = '\0';
+	ut.ut_id[0] = 'f';
+	ut.ut_id[1] = 't';
+	ut.ut_id[2] = 'p';
+	ut.ut_id[3] = '\xff';
+	ut.ut_type = USER_PROCESS;
+	strlcpy(ut.ut_line, ttyline, sizeof(ut.ut_line)); 
+	ut.ut_pid = getpid();
+#ifdef HAVE_STRUCT_UTMPX_E_EXIT
+	ut.ut_exit.e_termination = 0;
+	ut.ut_exit.e_exit = 0;
+#endif
+	ut.ut_session = 0;
+	gettimeofday(&ut.ut_tv, NULL);
+#ifdef HAVE_STRUCT_UTMPX_UT_SYSLEN
+	ut.ut_syslen = 1;
+#endif
+	ut.ut_host[0] = '\0';
+#else
+	logwtmp(ttyline, "", "");
+#endif
+}
+
 /*
  * Terminate login as previous user, if any, resetting state;
  * used when USER command is given or login fails.
  */
-end_login()
+static void end_login(void)
 {
 
 	(void) seteuid((uid_t)0);
-	if (logged_in)
-		logwtmp_mod(ttyline, "", "");
+	if (logged_in) 
+		update_wtmp();
 	pw = NULL;
 	logged_in = 0;
 	guest = 0;
 }
 
-pass(passwd)
-	char *passwd;
+void pass(char *passwd)
 {
-	char *xpasswd, *salt;
 	char buf[1024];
 	if (logged_in || askpasswd == 0) {
 		reply(503, "Login with USER first.");
@@ -667,9 +708,6 @@ pass(passwd)
 	}
 	
 	login_attempts = 0;		/* this time successful */
-//	(void) setegid((gid_t)mcfg.CFG_BBSGID);
-//	(void) seteuid((gid_t)mcfg.CFG_BBSUID);
-//	(void) initgroups(pw->pw_name, pw->pw_gid);
 
 	/* open wtmp before chroot */
 	(void)sprintf(ttyline, "ftp%d", getpid());
@@ -692,8 +730,12 @@ pass(passwd)
 	}
 	getcwd(myroot, sizeof(myroot));
 	seteuid((uid_t)0);
+	if (setegid((gid_t) mcfg.CFG_BBSGID) < 0) {
+		reply(550, "Cannot set GID.");
+		goto bad;
+	}
 	if (seteuid((uid_t)mcfg.CFG_BBSUID) < 0) {
-		reply(550, "Can't set uid.");
+		reply(550, "Cannot set UID.");
 		goto bad;
 	}
 	infoname=getddinfo();
@@ -708,7 +750,6 @@ pass(passwd)
         lreply(230, "Help");
 	printf("Upload files to ul/ dir. After uploading log on to the BBS and hit U\nin the destination conference.\n\nFiles to DL are in dl/ dir. Please remove them after you've leeched them.\n\n");
 	reply(230, "User %s logged in.", pw->user_handle);
-	setulconf(pw->user_joinconference);
 
 	sprintf(proctitle, "%s: %s", remotehost, pw->user_handle);
 	setproctitle(proctitle);
@@ -721,8 +762,7 @@ bad:
 	end_login();
 }
 
-retrieve(cmd, name)
-	char *cmd, *name;
+void retrieve(char *cmd, char *name)
 {
 	FILE *fin, *dout;
 	struct stat st;
@@ -785,8 +825,7 @@ done:
 	(*closefunc)(fin);
 }
 
-
-_store(char *name, const char *mode, int unique)
+void _store(char *name, const char *mode, int unique)
 {
 	FILE *fout, *din;
 	struct stat st;
@@ -835,7 +874,7 @@ _store(char *name, const char *mode, int unique)
 			 * because we are changing from reading to
 			 * writing.
 			 */
-			if (fseek(fout, 0L, L_INCR) < 0) {
+			if (fseek(fout, 0L, SEEK_CUR) < 0) {
 				perror_reply(550, name);
 				goto done;
 			}
@@ -854,8 +893,9 @@ _store(char *name, const char *mode, int unique)
 		getcwd(&buf[512],512);
 		if (ulconf) {
 			chdir(datadir);
-			sprintf(buf,"daydream -u %d %d %s/%s",pw->user_account_id,
-				ulconf,&buf[512],filepart(name));
+			sprintf(buf, "daydream -u %d %d %s/%s",
+				pw->user_account_id, ulconf,
+				&buf[512], filepart(name));
 			system(buf);
 			chdir(&buf[512]);
 		}
@@ -873,9 +913,7 @@ done:
 	(*closefunc)(fout);
 }
 
-FILE *
-getdatasock(mode)
-	char *mode;
+FILE *getdatasock(char *mode)
 {
 	int s, on = 1, tries;
 
@@ -912,11 +950,7 @@ bad:
 	return (NULL);
 }
 
-FILE *
-dataconn(name, size, mode)
-	char *name;
-	off_t size;
-	char *mode;
+FILE *dataconn(char *name, off_t size, char *mode)
 {
 	char sizebuf[32];
 	FILE *file;
@@ -993,9 +1027,7 @@ dataconn(name, size, mode)
  *
  * NB: Form isn't handled.
  */
-send_data(instr, outstr, blksize)
-	FILE *instr, *outstr;
-	off_t blksize;
+static void send_data(FILE *instr, FILE *outstr, off_t blksize)
 {
 	register int c, cnt;
 	register char *buf;
@@ -1090,8 +1122,7 @@ file_err:
  *
  * N.B.: Form isn't handled.
  */
-receive_data(instr, outstr)
-	FILE *instr, *outstr;
+static int receive_data(FILE *instr, FILE *outstr)
 {
 	register int c;
 	int cnt, bare_lfs = 0;
@@ -1177,8 +1208,7 @@ file_err:
 	return (-1);
 }
 
-statfilecmd(filename)
-	char *filename;
+void statfilecmd(char *filename)
 {
 	char line[BUFSIZ];
 	FILE *fin;
@@ -1208,12 +1238,12 @@ statfilecmd(filename)
 	reply(211, "End of Status");
 }
 
-statcmd()
+void statcmd(void)
 {
 	struct sockaddr_in *sin;
 	u_char *a, *p;
 
-	lreply(211, "%s FTP server status:", hostname, version);
+	lreply(211, "%s FTP server status:", hostname);
 	printf("     %s\r\n", version);
 	printf("     Connected to %s", remotehost);
 	if (!isdigit(remotehost[0]))
@@ -1260,8 +1290,7 @@ printaddr:
 	reply(211, "End of status");
 }
 
-fatal(s)
-	char *s;
+void fatal(char *s)
 {
 	reply(451, "Error in server: %s\n", s);
 	reply(221, "Closing connection due to server error.");
@@ -1269,67 +1298,55 @@ fatal(s)
 	/* NOTREACHED */
 }
 
-/* VARARGS2 */
-reply(n, fmt, p0, p1, p2, p3, p4, p5)
-	int n;
-	char *fmt;
+void reply(int n, const char *fmt, ...)
 {
+	va_list args;
+	va_start(args, fmt);
 	printf("%d ", n);
-	printf(fmt, p0, p1, p2, p3, p4, p5);
+	vprintf(fmt, args);
 	printf("\r\n");
-	(void)fflush(stdout);
-	if (debug) {
-		syslog(LOG_DEBUG, "<--- %d ", n);
-		syslog(LOG_DEBUG, fmt, p0, p1, p2, p3, p4, p5);
-}
+	fflush(stdout);
+	va_end(args);
 }
 
-/* VARARGS2 */
-lreply(n, fmt, p0, p1, p2, p3, p4, p5)
-	int n;
-	char *fmt;
+void lreply(int n, const char *fmt, ...)
 {
+	va_list args;
+	va_start(args, fmt);
 	printf("%d- ", n);
-	printf(fmt, p0, p1, p2, p3, p4, p5);
+	vprintf(fmt, args);
 	printf("\r\n");
-	(void)fflush(stdout);
-	if (debug) {
-		syslog(LOG_DEBUG, "<--- %d- ", n);
-		syslog(LOG_DEBUG, fmt, p0, p1, p2, p3, p4, p5);
-	}
+	fflush(stdout);
+	va_end(args);
 }
 
-ack(s)
-	char *s;
+void ack(char *s)
 {
 	reply(250, "%s command successful.", s);
 }
 
-nack(s)
-	char *s;
+void nack(const char *s)
 {
 	reply(502, "%s command not implemented.", s);
 }
 
 /* ARGSUSED */
-yyerror(s)
-	char *s;
+void yyerror(char *s)
 {
 	char *cp;
 
-	if (cp = index(cbuf,'\n'))
+	if ((cp = strchr(cbuf,'\n')) != 0)
 		*cp = '\0';
 	reply(500, "'%s': command not understood.", cbuf);
 }
 
-_delete(name)
-	char *name;
+void _delete(char *name)
 {
 	struct stat st;
 
 	if (!checkpath(name)) {
 		reply(550,"Illegal filename");
-		return 0;
+		return;
 	}
 
 	if (stat(name, &st) < 0) {
@@ -1341,46 +1358,38 @@ _delete(name)
 		perror_reply(550, name);
 		return;
 	}
-done:
 	ack("DELE");
 }
 
-cwd(path)
-	char *path;
+void cwd(char *path)
 {
-	char *cwd;
 	if (checkpath(path)) {
 		chdir(path);
 		if (strstr(path,"ul")) showfile("ftp-upload.txt");
 		ack("CWD");
-	} else {
-		//			chdir(myroot);
+	} else 
 		reply(550,"Don't even try it.");
-	} 
-	
 }
 
-makedir(name)
-	char *name;
+void makedir(char *name)
 {
 	if (!checkpath(name)) {
 		reply(550,"Illegal filename");
-		return 0;
+		return;
 	}
 	mkdir(name,0755);
 }
 
-removedir(name)
-	char *name;
+void removedir(char *name)
 {
 	if (!checkpath(name)) {
 		reply(550,"Illegal filename");
-		return 0;
+		return;
 	}
 	rmdir (name);
 }
 
-pwd()
+void pwd(void)
 {
 	char path[MAXPATHLEN + 1];
 
@@ -1390,9 +1399,7 @@ pwd()
 		reply(257, "\"%s\" is current directory.", path);
 }
 
-char *
-renamefrom(name)
-	char *name;
+char *renamefrom(char *name)
 {
 	struct stat st;
 	
@@ -1408,12 +1415,12 @@ renamefrom(name)
 	return (name);
 }
 
-renamecmd(from, to)
-	char *from, *to;
+/* FIXME: 'to' should not contain '/' */
+void renamecmd(char *from, char *to)
 {
-	if (!checkpath(from) || !checkpath(to,'/')) {
+	if (!checkpath(from) || !checkpath(to)) {
 		reply(550, "Illegal filename");
-		return 0;
+		return;
 	}
 	if (rename(from, to) < 0)
 		perror_reply(550, "rename");
@@ -1421,8 +1428,7 @@ renamecmd(from, to)
 		ack("RNTO");
 }
 
-dolog(sin)
-	struct sockaddr_in *sin;
+void dolog(struct sockaddr_in *sin)
 {
 	struct hostent *hp = gethostbyaddr((char *)&sin->sin_addr,
 		sizeof (struct in_addr), AF_INET);
@@ -1449,12 +1455,11 @@ dolog(sin)
  * Record logout in wtmp file
  * and exit with supplied status.
  */
-dologout(status)
-	int status;
+void dologout(int status)
 {
 	if (logged_in) {
 		(void) seteuid((uid_t)0);
-		logwtmp_mod(ttyline, "", "");
+		update_wtmp();
 	}
 	if (infoname) unlink(infoname);
 
@@ -1462,8 +1467,7 @@ dologout(status)
 	_exit(status);
 }
 
-void
-myoob()
+void myoob(void)
 {
 	char *cp;
 
@@ -1497,12 +1501,17 @@ myoob()
  * 	a legitimate response by Jon Postel in a telephone conversation
  *	with Rick Adams on 25 Jan 89.
  */
-passive()
+void passive(void)
 {
 	int len;
 	u_short port;
 	register char *p, *a;
 
+	if (!logged_in) {
+		reply(530, "You are not logged in");
+		return;
+	}
+
 	pdata = socket(AF_INET, SOCK_STREAM, 0);
 	if (pdata < 0) {
 		perror_reply(425, "Can't open passive connection");
@@ -1545,6 +1554,8 @@ passive()
 	if (listen(pdata, 1) < 0)
 		goto pasv_error;
 	a = (char *) &pasv_addr.sin_addr;
+	if (forced_passive_ip)
+		a = (char *) &passive_ip.sin_addr;
 	p = (char *) &pasv_addr.sin_port;
 
 #define UC(b) (((int) b) & 0xff)
@@ -1565,13 +1576,11 @@ pasv_error:
  * The file named "local" is already known to exist.
  * Generates failure reply on error.
  */
-char *
-gunique(local)
-	char *local;
+char *gunique(char *local)
 {
 	static char new[MAXPATHLEN];
 	struct stat st;
-	char *cp = rindex(local, '/');
+	char *cp = strrchr(local, '/');
 	int count = 0;
 
 	if (cp)
@@ -1597,9 +1606,7 @@ gunique(local)
 /*
  * Format and send reply containing system error number.
  */
-perror_reply(code, string)
-	int code;
-	char *string;
+void perror_reply(int code, char *string)
 {
 	reply(code, "%s: %s.", string, strerror(errno));
 }
@@ -1609,8 +1616,7 @@ static char *onefile[] = {
 	0
 };
 
-send_file_list(whichfiles)
-	char *whichfiles;
+void send_file_list(char *whichfiles)
 {
 	struct stat st;
 	DIR *dirp = NULL;
@@ -1642,7 +1648,7 @@ send_file_list(whichfiles)
 		transflag = 0;
 		return;
 	}
-	while (dirname = *dirlist++) {
+	while ((dirname = *dirlist++) != NULL) {
 		if (stat(dirname, &st) < 0) {
 			/*
 			 * If user typed "ls -l", etc, and the client
@@ -1737,43 +1743,7 @@ send_file_list(whichfiles)
 	pdata = -1;
 }
 
-#ifndef HAVE_SETPROCTITLE
-/*
- * clobber argv so ps will show what we're doing.
- * (stolen from sendmail)
- * warning, since this is usually started from inetd.conf, it
- * often doesn't have much of an environment or arglist to overwrite.
- */
-
-/*VARARGS1*/
-setproctitle(fmt, a, b, c)
-char *fmt;
-{
-	register char *p, *bp, ch;
-	register int i;
-	char buf[BUFSIZ];
-
-	(void) sprintf(buf, fmt, a, b, c);
-
-	/* make ps print our process name */
-	p = Argv[0];
-	*p++ = '-';
-
-	i = strlen(buf);
-	if (i > LastArgv - p - 2) {
-		i = LastArgv - p - 2;
-		buf[i] = '\0';
-	}
-	bp = buf;
-	while (ch = *bp++)
-		if (ch != '\n' && ch != '\r')
-			*p++ = ch;
-	while (p < LastArgv)
-		*p++ = ' ';
-}
-#endif /* HAVE_SETPROCTITLE */
-
-int cmppasswds(char *passwd, unsigned char *thepw)
+static int cmppasswds(char *passwd, unsigned char *thepw)
 {
 	MD_CTX context;
 	unsigned char digest[16];
@@ -1793,7 +1763,7 @@ int cmppasswds(char *passwd, unsigned char *thepw)
 	return(1);
 }
 
-int dupecheck(char *name)
+static int dupecheck(const char *name)
 {
 	struct DayDream_Conference *tconf=confs;
 	struct DayDream_MsgBase *tbase;
@@ -1806,7 +1776,8 @@ int dupecheck(char *name)
 				return 1;
 			}
 		}
-		(struct DayDream_Conference *)tbase=tconf+1;
+		/* FIXME: very suspicious code */
+		tbase = (struct DayDream_MsgBase *) tconf + 1;
 		bcnt=tconf->CONF_MSGBASES;
 		while(bcnt) {
 			tbase++;
@@ -1817,13 +1788,13 @@ int dupecheck(char *name)
 	return 0;
 }
 
-int findfile(char *file, char *de, struct DayDream_Conference *tc)
+static int findfile(const char *file, char *de, struct DayDream_Conference *tc)
 {
 	char buf1[1024];
 	char buf2[1024];
 	int fd1;
 	FILE *plist;
-	char *s;
+	const char *s;
 	DIR *dh;
 	struct dirent *dent;
 	struct DayDream_Conference *co=tc;
@@ -1834,7 +1805,7 @@ int findfile(char *file, char *de, struct DayDream_Conference *tc)
 		if (*s=='/') return 0;
 		s++;
 	}	
-	sprintf(buf1,"%sdata/paths.dat",co->CONF_PATH);
+	sprintf(buf1, "%s/data/paths.dat", co->CONF_PATH);
 	if ((plist=fopen(buf1,"r"))) {
 		while(fgetsnolf(buf1,512,plist))
 		{
@@ -1850,7 +1821,7 @@ int findfile(char *file, char *de, struct DayDream_Conference *tc)
 		fclose(plist);
 	}
 
-	sprintf(buf1,"%sdata/paths.dat",co->CONF_PATH);
+	sprintf(buf1, "%s/data/paths.dat", co->CONF_PATH);
 	if ((plist=fopen(buf1,"r"))) {
 		while(fgetsnolf(buf1,512,plist))
 		{
@@ -1873,7 +1844,7 @@ int findfile(char *file, char *de, struct DayDream_Conference *tc)
 	return 0;
 }
 
-char * fgetsnolf(char *buf, int n, FILE *fh)
+static char *fgetsnolf(char *buf, int n, FILE *fh)
 {
 	char *hih;
 	char *s;
@@ -1892,7 +1863,7 @@ char * fgetsnolf(char *buf, int n, FILE *fh)
 	return hih;
 }
 
-int checkpath(char *pa)
+static int checkpath(char *pa)
 {
 	char olddir[1024];
 	struct stat st;
@@ -1900,7 +1871,7 @@ int checkpath(char *pa)
 	
 	getcwd(olddir, sizeof(olddir));
 	if ( (stat(pa, &st) <0) || (S_ISDIR(st.st_mode)==0) ) {
-		_splitpath(pa,&buf,0);
+		_splitpath(pa,buf,0);
 		if (!*buf) return 1;
 	} else {
 		strcpy(buf,pa);
@@ -1915,7 +1886,7 @@ int checkpath(char *pa)
 	return 0;
 }
 
-int _splitpath(char *file, char *path, char *name)
+static void _splitpath(char *file, char *path, char *name)
 {
 	char *s, *t;
 	
