@@ -1,12 +1,7 @@
-/*
-	common.c
-	
-	common stuff for the ttysnoops & ttysnoop programs
-*/
-
-#include <stdio.h>
-#include <termios.h>
 #include <stdarg.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <termios.h>
 
 #include "common.h"
 
