@@ -10,6 +10,11 @@
 #include "config.h"
 #include "../main/dd.h"
 
+#include <zm.h>
+#include <zmodem.h>
+#include <crctab.h>
+#include <rbsb.h>
+
 #if STDC_HEADERS
 # include <string.h>
 #else
@@ -29,11 +34,16 @@ char *strchr(), *strrchr();
 
 #define SS_NORMAL 0
 #define LOGFILE "/var/adm/rzlog"
+#include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
 #include <ctype.h>
 #include <errno.h>
+#include <fcntl.h>
+#include <unistd.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #ifdef TIME_WITH_SYS_TIME
 #  include <sys/time.h>
 #  include <time.h>
@@ -44,44 +54,16 @@ char *strchr(), *strrchr();
 #    include <time.h>
 #  endif
 #endif
-#include "timing.h"
+
+#include <sys/ioctl.h>
+#include <termios.h>
+
 extern int errno;
 FILE *popen();
 
-#define OK 0
-#define FALSE 0
-#define TRUE 1
-#define ERROR (-1)
-
 #define MAX_BLOCK 8192
 
-/*
- * Max value for HOWMANY is 255 if NFGVMIN is not defined.
- *   A larger value reduces system overhead but may evoke kernel bugs.
- *   133 corresponds to an XMODEM/CRC sector
- */
-#ifndef HOWMANY
-#ifdef NFGVMIN
-#define HOWMANY MAX_BLOCK
-#else
-#define HOWMANY 255
-#endif
-#endif
-
 /* Ward Christensen / CP/M parameters - Don't change these! */
-#define ENQ 005
-#define CAN ('X'&037)
-#define XOFF ('s'&037)
-#define XON ('q'&037)
-#define SOH 1
-#define STX 2
-#define EOT 4
-#define ACK 6
-#define NAK 025
-#define CPMEOF 032
-#define WANTCRC 0103		/* send C not NAK to get crc not checksum */
-#define TIMEOUT (-2)
-#define RCDO (-3)
 #define ERRORMAX 5
 #define RETRYMAX 5
 #define WCEOT (-10)
@@ -89,11 +71,9 @@ FILE *popen();
 #define UNIXFILE 0xF000		/* The S_IFMT file mask bit for stat */
 int Zmodem = 0;			/* ZMODEM protocol requested */
 int Nozmodem = 0;		/* If invoked as "rb" */
-unsigned Baudrate = 2400;
 
-#include "rbsb.c"		/* most of the system dependent stuff here */
+#define HOWMANY MAX_BLOCK
 
-#include "crctab.c"
 #include <dirent.h>
 
 char *fgetsnolf(char *buf, int n, FILE * fh);
@@ -116,7 +96,6 @@ int Lastrx;
 int Crcflg;
 int Firstsec;
 int Eofseen;			/* indicates cpm eof (^Z) has been received */
-int errors;
 int Restricted = 1;		/* restricted; no /.. or ../ in filenames */
 int Readnum = HOWMANY;		/* Number of bytes to ask for in read() from modem */
 
@@ -145,7 +124,6 @@ int Thisbinary;			/* current file is to be received in bin mode */
 int Blklen;			/* record length of received packets */
 long rxbytes;
 int try_resume = FALSE;
-int allow_remote_commands = FALSE;
 int no_timeout = FALSE;
 
 #ifdef SEGMENTS
@@ -162,28 +140,49 @@ int in_timesync = 0;
 
 char linbuf[HOWMANY];
 int Lleft = 0;			/* number of characters in linbuf */
-time_t timep[2];
+
 char Lzmanag;			/* Local file management request */
 char zconv;			/* ZMODEM file conversion request */
 char zmanag;			/* ZMODEM file management request */
 char ztrans;			/* ZMODEM file transport request */
-int Zctlesc;			/* Encode control characters */
+
 int Zrwindow = 1400;		/* RX window size (controls garbage count) */
 
 time_t lup = 0;
 struct DayDream_NodeInfo nin;
 
 int closeok = 0;
-#define xsendline(c) sendline(c)
 
-char *readline_ptr;		/* pointer for removing chars from linbuf */
-#define READLINE_PF(timeout) \
-	(--Lleft >= 0? (*readline_ptr++ & 0377) : readline(timeout))
-
-#include "zm.c"
+char *readline_ptr;           /* pointer for removing chars from linbuf */
 
+int readline(int timeout);
+	
 int tryzhdrtype = ZRINIT;	/* Header type to send corresponding to Last rx close */
 
+static void ackbibi(void);
+static void zmputs(char *s);
+static int rzfiles(void);
+static int rzfile(void);
+static int tryz(void);
+static int closeit(void);
+static int wcgetsec(char *rxbuf, int maxtime);
+static int putsec(char *buf, int n);
+static void usage(void);
+static int findfile(char *file, char *pa);
+static int make_dirs(char *pathname);
+static int wcrx(void);
+static int wcrxpn(char *rpn);
+static void checkpath(char *name);
+static void report(int sct);
+static void uncaps(char *s);
+static void canit(void);
+static int IsAnyLower(const char *s);
+static void chkinvok(char *s);
+static int wcreceive(int argc, char **argp);
+
+int procheader(char *name);
+void purgeline(void);
+
 RETSIGTYPE alrm(int dummy)
 {
 	/* doesn't need to do anything */
@@ -202,12 +201,11 @@ RETSIGTYPE bibi(n)
 	exit(128 + n);
 }
 
-main(argc, argv)
-char *argv[];
+int main(int argc, char *argv[])
 {
 	register char *cp;
-	register npats;
-	char *virgin, **patts;
+	int npats;
+	char *virgin, **patts = NULL;
 	char *getenv();
 	int exitcode = 0;
 	int under_rsh = FALSE;
@@ -250,10 +248,6 @@ char *argv[];
 				case 'c':
 					Crcflg = TRUE;
 					break;
-				case 'C':
-					allow_remote_commands = TRUE;
-					break;
-
 				case 'g':
 					if (--argc < 1) {
 						usage();
@@ -380,8 +374,6 @@ char *argv[];
 		usage();
 	if (Batch && npats)
 		usage();
-	if (Restricted && allow_remote_commands)
-		allow_remote_commands = FALSE;
 	if (Verbose) {
 #if 0
 		if (freopen(LOGFILE, "a", stderr) == NULL) {
@@ -420,10 +412,11 @@ char *argv[];
 	if (exitcode)
 		cucheck();
 	exit(exitcode ? exitcode : SS_NORMAL);
+
+	return 0; /* compilers would complain */
 }
 
-
-usage()
+static void usage(void)
 {
 	cucheck();
 	fprintf(stderr, "Usage: lrz [-abeuvy] [-L FILE] (ZMODEM)\n");
@@ -443,18 +436,6 @@ usage()
 		Progname, VERSION, CPU, OS);
 	exit(SS_NORMAL);
 }
-/*
- *  Debugging information output interface routine
- */
-/* VARARGS1 */
-vfile(f, a, b, c)
-register char *f;
-{
-	if (Verbose > 2) {
-		fprintf(stderr, f, a, b, c);
-		fprintf(stderr, "\n");
-	}
-}
 
 /*
  * Let's receive something already.
@@ -462,17 +443,16 @@ register char *f;
 
 char *rbmsg = "%s waiting to receive.";
 
-wcreceive(argc, argp)
-char **argp;
+static int wcreceive(int argc, char **argp)
 {
-	register c;
+	int c;
 
 	if (Batch || argc == 0) {
 		Crcflg = 1;
 		if (!Quiet)
 			fprintf(stderr, rbmsg, Progname,
 				Nozmodem ? "sb" : "sz");
-		if (c = tryz()) {
+		if ((c = tryz())) {
 			if (c == ZCOMPL)
 				return OK;
 			if (c == ERROR)
@@ -526,33 +506,28 @@ char **argp;
  * Length is indeterminate as long as less than Blklen
  * A null string represents no more files (YMODEM)
  */
-wcrxpn(rpn)
-char *rpn;			/* receive a pathname */
+static int wcrxpn(char *rpn)
 {
-	register c;
+	int c;
 
-#ifdef NFGVMIN
 	readline(1);
-#else
-	purgeline();
-#endif
 
       et_tu:
 	Firstsec = TRUE;
 	Eofseen = FALSE;
-	sendline(Crcflg ? WANTCRC : NAK);
+	xsendline(Crcflg ? WANTCRC : NAK);
 	Lleft = 0;		/* Do read next time ... */
 	while ((c = wcgetsec(rpn, 100)) != 0) {
 		if (c == WCEOT) {
 			zperr("Pathname fetch returned %d", c);
-			sendline(ACK);
+			xsendline(ACK);
 			Lleft = 0;	/* Do read next time ... */
 			readline(1);
 			goto et_tu;
 		}
 		return ERROR;
 	}
-	sendline(ACK);
+	xsendline(ACK);
 	return OK;
 }
 
@@ -561,11 +536,10 @@ char *rpn;			/* receive a pathname */
  * Jack M. Wierda and Roderick W. Hart
  */
 
-wcrx()
+static int wcrx(void)
 {
 	register int sectnum, sectcurr;
 	register char sendchar;
-	register char *p;
 	int cblklen;		/* bytes to dump this block */
 
 	Firstsec = TRUE;
@@ -574,11 +548,11 @@ wcrx()
 	sendchar = Crcflg ? WANTCRC : NAK;
 
 	for (;;) {
-		sendline(sendchar);	/* send it now, we're ready! */
+		xsendline(sendchar);	/* send it now, we're ready! */
 		Lleft = 0;	/* Do read next time ... */
 		sectcurr = wcgetsec(secbuf, (sectnum & 0177) ? 50 : 130);
 		report(sectcurr);
-		if (sectcurr == (sectnum + 1 & 0377)) {
+		if (sectcurr == ((sectnum + 1) & 0377)) {
 			sectnum++;
 			cblklen = Bytesleft > Blklen ? Blklen : Bytesleft;
 			if (putsec(secbuf, cblklen) == ERROR)
@@ -592,7 +566,7 @@ wcrx()
 		} else if (sectcurr == WCEOT) {
 			if (closeit())
 				return ERROR;
-			sendline(ACK);
+			xsendline(ACK);
 			Lleft = 0;	/* Do read next time ... */
 			return OK;
 		} else if (sectcurr == ERROR)
@@ -614,11 +588,9 @@ wcrx()
  *    (Caller must do that when he is good and ready to get next sector)
  */
 
-wcgetsec(rxbuf, maxtime)
-char *rxbuf;
-int maxtime;
+static int wcgetsec(char *rxbuf, int maxtime)
 {
-	register checksum, wcj, firstch;
+	int checksum, wcj, firstch;
 	register unsigned short oldcrc;
 	register char *p;
 	int sectcurr;
@@ -666,13 +638,8 @@ int maxtime;
 				zperr("Sector number garbled");
 		}
 		/* make sure eot really is eot and not just mixmash */
-#ifdef NFGVMIN
 		else if (firstch == EOT && readline(1) == TIMEOUT)
 			return WCEOT;
-#else
-		else if (firstch == EOT && Lleft == 0)
-			return WCEOT;
-#endif
 		else if (firstch == CAN) {
 			if (Lastrx == CAN) {
 				zperr("Sender CANcelled");
@@ -693,11 +660,11 @@ int maxtime;
 		Lastrx = 0;
 		while (readline(1) != TIMEOUT);
 		if (Firstsec) {
-			sendline(Crcflg ? WANTCRC : NAK);
+			xsendline(Crcflg ? WANTCRC : NAK);
 			Lleft = 0;	/* Do read next time ... */
 		} else {
 			maxtime = 40;
-			sendline(NAK);
+			xsendline(NAK);
 			Lleft = 0;	/* Do read next time ... */
 		}
 	}
@@ -713,13 +680,10 @@ int maxtime;
  *
  * timeout is in tenths of seconds
  */
-readline(timeout)
-int timeout;
+int readline(int timeout)
 {
-	register n;
-#ifndef READLINE_PF
-	static char *readline_ptr;	/* pointer for removing chars from linbuf */
-#endif
+	fd_set rset;
+	struct timeval tv;
 
 	if (--Lleft >= 0) {
 		if (Verbose > 8) {
@@ -728,25 +692,16 @@ int timeout;
 		return (*readline_ptr++ & 0377);
 	}
 
-	if (!no_timeout) {
-		n = timeout / 10;
-		if (n < 2)
-			n = 3;
-		if (Verbose > 5)
-			fprintf(stderr,
-				"Calling read: alarm=%d  Readnum=%d ", n,
-				Readnum);
-		signal(SIGALRM, alrm);
-		alarm(n);
-	} else if (Verbose > 5)
-		fprintf(stderr, "Calling read: Readnum=%d ", Readnum);
+	FD_ZERO(&rset);
+	FD_SET(iofd, &rset);
+	if (timeout < 20) 
+		timeout = 30;
+	tv.tv_sec = timeout / 10;
+	tv.tv_usec = (timeout % 10) * 100000;
+	if (select(iofd + 1, &rset, NULL, NULL, &tv) == 0)
+		return TIMEOUT;
 
 	Lleft = read(iofd, readline_ptr = linbuf, Readnum);
-	if (!no_timeout)
-		alarm(0);
-	if (Verbose > 5) {
-		fprintf(stderr, "Read returned %d bytes\n", Lleft);
-	}
 	if (Lleft < 1)
 		return TIMEOUT;
 	--Lleft;
@@ -761,24 +716,18 @@ int timeout;
 /*
  * Purge the modem input queue of all characters
  */
-purgeline()
+void purgeline(void)
 {
 	Lleft = 0;
-#ifdef TCFLSH
-	ioctl(iofd, TCFLSH, 0);
-#else
-	lseek(iofd, 0L, 2);
-#endif
+	tcflush(iofd, TCIFLUSH);
 }
 
 /*
  * Process incoming file information header
  */
-procheader(name)
-char *name;
+int procheader(char *name)
 {
-	register char *openmode, *p, **pp;
-	int tabs, tab_num;
+	register char *openmode, *p;
 
 	closeok = 0;
 
@@ -842,7 +791,7 @@ char *name;
 			d = 0;
 		if ((Verbose && d > 60) || Verbose > 1)
 			fprintf(stderr,
-				"TIMESYNC: here %ld, remote %ld, diff %d seconds\n",
+				"TIMESYNC: here %ld, remote %ld, diff %ld seconds\n",
 				(long) t, (long) Modtime, (long) d);
 #ifdef HAVE_SETTIMEOFDAY
 		if (timesync_flag > 1 && d > 10) {
@@ -908,7 +857,7 @@ char *name;
 
 			FILE *pathsfd;
 
-			if (pathsfd = fopen(tempsname, "r")) {
+			if ((pathsfd = fopen(tempsname, "r")) != NULL) {
 				while (fgets(puskuri, 300, pathsfd)) {
 					char *s;
 					int filefd;
@@ -978,8 +927,7 @@ char *name;
  * it's because some required directory was not present, and if
  * so, create all required dirs.
  */
-make_dirs(pathname)
-register char *pathname;
+static int make_dirs(char *pathname)
 {
 	register char *p;	/* Points into path */
 	int madeone = 0;	/* Did we do anything yet? */
@@ -1021,9 +969,7 @@ register char *pathname;
  *  If not in binary mode, carriage returns, and all characters
  *  starting with CPMEOF are discarded.
  */
-putsec(buf, n)
-char *buf;
-register n;
+static int putsec(char *buf, int n)
 {
 	register char *p;
 
@@ -1048,26 +994,8 @@ register n;
 	return OK;
 }
 
-/*
- *  Send a character to modem.  Small is beautiful.
- */
-sendline(c)
-{
-	char d;
-
-	d = c;
-	if (Verbose > 6)
-		fprintf(stderr, "Sendline: %x\n", c);
-	write(iofd, &d, 1);
-}
-
-flushmo()
-{
-}
-
 /* make string s lower case */
-uncaps(s)
-register char *s;
+static void uncaps(char *s)
 {
 	for (; *s; ++s)
 		if (isupper(*s))
@@ -1076,8 +1004,7 @@ register char *s;
 /*
  * IsAnyLower returns TRUE if string s has lower case letters.
  */
-IsAnyLower(s)
-register char *s;
+static int IsAnyLower(const char *s)
 {
 	for (; *s; ++s)
 		if (islower(*s))
@@ -1085,22 +1012,8 @@ register char *s;
 	return FALSE;
 }
 
-/*
- * Log an error
- */
-/*VARARGS1*/
-zperr(s, p, u)
-char *s, *p, *u;
-{
-	if (Verbose <= 0)
-		return;
-	fprintf(stderr, "Retry %d: ", errors);
-	fprintf(stderr, s, p, u);
-	fprintf(stderr, "\n");
-}
-
 /* send cancel string to get the other end to shut up */
-canit()
+static void canit(void)
 {
 	static char canistr[] = {
 		24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 8, 8, 8, 8, 8, 8,
@@ -1112,9 +1025,7 @@ canit()
 	fflush(stdout);
 }
 
-
-report(sct)
-int sct;
+static void report(int sct)
 {
 	if (Verbose > 1)
 		fprintf(stderr, "Blocks received: %d\r", sct);
@@ -1125,8 +1036,7 @@ int sct;
  * If called as [-][dir/../]rzCOMMAND set the pipe flag
  * If called as rb use YMODEM protocol
  */
-chkinvok(s)
-char *s;
+static void chkinvok(char *s)
 {
 	register char *p;
 
@@ -1155,11 +1065,7 @@ char *s;
 		Topipe = 1;
 }
 
-/*
- * Totalitarian Communist pathname processing
- */
-checkpath(name)
-char *name;
+static void checkpath(char *name)
 {
 	if (Restricted) {
 		/* restrict pathnames to current tree or uucppublic */
@@ -1191,10 +1097,10 @@ char *name;
  *  Return ZFILE if Zmodem filename received, -1 on error,
  *   ZCOMPL if transaction finished,  else 0
  */
-tryz()
+static int tryz(void)
 {
-	register c, n;
-	register cmdzack1flg;
+	int c, n;
+	int cmdzack1flg;
 
 	if (Nozmodem)		/* Check for "rb" program name */
 		return 0;
@@ -1207,11 +1113,7 @@ tryz()
 #else
 		stohdr(0L);
 #endif
-#ifdef CANBREAK
 		Txhdr[ZF0] = CANFC32 | CANFDX | CANOVIO | CANBRK;
-#else
-		Txhdr[ZF0] = CANFC32 | CANFDX | CANOVIO;
-#endif
 #ifdef ENABLE_TIMESYNC
 		if (timesync_flag)
 			Txhdr[ZF1] |= ZF1_TIMESYNC;
@@ -1262,29 +1164,12 @@ tryz()
 					fprintf(stderr, "lrz: %s\n",
 						secbuf);
 				}
-				if (!allow_remote_commands) {
-					if (Verbose)
-						fprintf(stderr,
-							"lrz: not executed\n");
-					zshhdr(ZCOMPL, Txhdr);
-					return ZCOMPL;
-				}
-				if (cmdzack1flg & ZCACK1)
-					stohdr(0L);
-				else
-					stohdr((long) sys2(secbuf));
-				purgeline();	/* dump impatient questions */
-				do {
-					zshhdr(ZCOMPL, Txhdr);
-				}
-				while (++errors < 20
-				       && zgethdr(Rxhdr, 1) != ZFIN);
-				ackbibi();
-				if (cmdzack1flg & ZCACK1)
-					exec2(secbuf);
+				if (Verbose)
+					fprintf(stderr,
+						"lrz: not executed\n");
+				zshhdr(ZCOMPL, Txhdr);
 				return ZCOMPL;
 			}
-			zshhdr(ZNAK, Txhdr);
 			goto again;
 		case ZCOMPL:
 			goto again;
@@ -1303,9 +1188,9 @@ tryz()
 /*
  * Receive 1 or more files with ZMODEM protocol
  */
-rzfiles()
+static int rzfiles(void)
 {
-	register c;
+	int c;
 
 	for (;;) {
 		switch (c = rzfile()) {
@@ -1332,9 +1217,9 @@ rzfiles()
  * Receive a file with ZMODEM protocol
  *  Assumes file name frame is in secbuf
  */
-rzfile()
+static int rzfile(void)
 {
-	register c, n;
+	int c, n;
 	long last_rxbytes = 0;
 	long not_printed = 0;
 
@@ -1400,9 +1285,8 @@ rzfile()
 					     last_bps) % 60;
 				}
 				fprintf(stderr,
-					"\rBytes Received: %7ld/%7ld   BPS:%-6ld                   \r\n",
-					rxbytes, Bytesleft, last_bps,
-					minleft, secleft);
+					"\rBytes Received: %7ld/%7ld   BPS:%-6d                   \r\n",
+					rxbytes, Bytesleft, last_bps);
 			}
 			closeok = 1;
 			if (closeit()) {
@@ -1460,7 +1344,7 @@ rzfile()
 					     last_bps) % 60;
 				}
 				fprintf(stderr,
-					"\rBytes Received: %7ld/%7ld   BPS:%-6ld ETA %02d:%02d  ",
+					"\rBytes Received: %7ld/%7ld   BPS:%-6d ETA %02d:%02d  ",
 					rxbytes, Bytesleft, last_bps,
 					minleft, secleft);
 				last_rxbytes = rxbytes;
@@ -1531,7 +1415,7 @@ rzfile()
 				rxbytes += Rxcount;
 				stohdr(rxbytes);
 				zshhdr(ZACK, Txhdr);
-				sendline(XON);
+				xsendline(XON);
 				goto nxthdr;
 			case GOTCRCQ:
 				n = 20;
@@ -1571,8 +1455,7 @@ rzfile()
  * Send a string to the modem, processing for \336 (sleep 1 sec)
  *   and \335 (break signal)
  */
-zmputs(s)
-char *s;
+static void zmputs(char *s)
 {
 	char *p;
 
@@ -1597,9 +1480,9 @@ char *s;
 /*
  * Close the receive dataset, return OK or ERROR
  */
-closeit()
+static int closeit(void)
 {
-	time_t time();
+	struct timeval timep[2];
 	char dszb[512];
 
 	if (Topipe) {
@@ -1617,22 +1500,24 @@ closeit()
 	if (closeok) {
 		if (dszlog != -1) {
 			sprintf(dszb,
-				"S %6ld %5ld bps %4d cps   0 errors     0 %4d %s %lu\n",
+				"S %6ld %5d bps %4d cps   0 errors     0 %4d %s %u\n",
 				rxbytes, 0, last_bps, 1024, myname, 0);
 			write(dszlog, dszb, strlen(dszb));
 		}
 	} else {
 		if (dszlog != -1) {
 			sprintf(dszb,
-				"E %6ld %5ld bps %4d cps   0 errors     0 %4d %s %lu\n",
+				"E %6ld %5d bps %4d cps   0 errors     0 %4d %s %u\n",
 				rxbytes, 0, last_bps, 1024, myname, 0);
 			write(dszlog, dszb, strlen(dszb));
 		}
 	}
 	if (Modtime) {
-		timep[0] = time(NULL);
-		timep[1] = Modtime;
-		utime(Pathname, timep);
+		timep[0].tv_sec = time(NULL);
+		timep[1].tv_usec = 0;
+		timep[1].tv_sec = Modtime;
+		timep[1].tv_usec = 0;
+		utimes(Pathname, timep);
 	}
 	if ((Filemode & S_IFMT) == S_IFREG)
 		chmod(Pathname, (07777 & Filemode));
@@ -1642,9 +1527,9 @@ closeit()
 /*
  * Ack a ZFIN packet, let byegones be byegones
  */
-ackbibi()
+static void ackbibi(void)
 {
-	register n;
+	register int n;
 
 	vfile("ackbibi:");
 	Readnum = 1;
@@ -1666,40 +1551,7 @@ ackbibi()
 	}
 }
 
-
-
-/*
- * Local console output simulation
- */
-bttyout(c)
-{
-	if (Verbose || Fromcu)
-		putc(c, stderr);
-}
-
-/*
- * Strip leading ! if present, do shell escape. 
- */
-sys2(s)
-register char *s;
-{
-	if (*s == '!')
-		++s;
-	return system(s);
-}
-/*
- * Strip leading ! if present, do exec.
- */
-exec2(s)
-register char *s;
-{
-	if (*s == '!')
-		++s;
-	mode(0);
-	execl("/bin/sh", "sh", "-c", s);
-}
-
-int findfile(char *file, char *pa)
+static int findfile(char *file, char *pa)
 {
 	char buf1[1024];
 	char buf2[1024];
