@@ -8,6 +8,11 @@
 #include "config.h"
 #include "../main/dd.h"
 
+#include <zmodem.h>
+#include <zm.h>
+#include <rbsb.h>
+#include <crctab.h>
+
 #if STDC_HEADERS
 # include <string.h>
 #else
@@ -27,17 +32,24 @@ char *strchr (), *strrchr ();
 
 #define NEW_ERROR
 
+#include <sys/ioctl.h>
+
 char *getenv();
 
 #define SS_NORMAL 0
 #define LOGFILE "/var/log/szlog"
 #include <unistd.h>
+#include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
 #include <setjmp.h>
 #include <ctype.h>
 #include <errno.h>
+#include <fcntl.h>
+#include <unistd.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #ifdef HAVE_LIMITS_H
 #  include <limits.h>
 #else 
@@ -60,6 +72,8 @@ char *getenv();
 #  endif
 #endif
 
+#include <termios.h>
+
 #if defined(HAVE_SYS_MMAN_H) && defined(HAVE_MMAP)
 #  include <sys/mman.h>
 size_t mm_size;
@@ -67,32 +81,12 @@ void *mm_addr=NULL;
 #else
 #  undef HAVE_MMAP
 #endif
-#include "timing.h"
 extern int errno;
-/* #define sendline(c) putchar(c & 0377) */
-/* #define xsendline(c) putchar(c) */
-#define sendline(c) xsendline(c & 0377);
 
 #define PATHLEN 256
-#define OK 0
-#define FALSE 0
-#define TRUE 1
-#define ERROR (-1)
+
 /* Ward Christensen / CP/M parameters - Don't change these! */
-#define ENQ 005
-#define CAN ('X'&037)
-#define XOFF ('s'&037)
-#define XON ('q'&037)
-#define SOH 1
-#define STX 2
-#define EOT 4
-#define ACK 6
-#define NAK 025
-#define CPMEOF 032
-#define WANTCRC 0103	/* send C not NAK to get crc not checksum */
 #define WANTG 0107	/* Send G not NAK to get nonstop batch xmsn */
-#define TIMEOUT (-2)
-#define RCDO (-3)
 #define RETRYMAX 10
 
 
@@ -101,19 +95,12 @@ int nodeinf;
 struct DayDream_NodeInfo nin;
 
 int Zmodem=0;		/* ZMODEM protocol requested by receiver */
-unsigned Baudrate=2400;	/* Default, should be set by first mode() call */
 unsigned Txwindow;	/* Control the size of the transmitted window */
 unsigned Txwspac;	/* Spacing between zcrcq requests */
 unsigned Txwcnt;	/* Counter used to space ack requests */
 long Lrxpos;		/* Receiver's last reported offset */
-int errors;
-
 int Canseek=1; /* 1: can; 0: only rewind, -1: neither */
 
-#include "rbsb.c"	/* most of the system dependent stuff here */
-
-#include "crctab.c"
-
 static int wctx(off_t flen);
 
 int Filesleft;
@@ -125,11 +112,7 @@ time_t lup;
  *  when an error is detected.  A pause (0336) may be needed before the
  *  ^C (03) or after it.
  */
-#ifdef READCHECK
 char Myattn[] = { 0 };
-#else
-char Myattn[] = { 03, 0336, 0 };
-#endif
 
 FILE *in;
 
@@ -197,20 +180,37 @@ char listname[256];
 #define MK_STRING(x) MK_STRING2(x)
 #define MK_STRING2(x) #x
 
+static int zsendcmd(char *buf, int blen);
+static int getnak(void);
+static int wcsend(const char *list);
+static int wcs(char *oname);
+static int wctxpn(char *name);;
+static int filbuf(char *buf, int count);
+static int wcputsec(char *buf, int sectnum, int cseclen);
+static int zsendfdata(void);
+static int getzrxinit(void);
+static int zsendfile(char *buf, int blen);
+static int calc_blklen(long total_sent);
+static int sendzsinit(void);
+static void usage(void);
+static void saybibi(void);
+static int getinsync(int flag);
+static void countem(char *list);
+static void chkinvok(char *s);
+
+
+int readline(int);
+void purgeline(void);
+void zperr(const char *fmt, ...);
+void canit(void);
 
 jmp_buf intrjmp;	/* For the interrupt on RX CAN */
 
-void xsendline(char c)
-{
-	write(iofd,&c,1);
-	fflush(stdout);
-}
-
 /* called by signal interrupt or terminate to clean things up */
 RETSIGTYPE
 bibi(int n)
 {
-	canit(); fflush(stdout); mode(0);
+	canit(); mode(0);
 	fprintf(stderr, "sz: caught signal %d; exiting\n", n);
 	fflush(stderr);
 	
@@ -229,20 +229,15 @@ onintr()
 	longjmp(intrjmp, -1);
 }
 
-int Zctlesc;	/* Encode control characters */
 int Nozmodem = 0;	/* If invoked as "sb" */
 char *Progname = "sz";
 int Zrwindow = 1400;	/* RX window size (controls garbage count) */
 
-#define READLINE_PF(x) readline(x)
-#include "zm.c"
-
-
 int 
 main(int argc, char **argv)
 {
 	register char *cp;
-	register npats;
+	int npats;
 	int dm;
 	int under_rsh=FALSE;
 
@@ -275,10 +270,8 @@ main(int argc, char **argv)
 					 *cp = toupper(*cp);  continue;
 				case '+':
 					Lzmanag = ZMAPND; break;
-#ifdef CSTOPB
 				case '2':
 					Twostop = TRUE; break;
-#endif
 				case '8':
 					if (max_blklen==8192)
 						start_blklen=8192;
@@ -325,8 +318,6 @@ main(int argc, char **argv)
 					}
 					
 					iofd=open(*++argv,O_RDWR);
-//					dup2(iofd,0);
-//					dup2(iofd,1);
 					break;
 				case 'h':
 					usage(); break;
@@ -381,19 +372,13 @@ main(int argc, char **argv)
 					if (Rxtimeout<10 || Rxtimeout>1000)
 						usage();
 					break;
-				case 'T':
-					if (++Test > 1) {
-						chartest(1); chartest(2);
-						mode(0);  exit(0);
-					}
-					break;
 				case 'u':
 					++Unlinkafter; break;
 				case 'U':
 					if (!under_rsh)
 						Restricted=0;
 					else {
-fprintf(stderr,"security violation: running under restricted shell\n",stderr);
+fprintf(stderr,"security violation: running under restricted shell\n");
 						exit(1);
 					}
 					break;
@@ -513,8 +498,7 @@ fprintf(stderr,"security violation: running under restricted shell\n",stderr);
 	if (Command) {
 		if (getzrxinit()) {
 			Exitcode=0200; canit();
-		}
-		else if (zsendcmd(Cmdstr, 1+strlen(Cmdstr))) {
+		} else if (zsendcmd(Cmdstr, 1+strlen(Cmdstr))) {
 			Exitcode=0200; canit();
 		}
 	} else if (wcsend(listname)==ERROR) {
@@ -538,10 +522,8 @@ char *fgetsm(char *a1, int a2, FILE *a3)
 	return a1;
 }
 
-wcsend(list)
-char *list;
+static int wcsend(const char *list)
 {
-	register n;
 	char munb[500];
 	FILE *listh;
 	
@@ -596,11 +578,10 @@ char *list;
 				fprintf(stderr, " (tz unknown)\r\n");
 #endif
 			fclose(f);
-			if (wcs(tmp)==ERROR)
+			if (wcs(tmp)==ERROR) {
 				if (Verbose)
 					fprintf(stderr, "\r\nTIMESYNC: failed\n");
-			else
-			{
+			} else {
 				if (Verbose)
 					fprintf(stderr, "\r\nTIMESYNC: ok\n");
 				Filcnt--;
@@ -644,10 +625,9 @@ char *list;
 	return OK;
 }
 
-wcs(oname)
-char *oname;
+static int wcs(char *oname)
 {
-	register c;
+	int c;
 	register char *p;
 	struct stat f;
 	char name[PATHLEN];
@@ -725,8 +705,7 @@ char *oname;
  *  as provided by the Unix fstat call.
  *  N.B.: modifies the passed name, may extend it!
  */
-wctxpn(name)
-char *name;
+static int wctxpn(char *name)
 {
 	register char *p, *q;
 	char *s;
@@ -802,9 +781,9 @@ char *name;
 	return OK;
 }
 
-getnak()
+static int getnak(void)
 {
-	register firstch;
+	int firstch;
 
 	Lastrx = 0;
 	for (;;) {
@@ -818,9 +797,7 @@ getnak()
 			zperr("Timeout on pathname");
 			return TRUE;
 		case WANTG:
-#ifdef MODE2OK
 			mode(2);	/* Set cbreak, XON/XOFF, etc. */
-#endif
 			Optiong = TRUE;
 			blklen=1024;
 		case WANTCRC:
@@ -873,11 +850,9 @@ wctx(off_t flen)
 	attempts=0;
 	do {
 		purgeline();
-		sendline(EOT);
-		fflush(stdout);
+		xsendline(EOT);
 		++attempts;
-	}
-		while ((firstch=(readline(Rxtimeout)) != ACK) && attempts < RETRYMAX);
+	} while ((firstch=(readline(Rxtimeout)) != ACK) && attempts < RETRYMAX);
 	if (attempts == RETRYMAX) {
 		zperr("No ACK on EOT");
 		return ERROR;
@@ -886,12 +861,9 @@ wctx(off_t flen)
 		return OK;
 }
 
-wcputsec(buf, sectnum, cseclen)
-char *buf;
-int sectnum;
-int cseclen;	/* data length of this sector to send */
+static int wcputsec(char *buf, int sectnum, int cseclen)
 {
-	register checksum, wcj;
+	int checksum, wcj;
 	register char *cp;
 	unsigned oldcrc;
 	int firstch;
@@ -903,22 +875,22 @@ int cseclen;	/* data length of this sector to send */
 		fprintf(stderr, "\rYmodem sectors/kbytes sent: %3d/%2dk", Totsecs, Totsecs/8 );
 	for (attempts=0; attempts <= RETRYMAX; attempts++) {
 		Lastrx= firstch;
-		sendline(cseclen==1024?STX:SOH);
-		sendline(sectnum);
-		sendline(-sectnum -1);
+		xsendline(cseclen==1024?STX:SOH);
+		xsendline(sectnum);
+		xsendline(-sectnum -1);
 		oldcrc=checksum=0;
 		for (wcj=cseclen,cp=buf; --wcj>=0; ) {
-			sendline(*cp);
+			xsendline(*cp);
 			oldcrc=updcrc((0377& *cp), oldcrc);
 			checksum += *cp++;
 		}
 		if (Crcflg) {
 			oldcrc=updcrc(0,updcrc(0,oldcrc));
-			sendline((int)oldcrc>>8);
-			sendline((int)oldcrc);
+			xsendline((int)oldcrc>>8);
+			xsendline((int)oldcrc);
 		}
 		else
-			sendline(checksum);
+			xsendline(checksum);
 
 		if (Optiong) {
 			firstsec = FALSE; return OK;
@@ -963,10 +935,9 @@ cancan:
 }
 
 /* fill buf with count chars padding with ^Z for CPM */
-filbuf(buf, count)
-register char *buf;
+static int filbuf(char *buf, int count)
 {
-	register c, m;
+	int c, m;
 
 	if ( !Ascii) {
 		m = read(fileno(in), buf, count);
@@ -1000,7 +971,7 @@ register char *buf;
 }
 
 /* Fill buffer with blklen chars */
-zfilbuf()
+int zfilbuf(void)
 {
 	int n;
 
@@ -1010,18 +981,6 @@ zfilbuf()
 	return n;
 }
 
-/* VARARGS1 */
-vfile(f, a, b, c)
-register char *f;
-{
-	if (Verbose > 2) {
-		fprintf(stderr, f, a, b, c);
-		fprintf(stderr, "\n");
-		fflush(stderr);
-	}
-}
-
-
 RETSIGTYPE
 alrm(int dummy)
 {
@@ -1034,7 +993,7 @@ alrm(int dummy)
  * readline(timeout) reads character(s) from file descriptor 0
  * timeout is in tenths of seconds
  */
-readline(timeout)
+int readline(int timeout)
 {
 	register int c;
 	static char buf[64];
@@ -1057,7 +1016,6 @@ readline(timeout)
 		return c;
 	}
 
-	fflush(stdout);
 	if (!no_timeout) {
 		c = timeout/10;
 		if (c<2)
@@ -1081,24 +1039,14 @@ readline(timeout)
 	return (buf[0]&0377);
 }
 
-flushmo()
-{
-	fflush(stdout);
-}
-
-
-purgeline()
+void purgeline(void)
 {
 	readline(-1);
-#ifdef TCFLSH
-	ioctl(iofd, TCFLSH, 0);
-#else
-	lseek(iofd, 0L, 2);
-#endif
+	tcflush(iofd, TCIFLUSH);
 }
 
 /* send cancel string to get the other end to shut up */
-canit()
+void canit(void)
 {
 	static char canistr[] = {
 	 24,24,24,24,24,24,24,24,24,24,8,8,8,8,8,8,8,8,8,8,0
@@ -1108,24 +1056,6 @@ canit()
 	fflush(stdout);
 }
 
-
-/*
- * Log an error
- */
-/*VARARGS1*/
-zperr(s,p,u)
-char *s, *p, *u;
-{
-	if (Verbose <= 0)
-		return;
-	fprintf(stderr, "Retry %d: ", errors);
-	fprintf(stderr, s, p, u);
-	fprintf(stderr, "\n");
-	fflush(stderr);
-
-}
-
-
 const char *babble[] = {
 	"Send file(s) with ZMODEM/YMODEM/XMODEM Protocol",
 	"	(Y) = Option applies to YMODEM only",
@@ -1134,9 +1064,7 @@ const char *babble[] = {
 	"	lsz [-2Ceqv] -c COMMAND",
 	"	lsb [-2adfkquv] [-] file ...",
 	"	lsx [-2akquv] [-] file",
-#ifdef CSTOPB
 	"	2 Use 2 stop bits",
-#endif
 	"	+ Append to existing destination file (Z)",
 	"	a (ASCII) change NL to CR/LF",
 	"	b Binary file transfer override",
@@ -1164,7 +1092,7 @@ const char *babble[] = {
 	""
 };
 
-usage()
+static void usage(void)
 {
 	const char **pp;
 
@@ -1179,9 +1107,9 @@ usage()
 /*
  * Get the receiver's init parameters
  */
-getzrxinit()
+static int getzrxinit(void)
 {
-	register n;
+	int n;
 	struct stat f;
 
 	for (n=10; --n>=0; ) {
@@ -1206,16 +1134,7 @@ getzrxinit()
 			vfile("Rxbuflen=%d Tframlen=%d", Rxbuflen, Tframlen);
 			if ( !Fromcu)
 				signal(SIGINT, SIG_IGN);
-#ifdef MODE2OK
 			mode(2);	/* Set cbreak, XON/XOFF, etc. */
-#endif
-#ifndef READCHECK
-			/* Use MAX_BLOCK byte frames if no sample/interrupt */
-			if (Rxbuflen < 32 || Rxbuflen > MAX_BLOCK) {
-				Rxbuflen = MAX_BLOCK;
-				vfile("Rxbuflen=%d", Rxbuflen);
-			}
-#endif
 			/* Override to force shorter frame length */
 			if (Rxbuflen && (Rxbuflen>Tframlen) && (Tframlen>=32))
 				Rxbuflen = Tframlen;
@@ -1271,9 +1190,9 @@ getzrxinit()
 }
 
 /* Send send-init information */
-sendzsinit()
+static int sendzsinit(void)
 {
-	register c;
+	int c;
 
 	if (Myattn[0] == '\0' && (!Zctlesc || (Rxflags & TESCCTL)))
 		return OK;
@@ -1301,10 +1220,9 @@ sendzsinit()
 }
 
 /* Send file name and related info */
-zsendfile(buf, blen)
-char *buf;
+static int zsendfile(char *buf, int blen)
 {
-	register c;
+	int c;
 	register unsigned long crc;
 
 	for (;;) {
@@ -1376,10 +1294,11 @@ again:
 }
 
 /* Send the data in the file */
-zsendfdata()
+static int zsendfdata(void)
 {
-	register c, e, n;
-	register newcnt;
+	int e, n;
+	int c = 0;
+	int newcnt;
 	register long tcount = 0;
 	int junkcount;		/* Counts garbage chars received by TX */
 	static int tleft = 6;	/* Counter for test mode */
@@ -1431,7 +1350,6 @@ gotack:
 		case ZRINIT:
 			return OK;
 		}
-#ifdef READCHECK
 		/*
 		 * If the reverse channel can be tested for data,
 		 *  this logic may be used to detect error packets
@@ -1439,12 +1357,7 @@ gotack:
 		 *  rdchk(fdes) returns non 0 if a character is available
 		 */
 		while (rdchk(iofd)) {
-#ifdef READCHECK_READS
-			switch (checked)
-#else
-			switch (readline(1))
-#endif
-			{
+			switch (readline(1)) {
 			case CAN:
 			case ZPAD:
 				c = getinsync(1);
@@ -1456,7 +1369,6 @@ gotack:
 				readline(100);
 			}
 		}
-#endif
 	}
 
 #ifndef linux
@@ -1479,19 +1391,11 @@ gotack:
 			while (tcount < 20000) {
 				printf(qbf); fflush(stdout);
 				tcount += strlen(qbf);
-#ifdef READCHECK
 				while (rdchk(iofd)) {
-#ifdef READCHECK_READS
-					switch (checked)
-#else
-					switch (readline(1))
-#endif
-					{
+					switch (readline(1)) {
 					case CAN:
 					case ZPAD:
-#ifdef TCFLSH
-						ioctl(iofd, TCFLSH, 1);
-#endif
+						tcflush(iofd, TCOFLUSH);
 						goto waitack;
 					case XOFF:	/* Wait for XON */
 #ifndef linux
@@ -1500,7 +1404,6 @@ gotack:
 						readline(100);
 					}
 				}
-#endif
 			}
 		signal(SIGINT, SIG_IGN); canit();
 		sleep(3); purgeline(); mode(0);
@@ -1518,7 +1421,7 @@ gotack:
 		blklen=calc_blklen(total_sent);
 		total_sent+=blklen+OVERHEAD;
 		if (Verbose >2 && blklen!=old)
-			fprintf(stderr,"blklen now %ld\n",blklen);
+			fprintf(stderr,"blklen now %d\n",blklen);
 #endif
 #ifdef HAVE_MMAP
 		if (mm_addr) {
@@ -1553,7 +1456,7 @@ gotack:
 				minleft =  (Filesize-Txpos)/last_bps/60;
 				secleft =  ((Filesize-Txpos)/last_bps)%60;
 			}
-			fprintf(stderr, "\rBytes Sent:%7ld/%7ld   BPS:%-6d ETA %02d:%02d  ",
+			fprintf(stderr, "\rBytes Sent:%7ld/%7ld   BPS:%-6ld ETA %02d:%02d  ",
 			 Txpos, Filesize, last_bps, minleft, secleft);
 			fflush(stderr);
 			if ( (nodeinf!=-1) && ((time(0)-lup) > 5)) {  
@@ -1576,29 +1479,20 @@ gotack:
 		bytcnt = Txpos += n;
 		if (e == ZCRCW)
 			goto waitack;
-#ifdef READCHECK
 		/*
 		 * If the reverse channel can be tested for data,
 		 *  this logic may be used to detect error packets
 		 *  sent by the receiver, in place of setjmp/longjmp
 		 *  rdchk(fdes) returns non 0 if a character is available
 		 */
-		fflush(stdout);
 		while (rdchk(iofd)) {
-#ifdef READCHECK_READS
-			switch (checked)
-#else
-			switch (readline(1))
-#endif
-			{
+			switch (readline(1)) {
 			case CAN:
 			case ZPAD:
 				c = getinsync(1);
 				if (c == ZACK)
 					break;
-#ifdef TCFLSH
-				ioctl(iofd, TCFLSH, 1);
-#endif
+				tcflush(iofd, TCOFLUSH);
 				/* zcrce - dinna wanna starta ping-pong game */
 				zsdata(txbuf, 0, ZCRCE);
 				goto gotack;
@@ -1611,7 +1505,6 @@ gotack:
 				++junkcount;
 			}
 		}
-#endif	/* READCHECK */
 		if (Txwindow) {
 			while ((tcount = Txpos - Lrxpos) >= Txwindow) {
 				vfile("%ld window >= %u", tcount, Txwindow);
@@ -1619,9 +1512,7 @@ gotack:
 					zsdata(txbuf, 0, e = ZCRCQ);
 				c = getinsync(1);
 				if (c != ZACK) {
-#ifdef TCFLSH
-					ioctl(iofd, TCFLSH, 1);
-#endif
+					tcflush(iofd, TCOFLUSH);
 					zsdata(txbuf, 0, ZCRCE);
 					goto gotack;
 				}
@@ -1630,13 +1521,13 @@ gotack:
 		}
 	} while (!Eofseen);
 	if (Verbose > 1)
-		fprintf(stderr, "\rBytes Sent:%7ld   BPS:%-6d                       \n",
+		fprintf(stderr, "\rBytes Sent:%7ld   BPS:%-6ld                       \n",
 		Filesize,last_bps);
 		fflush(stderr);
 
 		if (dszlog!=-1) {
 			char kbu[512];
-			sprintf(kbu,"s %6ld %5ld bps %4d cps   0 errors     0 %4d %s %lu\n",Filesize,0,last_bps,1024,myname,0);
+			sprintf(kbu,"s %6ld %5d bps %4ld cps   0 errors     0 %4d %s %u\n",Filesize,0,last_bps,1024,myname,0);
 			write(dszlog,kbu,strlen(kbu));
 		}
 	if ( !Fromcu)
@@ -1665,8 +1556,7 @@ gotack:
 }
 
 #ifdef NEW_ERROR
-int
-calc_blklen(long total_sent)
+static int calc_blklen(long total_sent)
 {
 	static long total_bytes=0;
 	static int calcs_done=0;
@@ -1784,9 +1674,9 @@ calcit:
 /*
  * Respond to receiver's complaint, get back in sync with receiver
  */
-getinsync(flag)
+static int getinsync(int flag)
 {
-	register c;
+	int c;
 
 	for (;;) {
 		if (Test) {
@@ -1853,14 +1743,14 @@ getinsync(flag)
 
 
 /* Say "bibi" to the receiver, try to do it cleanly */
-saybibi()
+static void saybibi(void)
 {
 	for (;;) {
 		stohdr(0L);		/* CAF Was zsbhdr - minor change */
 		zshhdr(ZFIN, Txhdr);	/*  to make debugging easier */
 		switch (zgethdr(Rxhdr, 0)) {
 		case ZFIN:
-			sendline('O'); sendline('O'); flushmo();
+			xsendline('O'); xsendline('O'); 
 		case ZCAN:
 		case TIMEOUT:
 			return;
@@ -1868,18 +1758,10 @@ saybibi()
 	}
 }
 
-/* Local screen character display function */
-bttyout(c)
-{
-	if (Verbose)
-		putc(c, stderr);
-}
-
 /* Send command and related info */
-zsendcmd(buf, blen)
-char *buf;
+static int zsendcmd(char *buf, int blen)
 {
-	register c;
+	int c;
 	long cmdnum;
 
 	cmdnum = getpid();
@@ -1927,8 +1809,7 @@ listen:
 /*
  * If called as lsb use YMODEM protocol
  */
-chkinvok(s)
-char *s;
+static void chkinvok(char *s)
 {
 	register char *p;
 
@@ -1952,10 +1833,9 @@ char *s;
 	}
 }
 
-countem(list)
-register char *list;
+static void countem(char *list)
 {
-	register c;
+	int c;
 	struct stat f;
 	FILE *listh;
 	char munb[300];
@@ -1986,40 +1866,6 @@ register char *list;
 #endif
 }
 
-chartest(m)
-{
-	register n;
-
-	mode(m);
-	printf("\r\n\nCharacter Transparency Test Mode %d\r\n", m);
-	printf("If Pro-YAM/ZCOMM is not displaying ^M hit ALT-V NOW.\r\n");
-	printf("Hit Enter.\021");  fflush(stdout);
-	readline(500);
-
-	for (n = 0; n < 256; ++n) {
-		if (!(n%8))
-			printf("\r\n");
-		printf("%02x ", n);  fflush(stdout);
-		sendline(n);	flushmo();
-		printf("  ");  fflush(stdout);
-		if (n == 127) {
-			printf("Hit Enter.\021");  fflush(stdout);
-			readline(500);
-			printf("\r\n");  fflush(stdout);
-		}
-	}
-	printf("\021\r\nEnter Characters, echo is in hex.\r\n");
-	printf("Hit SPACE or pause 40 seconds for exit.\r\n");
-
-	while (n != TIMEOUT && n != ' ') {
-		n = readline(400);
-		printf("%02x\r\n", n);
-		fflush(stdout);
-	}
-	printf("\r\nMode %d character transparency test ends.\r\n", m);
-	fflush(stdout);
-}
-
 /* End of lsz.c */
 
 
