@@ -22,49 +22,44 @@ struct Fakerdata {
 	char FD_Informer[26];
 };
 
-char *ExamineCfg(char *, char *);
+static char *ExamineCfg(char *, char *);
 
-void strupr(char *);
-void clearflist(void);
-void closeMe(void);
-void CpyToEOL(char *, char *);
-void CpyToLAINA(char *, char *);
-int ProcessLine(char *);
-void ProcessFile(void);
-void FmtDirName(char *dirikka);
-char *Fnd(char *, char *);
-int FindFile(char *);
-char *GetFe(char *);
-void GenComline(const char *source, char *dest);
-void SaveDir(char *dirikka);
-void Delete(int);
-void FreeFile(void);
-void NukeFile(void);
-void copytonuke(char *);
-void CommentFile(void);
-void ReturnCreds(void);
-void Reward(void);
-int fakelist();
-int LookForFile(char *);
+static void strupr(char *);
+static void closeMe(void);
+static void CpyToEOL(char *, char *);
+static void CpyToLAINA(char *, char *);
+static int ProcessLine(char *);
+static void ProcessFile(void);
+static int FindFile(char *);
+static char *GetFe(char *);
+static void GenComline(const char *source, char *dest);
+static void SaveDir(char *dirikka);
+static void Delete(int);
+static void FreeFile(void);
+static void NukeFile(void);
+static void copytonuke(char *);
+static void CommentFile(void);
+static void ReturnCreds(void);
+static void Reward(void);
+static int fakelist(void);
+static int LookForFile(char *);
 
-char username[40];
-char tbuf[200];
-char ubname[200];
-char ullog[200];
-char dllog[200];
+static char username[40];
+static char tbuf[200];
+static char ubname[200];
+static char ullog[200];
+static char dllog[200];
 
-int userlen;
-int node;
+static int userlen;
+static int node;
 
-int quitme = 0;
-char *fptr;
-char *feptr;
-char *DirMem = 0;
-int DirSize;
-char *CfgMem = 0;
-struct Library *DDBase;
-struct dif *d;
-int ConfigSize = 0;
+static int quitme = 0;
+static char *fptr;
+static char *feptr;
+static char *DirMem = 0;
+static int DirSize;
+static char *CfgMem = 0;
+static struct dif *d;
 
 struct StringMem {
 	char descbuffer[10000];
@@ -77,28 +72,22 @@ struct StringMem {
 	struct DD_UploadLog logbuffer[401];
 };
 
-struct FileInfoBlock *FBlock = NULL;
 
-char BUFFER[2048];
-char BUFFER2[2048];
-uint16_t SysOpAccess;
-uint16_t RewardPercentage;
-char CurrFile[40];
-char CurrPath[260];
-uint32_t CurrSize;
-char rewarded[30];
+static char BUFFER[2048];
+static char BUFFER2[2048];
+static uint16_t SysOpAccess;
+static uint16_t RewardPercentage;
+static char CurrFile[40];
+static char CurrPath[260];
+static uint32_t CurrSize;
+static char rewarded[30];
 
-struct DayDream_Conference *thisconf;
+static struct DayDream_Conference *thisconf;
 
-char origdir[130];
-int updatedir;
-char namecol[20];
-char flagcol[20];
-char sizecol[20];
-char datecol[20];
-char desccol[20];
+static char origdir[130];
+static int updatedir;
 
-struct StringMem *sta;
+static struct StringMem *sta;
 
 int main(int argc, char *argv[])
 {
@@ -281,7 +270,7 @@ int main(int argc, char *argv[])
 	return 0;
 }
 
-int fakelist()
+static int fakelist(void)
 {
 	char buf[1024];
 	int fd;
@@ -293,7 +282,7 @@ int fakelist()
 	dd_sendstring(d, "[36mAutoprompt flagged files? [35m(Yes/no) [0m: ");
 
 	if (dd_hotkey(d, HOT_YESNO) == 1) {
-		sprintf(buf, "%sfakerlist.%d", DDTMP, node);
+		sprintf(buf, "%s/fakerlist.%d", DDTMP, node);
 		if (dd_dumpfilestofile(d, buf)) {
 			fd = open(buf, O_RDONLY);
 			if (fd < 0)
@@ -309,7 +298,8 @@ int fakelist()
 	}
 	return 1;
 }
-int lookdata(char *file)
+
+static int lookdata(char *file)
 {
 	int datahandle;
 	int pos = 0;
@@ -329,7 +319,7 @@ int lookdata(char *file)
 	return -1;
 }
 
-void adddata(int confn, char *file, char *info)
+static void adddata(int confn, char *file, char *info)
 {
 	int datahandle;
 	struct Fakerdata datamem;
@@ -348,8 +338,7 @@ void adddata(int confn, char *file, char *info)
 	}
 }
 
-
-int ProcessLine(char *line)
+static int ProcessLine(char *line)
 {
 	char *token;
 
@@ -358,7 +347,7 @@ int ProcessLine(char *line)
 
 		CurrFile[0] = 0;
 		if (LookForFile(token)) {
-//                      ProcessFile();
+/*                      ProcessFile();*/
 		} else {
 			dd_sendstring(d, "\n[35mFile not found!\n\n");
 		}
@@ -369,7 +358,7 @@ int ProcessLine(char *line)
 	return 0;
 }
 
-char *fgetsnolf(char *buf, int n, FILE * fh)
+static char *fgetsnolf(char *buf, int n, FILE * fh)
 {
 	char *hih;
 	char *s;
@@ -388,7 +377,8 @@ char *fgetsnolf(char *buf, int n, FILE * fh)
 	return hih;
 }
 
-void _splitpath(char *file, char *drive, char *path, char *name, char *ext)
+static void _splitpath(char *file, char *drive, char *path, 
+	char *name, char *ext)
 {
 	char *s, *t;
 	if (ext)
@@ -418,7 +408,7 @@ void _splitpath(char *file, char *drive, char *path, char *name, char *ext)
 	strcpy(name, t);
 }
 
-int LookForFile(char *fname)
+static int LookForFile(char *fname)
 {
 	char buf1[512];
 	char buf2[512];
@@ -426,7 +416,7 @@ int LookForFile(char *fname)
 	struct stat st;
 	int fi = 0;
 
-	sprintf(buf1, "%sfaker%d", DDTMP, node);
+	sprintf(buf1, "%s/faker%d", DDTMP, node);
 	if (dd_findfilestolist(d, fname, buf1)) {
 		plist = fopen(buf1, "r");
 		if (plist) {
@@ -445,7 +435,7 @@ int LookForFile(char *fname)
 	return fi;
 }
 
-void ProcessFile(void)
+static void ProcessFile(void)
 {
 	int currdir;
 	int dirsleft;
@@ -459,7 +449,7 @@ void ProcessFile(void)
 
 	dirsleft = thisconf->CONF_FILEAREAS;
 	currdir = dirsleft;
-	sprintf(BUFFER, "%sdata/directory.%3.3d", thisconf->CONF_PATH, thisconf->CONF_UPLOADAREA);
+	sprintf(BUFFER, "%s/data/directory.%3.3d", thisconf->CONF_PATH, thisconf->CONF_UPLOADAREA);
 
 	if (FindFile(BUFFER))
 		return;
@@ -469,7 +459,7 @@ void ProcessFile(void)
 			currdir--;
 			dirsleft--;
 		} else {
-			sprintf(BUFFER, "%sdata/directory.%3.3d", thisconf->CONF_PATH, currdir);
+			sprintf(BUFFER, "%s/data/directory.%3.3d", thisconf->CONF_PATH, currdir);
 			if (FindFile(BUFFER))
 				return;
 			currdir--;
@@ -478,7 +468,7 @@ void ProcessFile(void)
 	}
 }
 
-int FindFile(char *dirikka)
+static int FindFile(char *dirikka)
 {
 	int dirhandle;
 	char fndbuf[40];
@@ -503,7 +493,7 @@ int FindFile(char *dirikka)
 /*      strupr(CurrFile); */
 
 	sprintf(fndbuf, "\n%s ", CurrFile);
-	if ((fptr = Fnd(fndbuf, DirMem))) {
+	if ((fptr = strstr(DirMem, fndbuf))) {
 		fptr++;
 		feptr = GetFe(fptr);
 
@@ -593,7 +583,7 @@ int FindFile(char *dirikka)
 	}
 }
 
-void Reward(void)
+static void Reward(void)
 {
 	int NukeHandle;
 	int offset;
@@ -668,7 +658,7 @@ void Reward(void)
 	}
 }
 
-void ReturnCreds(void)
+static void ReturnCreds(void)
 {
 	int NukeHandle;
 	int LogHandle;
@@ -755,8 +745,7 @@ void ReturnCreds(void)
 	dd_sendstring(d, "\n\n");
 }
 
-
-void NukeFile(void)
+static void NukeFile(void)
 {
 	int NukeHandle;
 	char nukebuf[512];
@@ -890,7 +879,7 @@ void NukeFile(void)
 	close(NukeHandle);
 }
 
-void copytonuke(char *src)
+static void copytonuke(char *src)
 {
 	char *dest;
 
@@ -900,12 +889,12 @@ void copytonuke(char *src)
 		*dest++ = *src++;
 }
 
-void FreeFile(void)
+static void FreeFile(void)
 {
 	FILE *FreeHandle;
 	char freebuf[256];
 
-	sprintf(freebuf, "%sdata/freedownloads.dat", thisconf->CONF_PATH);
+	sprintf(freebuf, "%s/data/freedownloads.dat", thisconf->CONF_PATH);
 
 	if ((FreeHandle = fopen(freebuf, "a"))) {
 		sprintf(freebuf, "%s\n", CurrFile);
@@ -922,7 +911,7 @@ void FreeFile(void)
 	updatedir = 1;
 }
 
-void Delete(int tapa)
+static void Delete(int tapa)
 {
 	int DelHandle;
 	char delbuf[256];
@@ -951,7 +940,7 @@ void Delete(int tapa)
 	}
 }
 
-void SaveDir(char *dirikka)
+static void SaveDir(char *dirikka)
 {
 	int dirhandle;
 	char fndbuf[40];
@@ -981,7 +970,7 @@ void SaveDir(char *dirikka)
 		DirMem[0] = 10;
 
 		sprintf(fndbuf, "\n%s ", CurrFile);
-		if ((fptr = Fnd(fndbuf, DirMem))) {
+		if ((fptr = strstr(DirMem, fndbuf))) {
 			fptr++;
 			feptr = GetFe(fptr);
 
@@ -1019,7 +1008,7 @@ void SaveDir(char *dirikka)
 	free(DirMem);
 }
 
-void CommentFile(void)
+static void CommentFile(void)
 {
 	char combuff[200];
 	char *cptr;
@@ -1052,37 +1041,30 @@ void CommentFile(void)
 		case 'A':
 			GenComline("Ancient!", sta->comline1);
 			return;
-			break;
 		case 'b':
 		case 'B':
 			GenComline("Broken!", sta->comline1);
 			return;
-			break;
 		case 'c':
 		case 'C':
 			GenComline("Crap!", sta->comline1);
 			return;
-			break;
 		case 'f':
 		case 'F':
 			GenComline("Fake!", sta->comline1);
 			return;
-			break;
 		case 'l':
 		case 'L':
 			GenComline("Lame!", sta->comline1);
 			return;
-			break;
 		case 'o':
 		case 'O':
 			GenComline("Old!", sta->comline1);
 			return;
-			break;
 		case 'r':
 		case 'R':
 			GenComline("Rename!", sta->comline1);
 			return;
-			break;
 		}
 	}
 	GenComline(combuff, sta->comline1);
@@ -1101,7 +1083,7 @@ void CommentFile(void)
 
 }
 
-void GenComline(const char *source, char *dest)
+static void GenComline(const char *source, char *dest)
 {
 	int i = 0;
 	int j = 0;
@@ -1136,7 +1118,7 @@ void GenComline(const char *source, char *dest)
 		dest[0] = 0;
 }
 
-char *GetFe(char *entry)
+static char *GetFe(char *entry)
 {
 	for (;;) {
 		while (*entry != 10)
@@ -1149,37 +1131,11 @@ char *GetFe(char *entry)
 			entry--;
 			*entry = 0;
 			return entry;
-			break;
 		}
 	}
 }
 
-/*
-   void ActivateFile(register __a0 char *path, register __a1 char *filename, register __d0 ULONG fsize, register __d1 ULONG unused)
-   {
-   struct Fakenode *fakenod;
-
-   fakenod=AllocMem(sizeof(struct Fakenode),MEMF_PUBLIC);
-
-   strcpy(fakenod->FN_Path,path);
-   strcpy(fakenod->FN_Name,filename);
-   fakenod->FN_Size=fsize;
-
-   AddTail(&Fakelist,fakenod);
-   }
-
-   void clearflist(void)
-   {
-   struct Fakenode *fakenod;
-
-   while (fakenod=RemTail((struct List *)&Fakelist))
-   {
-   FreeMem(fakenod,sizeof(struct Fakenode));
-   }
-   }
-
- */
-void CpyToEOL(char *src, char *dest)
+static void CpyToEOL(char *src, char *dest)
 {
 	int i = 0;
 
@@ -1190,7 +1146,7 @@ void CpyToEOL(char *src, char *dest)
 	dest[i] = 0;
 }
 
-void CpyToLAINA(char *src, char *dest)
+static void CpyToLAINA(char *src, char *dest)
 {
 	int i = 0;
 
@@ -1201,7 +1157,7 @@ void CpyToLAINA(char *src, char *dest)
 	dest[i] = 0;
 }
 
-void closeMe(void)
+static void closeMe(void)
 {
 	if (sta)
 		free(sta);
@@ -1210,7 +1166,7 @@ void closeMe(void)
 	dd_close(d);
 }
 
-void strupr(char *str)
+static void strupr(char *str)
 {
 	while (*str) {
 		*str = toupper(*str);
@@ -1218,10 +1174,10 @@ void strupr(char *str)
 	}
 }
 
-char *ExamineCfg(char *hay, char *need)
+static char *ExamineCfg(char *hay, char *need)
 {
 	char *s;
-	while (1) {
+	for (;;) {
 		s = need;
 		if (*hay == 0)
 			return 0;
@@ -1233,7 +1189,7 @@ char *ExamineCfg(char *hay, char *need)
 			}
 			continue;
 		}
-		while (1) {
+		for (;;) {
 			if (*s++ == *hay++) {
 				if (*s == 0)
 					return hay;
@@ -1243,8 +1199,3 @@ char *ExamineCfg(char *hay, char *need)
 		}
 	}
 }
-
-char *Fnd(char *needle, char *haystack)
-{
-	return strstr(haystack, needle);
-}
