@@ -11,49 +11,49 @@
 
 #include <config.h>
 
-void die(void);
-void nuketmp(void);
-void CpyToLAINA(char *, char *);
-char *ecfg(char *hay, char *need);
-int makeqwk();
-int getqwk();
-int maketmp();
-int findtmpnc(char *, char *);
-int get_conf_virt(int);
-void skipmsg(int, char *);
-void mystrcpy(char *, char *, int);
-void strupr(char *);
-void removespaces(char *);
-int myjoin(int);
-int get_virtual_conf(int, int);
-int procqwkm(struct DayDream_Message *);
-void setlrps(void);
-int inttoms(int, char[4]);
-void deldir(char *);
+static void die(void);
+static void nuketmp(void);
+static void CpyToLAINA(char *, char *);
+static char *ecfg(char *hay, char *need);
+static int makeqwk(void);
+static int getqwk(void);
+static int maketmp(void);
+static int findtmpnc(char *, char *);
+static int get_conf_virt(int);
+static void skipmsg(int, char *);
+static void mystrcpy(char *, char *, int);
+static void strupr(char *);
+static void removespaces(char *);
+static int myjoin(int);
+static int get_virtual_conf(int, int);
+static int procqwkm(struct DayDream_Message *);
+static void setlrps(void);
+static int inttoms(int, char[4]);
+static void deldir(char *);
 
-struct dif *d;
-char *cfg;
-int node;
+static struct dif *d;
+static char *cfg;
+static int node;
 
-unsigned short mcount = 0;
+static unsigned short mcount = 0;
 
-struct DayDream_MainConfig maincfg;
-char bbsid[20];
-char unzip[128];
-struct DayDream_LRP lp;
-struct DayDream_MsgPointers mp;
-struct DayDream_Conference *confs;
-struct DayDream_Conference *conf;
-struct DayDream_MsgBase *base;
-FILE *msgdfd;
-int ndxfd;
+static struct DayDream_MainConfig maincfg;
+static char bbsid[20];
+static char unzip[128];
+static struct DayDream_LRP lp;
+static struct DayDream_MsgPointers mp;
+static struct DayDream_Conference *confs;
+static struct DayDream_Conference *conf;
+static struct DayDream_MsgBase *base;
+static FILE *msgdfd;
+static int ndxfd;
 
-int blk_cnt = 2;
+static int blk_cnt = 2;
 
-char *fgetsnolf(char *buf, int n, FILE * fh);
-int cconf;
+static char *fgetsnolf(char *buf, int n, FILE * fh);
+static int cconf;
 
-int qwkcom(char *buf)
+static int qwkcom(char *buf)
 {
 	if (toupper(*buf) == 'D') {
 		makeqwk();
@@ -131,7 +131,7 @@ int main(int argc, char *argv[])
 	return 0;
 }
 
-int getqwk(void)
+static int getqwk(void)
 {
 	char qbuf[1024];
 	char tname[1024];
@@ -150,11 +150,11 @@ int getqwk(void)
 	if (!confs)
 		return 0;
 
-	sprintf(qbuf, "%sdd-grab%d/", DDTMP, node);
+	sprintf(qbuf, "%s/dd-grab%d/", DDTMP, node);
 	dd_getfiles(d, qbuf);
 
 	getcwd(olddir, 1024);
-	sprintf(qbuf, "%sdd-grab%d/", DDTMP, node);
+	sprintf(qbuf, "%s/dd-grab%d/", DDTMP, node);
 	chdir(qbuf);
 	sprintf(qbuf, "%s.rep", bbsid);
 	if (findtmpnc(qbuf, tname)) {
@@ -211,7 +211,7 @@ int getqwk(void)
 					mp.msp_high++;
 					msg.MSG_NUMBER = mp.msp_high;
 					dd_setmprs(d, &mp);
-					sprintf(qbuf, "%smessages/base%3.3d/msgbase.dat", conf->CONF_PATH, base->MSGBASE_NUMBER);
+					sprintf(qbuf, "%s/messages/base%3.3d/msgbase.dat", conf->CONF_PATH, base->MSGBASE_NUMBER);
 
 					if ((msgfd = open(qbuf, O_RDWR | O_CREAT, 0664)) < 0) {
 						skipmsg(repfd, qwkblock);
@@ -227,7 +227,7 @@ int getqwk(void)
 					write(msgfd, &msg, sizeof(struct DayDream_Message));
 					close(msgfd);
 
-					sprintf(qbuf, "%smessages/base%3.3d/msg%5.5d", conf->CONF_PATH, base->MSGBASE_NUMBER, msg.MSG_NUMBER);
+					sprintf(qbuf, "%s/messages/base%3.3d/msg%5.5d", conf->CONF_PATH, base->MSGBASE_NUMBER, msg.MSG_NUMBER);
 
 					msgfd = open(qbuf, O_CREAT | O_WRONLY, 0644);
 					if (toupper(base->MSGBASE_FN_FLAGS) == 'E') {
@@ -277,13 +277,13 @@ int getqwk(void)
 	return 1;
 }
 
-void mystrcpy(char *d, char *s, int l)
+static void mystrcpy(char *d, char *s, int l)
 {
 	strncpy(d, s, l);
 	removespaces(d);
 }
 
-void removespaces(char *strh)
+static void removespaces(char *strh)
 {
 	char *s;
 	s = strh;
@@ -297,18 +297,18 @@ void removespaces(char *strh)
 	*(s + 1) = 0;
 }
 
-void skipmsg(int fd, char *qbl)
+static void skipmsg(int fd, char *qbl)
 {
 	lseek(fd, (atoi(&qbl[116]) - 1) * 128, SEEK_CUR);
 }
 
-int findtmpnc(char *file, char *dname)
+static int findtmpnc(char *file, char *dname)
 {
 	char tbuf[1024];
 	DIR *dh;
 	struct dirent *dent;
 
-	sprintf(tbuf, "%sdd-grab%d/", DDTMP, node);
+	sprintf(tbuf, "%s/dd-grab%d/", DDTMP, node);
 
 	if ((dh = opendir(tbuf))) {
 		while ((dent = readdir(dh))) {
@@ -326,7 +326,7 @@ int findtmpnc(char *file, char *dname)
 	return 0;
 }
 
-int makeqwk(void)
+static int makeqwk(void)
 {
 	char qbuf[1024];
 	time_t t;
@@ -351,14 +351,14 @@ int makeqwk(void)
 		return 0;
 	conf = confs;
 
-	sprintf(qbuf, "%sdd-grab%d/messages.dat", DDTMP, node);
+	sprintf(qbuf, "%s/dd-grab%d/messages.dat", DDTMP, node);
 	msgdfd = fopen(qbuf, "w");
 	if (msgdfd == 0)
 		return 0;
 
 	fprintf(msgdfd, "%-128.128s", "Produced by Qmail...Copyright (c) 1987 by Sparkware.  All rights Reserved");
 
-	sprintf(qbuf, "%sdd-grab%d/control.dat", DDTMP, node);
+	sprintf(qbuf, "%s/dd-grab%d/control.dat", DDTMP, node);
 	ctrl = fopen(qbuf, "w");
 
 	fprintf(ctrl, "%s\r\n%s\r\n%s\r\n%s, Sysop\r\n", maincfg.CFG_BOARDNAME, "Location", "Phone", maincfg.CFG_SYSOPNAME);
@@ -366,7 +366,7 @@ int makeqwk(void)
 
 	dd_sendstring(d, "\n[35mPacking new messages\n\nConference                           Message Base          Progress\n[34m---------------------------------------------------------------------------\n");
 
-	while (1) {
+	for (;;) {
 		int newcnt;
 		struct stat st;
 		char mailbuf[1024];
@@ -379,7 +379,8 @@ int makeqwk(void)
 		if (myjoin(conf->CONF_NUMBER)) {
 			int confv = 0;
 
-			(struct DayDream_Conference *) base = conf + 1;
+			/* FIXME: Very suspicious code */
+			base = (struct DayDream_MsgBase *) conf + 1;
 			for (bcnt = conf->CONF_MSGBASES; bcnt; bcnt--, base++) {
 				int basefd;
 				char *cn;
@@ -406,13 +407,13 @@ int makeqwk(void)
 						dd_sendstring(d, "[31mNo tag!\n");
 						continue;
 					}
-					sprintf(qbuf, "%sdd-grab%d/%3.3d.ndx", DDTMP, node, get_virtual_conf(conf->CONF_NUMBER, base->MSGBASE_NUMBER));
+					sprintf(qbuf, "%s/dd-grab%d/%3.3d.ndx", DDTMP, node, get_virtual_conf(conf->CONF_NUMBER, base->MSGBASE_NUMBER));
 					ndxfd = open(qbuf, O_TRUNC | O_CREAT | O_WRONLY, 0644);
 					if (mp.msp_high > lp.lrp_read) {
 						struct DayDream_Message msg;
 						int seekp;
 
-						sprintf(mailbuf, "%smessages/base%3.3d/msgbase.dat", conf->CONF_PATH, base->MSGBASE_NUMBER);
+						sprintf(mailbuf, "%s/messages/base%3.3d/msgbase.dat", conf->CONF_PATH, base->MSGBASE_NUMBER);
 						basefd = open(mailbuf, O_RDONLY);
 						if (basefd == -1)
 							continue;
@@ -436,15 +437,17 @@ int makeqwk(void)
 						sprintf(mailbuf, "%d new msgs\n", newcnt);
 						dd_sendstring(d, mailbuf);
 					} else {
-						sprintf(qbuf, "%sdd-grab%d/%3.3d.ndx", DDTMP, node, get_virtual_conf(conf->CONF_NUMBER, base->MSGBASE_NUMBER));
+						sprintf(qbuf, "%s/dd-grab%d/%3.3d.ndx", DDTMP, node, get_virtual_conf(conf->CONF_NUMBER, base->MSGBASE_NUMBER));
 						unlink(qbuf);
 						dd_sendstring(d, "No new messages\n");
 					}
 				}
 			}
-			(struct DayDream_MsgBase *) conf = base;
+			/* FIXME: Very suspicious code */
+			conf = (struct DayDream_Conference *) base;
 		} else {
-			(struct DayDream_Conference *) base = conf + 1;
+			/* FIXME: Very suspicious code */
+			base = (struct DayDream_MsgBase *) conf + 1;
 			bcnt = conf->CONF_MSGBASES;
 
 			while (bcnt) {
@@ -452,7 +455,8 @@ int makeqwk(void)
 				base++;
 				bcnt--;
 			}
-			(struct DayDream_MsgBase *) conf = base;
+			/* FIXME: Very suspicious code */
+			conf = (struct DayDream_Conference *) base;
 		}
 	}
 	fprintf(ctrl, "%d\r\n%d\r\n", mcount, ccount - 1);
@@ -464,14 +468,16 @@ int makeqwk(void)
 		if (conf->CONF_NUMBER == 255)
 			break;
 
-		(struct DayDream_Conference *) base = conf + 1;
+		/* FIXME: Very suspicious code */
+		base = (struct DayDream_MsgBase *) conf + 1;
 		bcnt = conf->CONF_MSGBASES;
 
 		for (bcnt = conf->CONF_MSGBASES; bcnt; bcnt--, base++) {
 			vc++;
 			fprintf(ctrl, "%d\r\n%-13.13s\r\n", vc, base->MSGBASE_FN_TAG);
 		}
-		(struct DayDream_MsgBase *) conf = base;
+		/* FIXME: Very suspicious code */
+		conf = (struct DayDream_Conference *) base;
 	}
 	fprintf(ctrl, "\r\n\r\n\r\n");
 	fclose(ctrl);
@@ -483,15 +489,15 @@ int makeqwk(void)
 		sprintf(qbuf, "[34m---------------------------------------------------------------------------\n[36m%d messages archived. Compressing with zip...", mcount);
 		dd_sendstring(d, qbuf);
 		getcwd(cdir, 1024);
-		sprintf(qbuf, "%sdd-grab%d", DDTMP, node);
+		sprintf(qbuf, "%s/dd-grab%d", DDTMP, node);
 		chdir(qbuf);
 		sprintf(qbuf, "zip >/dev/null 2>/dev/null </dev/null %s.qwk *", bbsid);
 		system(qbuf);
 		dd_sendstring(d, "OK!\n");
 
-		sprintf(qbuf, "%sdd-grab%d/dltemp.lst", DDTMP, node);
+		sprintf(qbuf, "%s/dd-grab%d/dltemp.lst", DDTMP, node);
 		fl = fopen(qbuf, "w");
-		fprintf(fl, "%sdd-grab%d/%s.qwk\n", DDTMP, node, bbsid);
+		fprintf(fl, "%s/dd-grab%d/%s.qwk\n", DDTMP, node, bbsid);
 		fclose(fl);
 		dd_sendfiles(d, qbuf);
 		chdir(cdir);
@@ -502,7 +508,7 @@ int makeqwk(void)
 	return 1;
 }
 
-void setlrps(void)
+static void setlrps(void)
 {
 	dd_sendstring(d, "[36mSet last read pointers? [32m([33mYes[32m/[33mno[32m)[34m: [0m");
 	conf = confs;
@@ -516,8 +522,8 @@ void setlrps(void)
 				break;
 
 			if (myjoin(conf->CONF_NUMBER)) {
-
-				(struct DayDream_Conference *) base = conf + 1;
+				/* FIXME: Very suspicious code */
+				base = (struct DayDream_MsgBase *) conf + 1;
 				bcnt = conf->CONF_MSGBASES;
 				for (bcnt = conf->CONF_MSGBASES; bcnt; bcnt--, base++) {
 					dd_changemsgbase(d, base->MSGBASE_NUMBER, MC_QUICK | MC_NOSTAT);
@@ -530,22 +536,25 @@ void setlrps(void)
 						dd_setlprs(d, &lp);
 					}
 				}
-				(struct DayDream_MsgBase *) conf = base;
+				/* FIXME: Very suspicious code */
+				conf = (struct DayDream_Conference *) base;
 			} else {
-				(struct DayDream_Conference *) base = conf + 1;
+				/* FIXME: Very suspicious code */
+				base = (struct DayDream_MsgBase *) conf + 1;
 				bcnt = conf->CONF_MSGBASES;
 
 				while (bcnt) {
 					base++;
 					bcnt--;
 				}
-				(struct DayDream_MsgBase *) conf = base;
+				/* FIXME: Very suspicious code */
+				conf = (struct DayDream_Conference *) base;
 			}
 		}
 	}
 }
 
-int procqwkm(struct DayDream_Message *daheader)
+static int procqwkm(struct DayDream_Message *daheader)
 {
 	char realname[40];
 	char handle[40];
@@ -586,7 +595,7 @@ int procqwkm(struct DayDream_Message *daheader)
 	} else
 		return 0;
 
-	sprintf(msgname, "%smessages/base%3.3d/msg%5.5d", conf->CONF_PATH, base->MSGBASE_NUMBER, daheader->MSG_NUMBER);
+	sprintf(msgname, "%s/messages/base%3.3d/msg%5.5d", conf->CONF_PATH, base->MSGBASE_NUMBER, daheader->MSG_NUMBER);
 	if (stat(msgname, &st) == -1)
 		return 0;
 
@@ -645,10 +654,7 @@ int procqwkm(struct DayDream_Message *daheader)
 }
 
 /* inttoms ripped from uqwk */
-
-int inttoms(i, c)
-int i;
-char c[4];
+static int inttoms(int i, char c[4])
 /*
  *  Convert an integer into the Microsoft Basic floating format.
  *  This is the dumbest thing in the whole QWK standard.  Why in
@@ -677,7 +683,7 @@ char c[4];
 	return 0;
 }
 
-int myjoin(int confn)
+static int myjoin(int confn)
 {
 	if (!dd_isanybasestagged(d, confn))
 		return 0;
@@ -686,15 +692,15 @@ int myjoin(int confn)
 	return 1;
 }
 
-void nuketmp(void)
+static void nuketmp(void)
 {
 	char tb[1024];
-	sprintf(tb, "%sdd-grab%d", DDTMP, node);
+	sprintf(tb, "%s/dd-grab%d", DDTMP, node);
 	deldir(tb);
 	rmdir(tb);
 }
 
-void deldir(char *dir)
+static void deldir(char *dir)
 {
 	DIR *dirfd;
 	struct dirent *ent;
@@ -719,23 +725,23 @@ void deldir(char *dir)
 	}
 }
 
-int maketmp(void)
+static int maketmp(void)
 {
 	char tb[1024];
-	sprintf(tb, "%sdd-grab%d", DDTMP, node);
+	sprintf(tb, "%s/dd-grab%d", DDTMP, node);
 	nuketmp();
 	if (mkdir(tb, 0755) == 0)
 		return 1;
 	return 0;
 }
 
-void die(void)
+static void die(void)
 {
 	dd_joinconf(d, cconf, JC_QUICK | JC_SHUTUP);
 	dd_close(d);
 }
 
-char *ecfg(char *hay, char *need)
+static char *ecfg(char *hay, char *need)
 {
 	char *s;
 	while (1) {
@@ -761,7 +767,7 @@ char *ecfg(char *hay, char *need)
 	}
 }
 
-void CpyToLAINA(char *src, char *dest)
+static void CpyToLAINA(char *src, char *dest)
 {
 	int i = 0;
 
@@ -772,7 +778,7 @@ void CpyToLAINA(char *src, char *dest)
 	dest[i] = 0;
 }
 
-int get_virtual_conf(int confn, int basen)
+static int get_virtual_conf(int confn, int basen)
 {
 	struct DayDream_Conference *mconf;
 	struct DayDream_MsgBase *mbase;
@@ -785,19 +791,21 @@ int get_virtual_conf(int confn, int basen)
 		if (mconf->CONF_NUMBER == 255)
 			break;
 
-		(struct DayDream_Conference *) mbase = mconf + 1;
+		/* FIXME: Very suspicious code */
+		mbase = (struct DayDream_MsgBase *) mconf + 1;
 		bcnt = mconf->CONF_MSGBASES;
 		for (bcnt = mconf->CONF_MSGBASES; bcnt; bcnt--, mbase++) {
 			if (mbase->MSGBASE_NUMBER == basen && mconf->CONF_NUMBER == confn)
 				return i;
 			i++;
 		}
-		(struct DayDream_MsgBase *) mconf = mbase;
+		/* FIXME: Very suspicious code */
+		mconf = (struct DayDream_Conference *) mbase;
 	}
 	return 0;
 }
 
-int get_conf_virt(int virt)
+static int get_conf_virt(int virt)
 {
 	struct DayDream_Conference *mconf;
 	struct DayDream_MsgBase *mbase;
@@ -810,7 +818,8 @@ int get_conf_virt(int virt)
 		if (mconf->CONF_NUMBER == 255)
 			break;
 
-		(struct DayDream_Conference *) mbase = mconf + 1;
+		/* FIXME: Very suspicious code */
+		mbase = (struct DayDream_MsgBase *) mconf + 1;
 		for (bcnt = mconf->CONF_MSGBASES; bcnt; bcnt--, mbase++) {
 			if (virt == i) {
 				conf = mconf;
@@ -819,12 +828,13 @@ int get_conf_virt(int virt)
 			}
 			i++;
 		}
-		(struct DayDream_MsgBase *) mconf = mbase;
+		/* FIXME: Very suspicious code */
+		mconf = (struct DayDream_Conference *) mbase;
 	}
 	return 0;
 }
 
-char *fgetsnolf(char *buf, int n, FILE * fh)
+static char *fgetsnolf(char *buf, int n, FILE * fh)
 {
 	char *hih;
 	char *s;
@@ -843,9 +853,10 @@ char *fgetsnolf(char *buf, int n, FILE * fh)
 	return hih;
 }
 
-void strupr(char *strh)
+static void strupr(char *strh)
 {
 	while (*strh) {
-		*strh++ = toupper(*strh);
+		*strh = toupper(*strh);
+		strh++;
 	}
 }
