@@ -61,7 +61,7 @@ static int log_number()
 static void show_log()
 {
 	char buffer[1024];       
-	sprintf(buffer, "%s/logfiles/daydream%d.log", 
+	snprintf(buffer, sizeof(buffer), "%s/logfiles/daydream%d.log", 
 		getenv("DAYDREAM"), log_number());
 	dd_typefile(d, buffer, TYPE_WARN | TYPE_NOCODES);
 }
