@@ -3,7 +3,7 @@
 #include <stdio.h>
 #include <string.h>
 
-struct dif *d;
+static struct dif *d;
 
 int main(int argc, char *argv[])
 {
