@@ -3,7 +3,6 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <unistd.h>
-#include <daydream.h>
 #include <time.h>
 #include <fcntl.h>
 #include <sys/time.h>
@@ -11,10 +10,11 @@
 #include <sys/wait.h>
 #include <string.h>
 #include <stdlib.h>
-
 #include <syslog.h>
 
-#include "symtab.h"
+#include <daydream.h>
+#include <symtab.h>
+#include <ddcommon.h>
 
 static int write_dropfile(char *, int);
 
@@ -23,8 +23,6 @@ static int write_dropfile(char *, int);
 
 static int doorcnt = 0;
 
-extern int dsockfd;
-
 volatile int child_exit_timeout;
 
 list_t *child_pid_list = NULL;
@@ -36,40 +34,27 @@ static void sigalrm_handler(int signum)
 	socket_timeout = 1;
 }
 
-/* Caution: reviously scheduled alarm will be canceled. */
-static int accept_nonblock(int s, struct sockaddr *addr, int *addrlen)
+/* Caution: previously scheduled alarm will be canceled. */
+static int accept_nonblock(int sockfd, struct sockaddr *addr, 
+	socklen_t *addrlen)
 {
-	void (*old_sigalrm_handler)(int);
 	int fd;
-	
+
 	socket_timeout = 0;
 	alarm(3);
-	old_sigalrm_handler = signal(SIGALRM, sigalrm_handler);	
-	
-	fd = -1;	
-	if (old_sigalrm_handler != SIG_ERR)
-		do {
-			fd = accept(s, addr, addrlen);
-			if (fd != -1)
-				break;
-			if (errno != EAGAIN)
-				break;
-		} while (!socket_timeout);
+	signal(SIGALRM, sigalrm_handler);
+
+	do {
+		fd = accept(sockfd, addr, addrlen);
+	} while (!socket_timeout && fd == -1 && errno == EINTR);
 	
 	alarm(0);
-	if (old_sigalrm_handler == SIG_ERR)
-		return -1;
-	
-	if (signal(SIGALRM, old_sigalrm_handler) == SIG_ERR) {
-		if (fd != -1)
-			close(fd);
-		return -1;
-	}
-	
+	signal(SIGALRM, SIG_IGN);
+
 	return fd;
-}
+}	
 	
-int runpython(char *cmd, char *params)
+int runpython(const char *cmd, const char *params)
 {
 	char buf[1024];
 	if (!*maincfg.CFG_PYTHON)
@@ -78,7 +63,7 @@ int runpython(char *cmd, char *params)
 	return rundoor(buf, params);
 }
 
-static void door_loop(int sockfd, pid_t child_pid, char *door_params)
+static void door_loop(int sockfd, pid_t child_pid, const char *door_params)
 {
 	struct DayDream_DoorMsg msg;
 	char extstatus[50];
@@ -111,7 +96,7 @@ static void door_loop(int sockfd, pid_t child_pid, char *door_params)
 		tv.tv_usec = 0;
 		tv.tv_sec = 2;
 		
-		select_with_errchk(maxfd + 1, &setti, 0, 0, &tv);
+		select(maxfd + 1, &setti, 0, 0, &tv);
 		
 		if (!bgmode && FD_ISSET(dsockfd, &setti)) {
 			struct dd_nodemessage ddn;
@@ -133,7 +118,7 @@ static void door_loop(int sockfd, pid_t child_pid, char *door_params)
 				msg.ddm_data1 = Prompt(msg.ddm_string, msg.ddm_data1, msg.ddm_data2);
 				break;
 			case 4:
-				(int) msg.ddm_data1 = HotKey(msg.ddm_data1);
+				msg.ddm_data1 = (uint16_t) HotKey(msg.ddm_data1);
 				break;
 			case 5:
 				msg.ddm_data1 = TypeFile(msg.ddm_string, msg.ddm_data1);
@@ -145,18 +130,19 @@ static void door_loop(int sockfd, pid_t child_pid, char *door_params)
 				msg.ddm_data1 = findusername(msg.ddm_string);
 				break;
 			case 8:
-				runstdio(msg.ddm_string, 0, msg.ddm_data1);
+				runstdio(msg.ddm_string, -1, msg.ddm_data1);
 				break;
 			case 9:
-				docmd(msg.ddm_string, 0);
+				docmd(msg.ddm_string, 
+					strlen(msg.ddm_string), 0);
 				break;
 			case 10:
 				writelog(msg.ddm_string);
 				break;
 			case 11:
 				/* FIXME! */
-				strncpy(extstatus, msg.ddm_string, 50);
-				extstatus[49] = 0;
+				strlcpy(extstatus, msg.ddm_string, 
+					sizeof(extstatus));
 				changenodestatus(extstatus);
 				break;
 			case 12:
@@ -514,13 +500,8 @@ static void door_loop(int sockfd, pid_t child_pid, char *door_params)
 			case 143:
 				msg.ddm_data1 = current_msgbase->MSGBASE_NUMBER;
 				break;
-			case DPORT_OUTPUT_MASK:
-				userinput = msg.ddm_data1 & OUTPUT_REMOTE ? 1 : 0;
-				conout = msg.ddm_data1 & OUTPUT_LOCAL ? 2 : 0;
-				break;
-				
 			}
-			write(sockfd, &msg, sizeof(struct DayDream_DoorMsg));
+			safe_write(sockfd, &msg, sizeof(struct DayDream_DoorMsg));
 		}
 		if (!ispid(car(int, child_pid_list)))
 			break;
@@ -532,11 +513,8 @@ static void door_loop(int sockfd, pid_t child_pid, char *door_params)
 	close(sockfd);
 }
 
-int rundoor(char *command, char *params)
+int rundoor(const char *command, const char *params)
 {
-	int stored_conout = conout;
-	int stored_userinput = userinput;
-
 	int sockfd;
 	char sockname[80];
 	char *args[10];
@@ -554,6 +532,7 @@ int rundoor(char *command, char *params)
 
 	if (command == NULL)
 		return 0;
+
 	if (strspace(dcmd, command, sizeof(dcmd)) > sizeof(dcmd)) {
 		syslog(LOG_WARNING, "rundoor(): command too long");
 		return 0;
@@ -568,18 +547,13 @@ int rundoor(char *command, char *params)
 		return 0;
 	}
 	
-	/* socket must be non-blocking or we'll be waiting forever in
-	 * case of broken door.
-	 */
-	fcntl(sockfd, F_SETFL, O_NONBLOCK);
-	
 	if (!doorcnt) {
 		snprintf(sockname, sizeof(sockname), 
-			"%sdd_door%d", DDTMP, node);
+			"%s/dd_door%d", DDTMP, node);
 		snprintf(doom, sizeof(doom), "%d", node);
 	} else {
 		snprintf(sockname, sizeof(sockname),
-			 "%sdd_door%d_%d", DDTMP, node, doorcnt);
+			 "%s/dd_door%d_%d", DDTMP, node, doorcnt);
 		snprintf(doom, sizeof(doom), "%d_%d", node, doorcnt);
 	}
 	unlink(sockname);
@@ -615,24 +589,25 @@ int rundoor(char *command, char *params)
 	child_pid = fork();
 
 	if (child_pid == 0) {
-		execvp(args[0], &args[0]);
-		DDPut("exec failed!\n\n");
-		kill(getpid(), SIGKILL);
+		execv(args[0], &args[0]);
+		DDPut("WARNING: cannot invoke door program.\n");
+		DDPut("Please write a comment to SysOp.\n\n");
+		log_error(__FILE__, __LINE__,
+			"cannot execute door: %s (%d)",
+			strerror(errno), errno);
+		_exit(1);
 	} else if (child_pid == -1) {
-		DDPut("Fork failed!");
+		DDPut("WARNING: cannot spawn child process.\n");
+		log_error(__FILE__, __LINE__,
+			"cannot fork: %s (%d)",
+			strerror(errno), errno);
 	} else {
 		door_loop(sockfd, child_pid, params);		
 	}
 
-	signal(SIGPIPE, SIG_DFL);
-
 	close(sockfd);
 	unlink(sockname);
 
-	/* these may have been changed */
-	conout = stored_conout;
-	userinput = stored_userinput;
-
 	/* cancel the timeout alarm */
 	if (timeleft < 0) {
 		alarm(0);
@@ -642,13 +617,14 @@ int rundoor(char *command, char *params)
 	return 1;
 }
 
-int rundosdoor(char *command, char *params, int dropfile) {
-	char *dropfile_path, 
-	     *ptr, 
-	     *s,
-	     *args[10],
-	     dcmd[250], 
-	     doom[10];
+int rundosdoor(char *command, char *params, int dropfile) 
+{
+	char *dropfile_path; 
+	char *ptr; 
+	char *s;
+	char *args[10];
+	char dcmd[250]; 
+	char doom[10];
 	pid_t child_pid;
 	int status, i = 1;
 
@@ -702,12 +678,19 @@ int rundosdoor(char *command, char *params, int dropfile) {
 	child_pid = fork();
 
 	if (child_pid == 0) {
-		execvp(args[0], &args[0]);
-		ddprintf("exec failed: %s\n\n", strerror(errno));
-		kill(getpid(), SIGKILL);
-	} else if (child_pid == -1)
-		DDPut("Fork failed!");
-	else
+		execv(args[0], &args[0]);
+		DDPut("WARNING: cannot invoke door program.\n");
+		DDPut("Please write a comment to SysOp.\n\n");
+		log_error(__FILE__, __LINE__,
+			"cannot execute door: %s (%d)",
+			strerror(errno), errno);
+		_exit(1);
+	} else if (child_pid == -1) {
+		DDPut("WARNING: cannot spawn child process.\n");
+		log_error(__FILE__, __LINE__,
+			"cannot fork: %s (%d)",
+			strerror(errno), errno);
+	} else
 		waitpid(child_pid, &status, 0); /* wait for child to finish */
 
 	fflush(stdout);		
