@@ -3,10 +3,9 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "daydream.h"
-#include "menucmd.h"
-
-#include "symtab.h"
+#include <daydream.h>
+#include <menucmd.h>
+#include <symtab.h>
 
 struct command_arg {
 	struct atom callback_table;	
@@ -54,63 +53,60 @@ enum {
 	__retval__;							\
 })
 
-//#define string_car_and_cdr(__list__) car_and_cdr(__list__, char *, NULL, string)
 #define string_car_and_cdr(__list__) __string_car_and_cdr(&__list__)
 #define integer_car_and_cdr(__list__) car_and_cdr(__list__, int, 0, integer)
 			
 #define push(__list__, __x__) __list__ = push(__list__, __x__)
 
-#define ITEM(__x__) { __STRING(__x__), CMD_##__x__## }
-
 struct command_convert_string_to_int {
 	const char *string;
 	int code;
 };
 
 static struct command_convert_string_to_int cmd_strings[] = {
-	ITEM(VER),
-	ITEM(EXPERT_MODE),
-	ITEM(LOGOFF),
-	ITEM(CHANGE_INFO),
-	ITEM(FILE_SCAN),
-	ITEM(DOWNLOAD),
-	ITEM(TAG_EDITOR),
-	ITEM(BULLETINS),
-	ITEM(NEW_FILES),
-	ITEM(ZIPPY_SEARCH),
-	ITEM(TIME),
-	ITEM(STATS),
-	ITEM(ENTER_MSG),
-	ITEM(COMMENT),
-	ITEM(READ_MSGS),
-	ITEM(GLOBAL_READ),
-	ITEM(JOIN_CONF),
-	ITEM(CHANGE_MSGBASE),
-	ITEM(LOCAL_UPLOAD),
-	ITEM(UPLOAD),
-	ITEM(UPLOAD_RZ),
-	ITEM(VIEW_FILE),
-	ITEM(SCAN_MAIL),
-	ITEM(GLOBAL_FSCAN),
-	ITEM(TAG_MSGBASES),
-	ITEM(TAG_CONFS),
-	ITEM(PREV_CONF),
-	ITEM(NEXT_CONF),
-	ITEM(PREV_BASE),
-	ITEM(NEXT_BASE),
-	ITEM(WHO),
-	ITEM(HELP),
-	ITEM(USERLIST),
-	ITEM(MODE),
-	ITEM(PAGE),
-	ITEM(OLM),
-	ITEM(MOVE),
-	ITEM(COPY),
-	ITEM(LINK),
-	ITEM(SYSOP_DOWNLOAD),
-	ITEM(USERED),
-	ITEM(TEXT_SEARCH),
-	ITEM(TEST_DOOR),
+	{ "VER", CMD_VER },
+	{ "EXPERT_MODE", CMD_EXPERT_MODE },
+	{ "LOGOFF", CMD_LOGOFF },
+	{ "CHANGE_INFO", CMD_CHANGE_INFO },
+	{ "FILE_SCAN", CMD_FILE_SCAN },
+	{ "DOWNLOAD", CMD_DOWNLOAD },
+	{ "TAG_EDITOR", CMD_TAG_EDITOR },
+	{ "BULLETINS", CMD_BULLETINS },
+	{ "NEW_FILES", CMD_NEW_FILES },
+	{ "ZIPPY_SEARCH", CMD_ZIPPY_SEARCH },
+	{ "TIME", CMD_TIME },
+	{ "STATS", CMD_STATS },
+	{ "ENTER_MSG", CMD_ENTER_MSG },
+	{ "COMMENT", CMD_COMMENT },
+	{ "READ_MSGS", CMD_READ_MSGS },
+	{ "GLOBAL_READ", CMD_GLOBAL_READ },
+	{ "JOIN_CONF", CMD_JOIN_CONF },
+	{ "CHANGE_MSGBASE", CMD_CHANGE_MSGBASE },
+	{ "LOCAL_UPLOAD", CMD_LOCAL_UPLOAD },
+	{ "UPLOAD", CMD_UPLOAD },
+	{ "UPLOAD_RZ", CMD_UPLOAD_RZ },
+	{ "VIEW_FILE", CMD_VIEW_FILE },
+	{ "SCAN_MAIL", CMD_SCAN_MAIL },
+	{ "GLOBAL_FSCAN", CMD_GLOBAL_FSCAN },
+	{ "TAG_MSGBASES", CMD_TAG_MSGBASES },
+	{ "TAG_CONFS", CMD_TAG_CONFS },
+	{ "PREV_CONF", CMD_PREV_CONF },
+	{ "NEXT_CONF", CMD_NEXT_CONF },
+	{ "PREV_BASE", CMD_PREV_BASE },
+	{ "NEXT_BASE", CMD_NEXT_BASE },
+	{ "WHO", CMD_WHO },
+	{ "HELP", CMD_HELP },
+	{ "USERLIST", CMD_USERLIST },
+	{ "MODE", CMD_MODE },
+	{ "PAGE", CMD_PAGE },
+	{ "OLM", CMD_OLM },
+	{ "MOVE", CMD_MOVE },
+	{ "COPY", CMD_COPY },
+	{ "LINK", CMD_LINK },
+	{ "SYSOP_DOWNLOAD", CMD_SYSOP_DOWNLOAD },
+	{ "USERED", CMD_USERED },
+	{ "TEXT_SEARCH", CMD_TEXT_SEARCH },
+	{ "TEST_DOOR", CMD_TEST_DOOR },
 	{ NULL, 0 }	       
 };
 
@@ -149,7 +145,7 @@ enum {
 
 #undef CREATE_CMD
 #define CREATE_CMD(__x__, __z__, __y__) { 	\
-	__z__, REF_CMD(__x__)##, __y__, __x__ 	\
+	__z__, REF_CMD(__x__), __y__, __x__ 	\
 }
 
 #undef ARG0
@@ -189,7 +185,6 @@ static int print(list_t *args, list_t **stmt_list);
 static int kbd_stuff(list_t *args, list_t **stmt_list);
 static int do_command(list_t *args, list_t **stmt_list);
 static int do_return(list_t *args, list_t **stmt_list);
-//static int type_file(list_t *args, list_t **stmt_list);
 
 struct command_string command_strings[] = { 
 	COMMANDS,
@@ -203,7 +198,6 @@ static list_t *scope_args;
 static struct command_string *get_command_definition(int command_code);
 static int tokenize_command(const char *command);
 static int get_arg_template(int command_code);
-void parse_menu_command(const char **source);
 static int check_arguments(struct atom *command_atom);
 static int execute_command(list_t *command_list);
 
@@ -258,7 +252,7 @@ void fini_menu_system(void)
 		pop_menu(NULL, &dummy_list);
 }
 
-static int default_menu(list_t *args, list_t **stmt_list)
+static int default_menu(list_t *args, list_t **stmt_list) 
 {
 	*stmt_list = cdr(*stmt_list);
 	if (load_default_commands())
@@ -455,8 +449,6 @@ static int bind_hotkey(list_t *args, list_t **stmt_list)
 	return EXEC_OK;
 }
 
-int primitive_docmd(int command, char *params);
-
 static char *get_parameters(list_t *args)
 {
 	string_t *str = NULL;
@@ -515,6 +507,11 @@ static int test_door(list_t *args, list_t **stmt_list)
 	char *params, *command;
 	char *buffer;
 
+	if (user.user_securitylevel != 255) {
+		DDPut(sd[accessdeniedstr]);
+		return(EXEC_ARG_ERROR);
+	}
+	
 	*stmt_list = cdr(*stmt_list);
 	command = string_car_and_cdr(args);
 	
@@ -532,8 +529,28 @@ static int test_door(list_t *args, list_t **stmt_list)
 	return EXEC_OK;
 }
 
+static int ask_password_for_door(const char *doorname, const char *passwd)
+{
+	char temp[100];
+	int i;
+
+	if (!passwd[0])
+		return 1;
+	
+	snprintf(temp, sizeof(temp), "doorpasswd%s", doorname);
+	for (i = 0; i < strlen(temp); i++)
+		temp[i] = tolower(temp[i]);
+	TypeFile(temp, TYPE_MAKE | TYPE_SEC | TYPE_CONF);
+
+	temp[0] = 0;
+	if (!Prompt(temp, 16, PROMPT_SECRET))
+		return -1;
+	return strcasecmp(temp, passwd) ? 0 : 1;
+}	
+
 static int run_door(list_t *args, list_t **stmt_list)
 {
+	int retvalue;
 	struct DD_ExternalCommand *ext;
 	char *command = string_car_and_cdr(args);
 	char *params;
@@ -549,20 +566,28 @@ static int run_door(list_t *args, list_t **stmt_list)
 				g_free(params);
 				return EXEC_NULL; /* hmm? */
 			}
+			retvalue = ask_password_for_door(ext->EXT_NAME,
+				ext->EXT_PASSWD);
+			if (retvalue == -1)
+				return EXEC_HANGUP;
+			if (!retvalue) {
+				DDPut(sd[accessdeniedstr]);
+				return 1;
+			}
 			switch (ext->EXT_CMDTYPE) {
 			case 1:
 				rundoor(ext->EXT_COMMAND, params);
 				break;
 			case 2:
-				runstdio(ext->EXT_COMMAND, 0, 1);
+				runstdio(ext->EXT_COMMAND, -1, 1);
 				break;
 			case 3:
 				TypeFile(ext->EXT_COMMAND,
 					 TYPE_MAKE | TYPE_WARN);
 				break;
 			case 4:
-				return docmd(ext->EXT_COMMAND, 1);
-				break;
+				return docmd(ext->EXT_COMMAND, 
+					strlen(ext->EXT_COMMAND), 1);
 			case 5:
 				stdioout(ext->EXT_COMMAND);
 				break;
@@ -595,18 +620,6 @@ static int print(list_t *args, list_t **stmt_list)
 	return EXEC_OK;
 }
 
-/*static int type_file(list_t *args, list_t **stmt_list)
-{
-	char *fname = string_car_and_cdr(args);
-	int flags = integer_car_and_cdr(args);
-	*stmt_list = cdr(*stmt_list);	
-	if (fname)
-		TypeFile(fname, flags);
-	return EXEC_OK;
-}*/
-
-void keyboard_stuff(char *what);
-
 static int kbd_stuff(list_t *args, list_t **stmt_list)
 {
 	char *str = string_car_and_cdr(args);
@@ -643,7 +656,7 @@ static int do_command(list_t *args, list_t **stmt_list)
 	char *params;
 	
 	params = get_parameters(args);
-	retcode = docmd(params, 0);
+	retcode = docmd(params, strlen(params), 0);
 	g_free(params);
 	return retcode;
 }
@@ -761,10 +774,10 @@ static int substitute_args(list_t *args, list_t **stmt_list)
 	return error_code;	
 }
 
-int docmd(const char *command, int skipd)
+int docmd(const char *command, size_t command_len, int skipd)
 {
 	struct symbol *binding;
-	const char *params;
+	const char *params, *ptr;
 	int error_code = 0;
 	list_t *list = NULL;
 	if (!current_menu)
@@ -773,7 +786,8 @@ int docmd(const char *command, int skipd)
 	if (!command)
 		return EXEC_NULL;
 		
-	command = strdup(command);
+	ptr = strndup(command, command_len);
+	command = ptr;
 	
 	command = params = trim(command);
 	while (*params && !isspace(*params))
@@ -794,7 +808,7 @@ int docmd(const char *command, int skipd)
 	if (!(binding = symbol_table_lookup(current_menu->hotkeys, command)))
 		if (!(binding = symbol_table_lookup(current_menu->commands,
 						    command))) {
-			g_free((char *) command);
+			g_free((char *) ptr);
 			return EXEC_UNDEFINED;
 		}
 	
@@ -804,7 +818,7 @@ int docmd(const char *command, int skipd)
 	error_code = execute_command(binding->atom_list);
 	atom_list_destroy(shift(list_t *, scope_args));
 	
-	g_free((char *) command);
+	g_free((char *) ptr);
 	return error_code;
 }
 
@@ -832,7 +846,7 @@ int domenu(int pause)
 		return EXEC_HANGUP;
 
 	s = trim(buffer);
-	return docmd(s, 0);
+	return docmd(s, strlen(s), 0);
 }
 
 static int execute_command(list_t *command_list)
@@ -842,7 +856,6 @@ static int execute_command(list_t *command_list)
 	list_t *head, *work_list;
 	
 	work_list = head = atom_list_clone(command_list);
-//	head = work_list;
 	
 	for (;;) {
 		struct command *command;
@@ -1015,7 +1028,7 @@ static char *read_quoted_string(const char **source)
 			strappend_c (result, '\\');
 			break;		
 		case 'e':
-			strappend_c (result, '\e');
+			strappend_c (result, '\033');
 			break;
 		case 'n':
 			strappend_c (result, '\n');
