@@ -47,7 +47,7 @@ int readmessages(int cseekp, int premsg, char *mask)
 
 	getmsgptrs();
 
-	snprintf(rbuffer, sizeof(rbuffer), "%smessages/base%3.3d/msgbase.dat", 
+	snprintf(rbuffer, sizeof(rbuffer), "%s/messages/base%3.3d/msgbase.dat", 
 		conference()->conf.CONF_PATH, current_msgbase->MSGBASE_NUMBER);
 
 	msgbasesize = getfilesize(rbuffer);
@@ -205,11 +205,11 @@ static void updatemsgdatas(void)
 	int msgdatfd;
 
 	snprintf(rbuffer, sizeof(rbuffer), 
-		"%smessages/base%3.3d/msgbase.dat", 
+		"%s/messages/base%3.3d/msgbase.dat", 
 		conference()->conf.CONF_PATH, current_msgbase->MSGBASE_NUMBER);
 	msgdatfd = open(rbuffer, O_WRONLY);
 	lseek(msgdatfd, oldseekpoint, SEEK_SET);
-	write(msgdatfd, msgbuf, bread);
+	safe_write(msgdatfd, msgbuf, bread);
 	close(msgdatfd);
 }
 
@@ -243,20 +243,19 @@ static int showmsg(int showme, int mode)
 				hinkmode = 1;
 			} else {
 				snprintf(rbuffer, sizeof(rbuffer),
-					"%smessages/base%3.3d/msg%5.5d", 
+					"%s/messages/base%3.3d/msg%5.5d", 
 					conference()->conf.CONF_PATH, 
 					current_msgbase->MSGBASE_NUMBER, 
 					showme);
 				msghandle = open(rbuffer, O_RDONLY);
-				if (msghandle == -1) {
+				if (msghandle == -1)
 					return 2;
-				}
 				close(msghandle);
 				if (showme < msgbuf->MSG_NUMBER) {
 					lowmode = 1;
 					if (!oldseekpoint)
 						return 2;
-					seekpoint = seekpoint - 100 * sizeof(struct DayDream_MsgBase);
+					seekpoint = seekpoint - 100 * sizeof(struct DayDream_Message);
 					if (seekpoint == oldseekpoint)
 						return 2;
 					if (seekpoint < 0)
@@ -296,7 +295,7 @@ static int showmsg(int showme, int mode)
 			return 3;
 		}
 	}
-	snprintf(rbuffer, sizeof(rbuffer), "%smessages/base%3.3d/msg%5.5d", 
+	snprintf(rbuffer, sizeof(rbuffer), "%s/messages/base%3.3d/msg%5.5d", 
 		conference()->conf.CONF_PATH, current_msgbase->MSGBASE_NUMBER, 
 		showme);
 	msgfd = fopen(rbuffer, "r");
@@ -409,7 +408,7 @@ static int showmsg(int showme, int mode)
 		char buf2[1024];
 		FILE *atlist;
 
-		snprintf(fabuf, sizeof(fabuf), "%smessages/base%3.3d/msf%5.5d",
+		snprintf(fabuf, sizeof(fabuf), "%s/messages/base%3.3d/msf%5.5d",
 			conference()->conf.CONF_PATH, 
 			current_msgbase->MSGBASE_NUMBER, daheader->MSG_NUMBER);
 		atlist = fopen(fabuf, "r");
@@ -435,7 +434,7 @@ static int showmsg(int showme, int mode)
 				struct stat st;
 
 				snprintf(buf2, sizeof(buf2), 
-					"%smessages/base%3.3d/fa%5.5d/%s", 
+					"%s/messages/base%3.3d/fa%5.5d/%s", 
 					conference()->conf.CONF_PATH, 
 					current_msgbase->MSGBASE_NUMBER, 
 					daheader->MSG_NUMBER, fabuf);
@@ -464,7 +463,7 @@ static int showmsg(int showme, int mode)
 
 				getcwd(olddir, 1024);
 				snprintf(fabuf, sizeof(fabuf), 
-					"%smessages/base%3.3d/fa%5.5d", 
+					"%s/messages/base%3.3d/fa%5.5d", 
 					conference()->conf.CONF_PATH, 
 					current_msgbase->MSGBASE_NUMBER, 
 					daheader->MSG_NUMBER);
@@ -495,21 +494,21 @@ static int deletemsg(int delme)
 	}
 	if ((access1 & (1L << SECB_DELETEANY)) || (!strcasecmp(user.user_handle, daheader->MSG_AUTHOR)) || (!strcasecmp(user.user_realname, daheader->MSG_AUTHOR)) || (!strcasecmp(user.user_handle, daheader->MSG_RECEIVER)) || (!strcasecmp(user.user_realname, daheader->MSG_RECEIVER))) {
 		snprintf(rbuffer, sizeof(rbuffer), 
-			"%smessages/base%3.3d/msg%5.5d", 
+			"%s/messages/base%3.3d/msg%5.5d", 
 			conference()->conf.CONF_PATH, 
 			current_msgbase->MSGBASE_NUMBER, daheader->MSG_NUMBER);
 		unlink(rbuffer);
 		daheader->MSG_FLAGS |= (1L << 1);
 		if (*daheader->MSG_ATTACH == 1) {
 			snprintf(rbuffer, sizeof(rbuffer),
-				"%smessages/base%3.3d/fa%5.5d", 
+				"%s/messages/base%3.3d/fa%5.5d", 
 				conference()->conf.CONF_PATH, 
 				current_msgbase->MSGBASE_NUMBER, 
 				daheader->MSG_NUMBER);
 			deldir(rbuffer);
 			unlink(rbuffer);
 			snprintf(rbuffer, sizeof(rbuffer),
-				"%smessages/base%3.3d/msf%5.5d", 
+				"%s/messages/base%3.3d/msf%5.5d", 
 				conference()->conf.CONF_PATH, 
 				current_msgbase->MSGBASE_NUMBER, 
 				daheader->MSG_NUMBER);
@@ -529,7 +528,7 @@ static int editmsg(int editme)
 
 	if ((!strcasecmp(user.user_handle, daheader->MSG_AUTHOR)) || (!strcasecmp(user.user_realname, daheader->MSG_AUTHOR)) || user.user_securitylevel == 255) {
 		snprintf(rbuffer, sizeof(rbuffer),
-			"%smessages/base%3.3d/msg%5.5d", 
+			"%s/messages/base%3.3d/msg%5.5d", 
 			conference()->conf.CONF_PATH, 
 			current_msgbase->MSGBASE_NUMBER, daheader->MSG_NUMBER);
 		DDPut("\r                                                                 \r");
@@ -615,7 +614,7 @@ static int editfile(char *file)
 	wu = fopen(file, "r");
 	if (wu) {
 		rep = 1;
-		snprintf(mbuf, sizeof(mbuf), "%sdaydream%d.msg", DDTMP, node);
+		snprintf(mbuf, sizeof(mbuf), "%s/daydream%d.msg", DDTMP, node);
 		tang = fopen(mbuf, "w");
 		while (fgets(mbuf, 80, wu))
 			fputs(mbuf, tang);
@@ -674,7 +673,7 @@ static void readmsgdatas(void)
 	struct DayDream_Message *kalamsg;
 	int moffset;
 
-	snprintf(rbuffer, sizeof(rbuffer), "%smessages/base%3.3d/msgbase.dat", 
+	snprintf(rbuffer, sizeof(rbuffer), "%s/messages/base%3.3d/msgbase.dat", 
 		conference()->conf.CONF_PATH, current_msgbase->MSGBASE_NUMBER);
 	msgdatfd = open(rbuffer, O_RDONLY);
 	lseek(msgdatfd, seekpoint, SEEK_SET);
@@ -693,7 +692,6 @@ int globalread(void)
 	int oldconf;
 	conference_t *mc;
 	struct iterator *iterator;
-	int go = 1;
 
 	oldconf = user.user_joinconference;
 	
@@ -709,10 +707,8 @@ int globalread(void)
 			
 			changemsgbase(mbase->MSGBASE_NUMBER, MC_QUICK | MC_NOSTAT);
 			if (highest > lrp && isbasetagged(conference()->conf.CONF_NUMBER, current_msgbase->MSGBASE_NUMBER)) {
-				if (readmessages(-1, -1, NULL) == 2) {
-					go = 0;
+				if (readmessages(-1, -1, NULL) == 2) 
 					break;
-				}
 			}
 		}	       
 	}
@@ -736,23 +732,24 @@ static int getfilesize(char *pathi)
 
 int strlena(char *s)
 {
-	int i = 0;
+	int sz = 0;
+	char *p;
 
 	while (*s) {
-		if (*s == 27) {
-			s++;
-			while (isdigit(*s) || *s == ';') {
-				s++;
-				if (*s == 0)
-					return i;
-			}
-			s++;
+		if (s[0] == '\033' && s[1] == '[') {
+			p = strpbrk(s + 2, "@AbBcCdDgGHiIJKLmMnPRSTXZ");
+			if (p == NULL) {
+				sz += strlen(s);
+				break;
+			} else 
+				s = p + 1;
 		} else {
-			i++;
 			s++;
+			sz++;
 		}
 	}
-	return i;
+
+	return sz;
 }
 
 static int keepmsg(int msgnum)
@@ -766,8 +763,8 @@ static int keepmsg(int msgnum)
 	if (!daheader) {
 		return 0;
 	}
-	if ((!strcasecmp(msg.MSG_RECEIVER, user.user_handle))
-	    || (!strcasecmp(msg.MSG_RECEIVER, user.user_realname))
+	if ((!strcasecmp(daheader->MSG_RECEIVER, user.user_handle))
+	    || (!strcasecmp(daheader->MSG_RECEIVER, user.user_realname))
 	    || user.user_securitylevel >= maincfg.CFG_COSYSOPLEVEL) {
 
 		msg = *daheader;
@@ -781,24 +778,24 @@ static int keepmsg(int msgnum)
 		msg.MSG_NUMBER = highest;
 		setmsgptrs();
 
-		snprintf(buf1, sizeof(buf1), "%smessages/base%3.3d/msg%5.5d", 
+		snprintf(buf1, sizeof(buf1), "%s/messages/base%3.3d/msg%5.5d", 
 			conference()->conf.CONF_PATH, 
 			current_msgbase->MSGBASE_NUMBER, daheader->MSG_NUMBER);
-		snprintf(buf2, sizeof(buf2), "%smessages/base%3.3d/msg%5.5d", 
+		snprintf(buf2, sizeof(buf2), "%s/messages/base%3.3d/msg%5.5d", 
 			conference()->conf.CONF_PATH, 
 			current_msgbase->MSGBASE_NUMBER, msg.MSG_NUMBER);
 		newrename(buf1, buf2);
 
 		snprintf(buf1, sizeof(buf1),
-			"%smessages/base%3.3d/msgbase.dat", 
+			"%s/messages/base%3.3d/msgbase.dat", 
 			conference()->conf.CONF_PATH, 
 			current_msgbase->MSGBASE_NUMBER);
 
-		if ((msgfd = open(buf1, O_RDWR | O_CREAT, 0664)) < 0)
+		if ((msgfd = open(buf1, O_RDWR | O_CREAT, 0666)) == -1)
 			return 0;
-
+		fsetperm(msgfd, 0666);
 		lseek(msgfd, 0, SEEK_END);
-		write(msgfd, &msg, sizeof(struct DayDream_Message));
+		safe_write(msgfd, &msg, sizeof(struct DayDream_Message));
 		close(msgfd);
 
 		seekpoint = oldseekpoint;
