@@ -4,7 +4,7 @@
 #include <sys/types.h>
 #include <stdlib.h>
 
-#include "utility.h"
+#include <utility.h>
 
 /* An "atom" is a terminal symbol, e.g. terminal symbol.
  * 
