@@ -12,16 +12,18 @@
 #include <sys/mman.h>
 #include <signal.h>
 #include <syslog.h>
-
-#include "daydream.h"
-#include "utility.h"
-
+#ifdef HAVE_SYS_STATVFS_H
+#include <sys/statvfs.h>
+#endif
 #ifdef HAVE_SYS_VFS_H
 #include <sys/vfs.h>
 #endif
-
 #include <sys/mount.h>
 
+#include <daydream.h>
+#include <ddcommon.h>
+#include <utility.h>
+
 struct bgcheckinfo {
 	struct userbase bguser;
 	int totbytes;
@@ -31,8 +33,6 @@ struct bgcheckinfo {
 	int okbytes;
 };
 
-extern int lmode; /* FIXME! should be declared in daydream.h? */
-
 static void bghan(int);
 static int checkbgdone(int);
 static int checkfilename(char *);
@@ -58,9 +58,8 @@ int handleupload(char *);
 
 struct DayDream_Archiver *arc; /* FIXME! where does this belong? */
 
-int bgrun = 0; /* FIXME! should be declared in daydream.h */
-int wasbg; /* FIXME! should be declared in daydream.h */
-
+int bgrun = 0; 
+int wasbg; 
 
 static int addulbytes(int by)
 {
@@ -92,33 +91,33 @@ void recfiles(char *path, char *confp)
 	getcwd(olddir, 1024);
 	chdir(path);
 
-	snprintf(buf2, sizeof(buf2), "%sdszlog.%d", DDTMP, node);
+	snprintf(buf2, sizeof(buf2), "%s/dszlog.%d", DDTMP, node);
 	unlink(buf2);
 
 	if (lmode) {
 		HotKey(0);
 	} else if (protocol->PROTOCOL_TYPE == 1) {
 		if (confp) {
-			snprintf(udbuf, sizeof(udbuf), "%s/utils/ddrz -vv -b -r -f %sdata/paths.dat -F %stmplist%d -l %s -g %s -n %snodeinfo%d.data", origdir, conference()->conf.CONF_PATH, DDTMP, node, buf2, ttyname(serhandle), DDTMP, node);
+			snprintf(udbuf, sizeof(udbuf), "%s/utils/ddrz -vv -b -r -f %s/data/paths.dat -F %s/tmplist%d -l %s -g %s -n %s/nodeinfo%d.data", origdir, conference()->conf.CONF_PATH, DDTMP, node, buf2, ttyname(serhandle), DDTMP, node);
 		} else {
-			snprintf(udbuf, sizeof(udbuf), "%s/utils/ddrz -vv -b -r -l %s -g %s -n %snodeinfo%d.data", origdir, buf2, ttyname(serhandle), DDTMP, node);
+			snprintf(udbuf, sizeof(udbuf), "%s/utils/ddrz -vv -b -r -l %s -g %s -n %s/nodeinfo%d.data", origdir, buf2, ttyname(serhandle), DDTMP, node);
 		}
-		runstdio(udbuf, 0, 2);
+		runstdio(udbuf, -1, 2);
 	} else if (protocol->PROTOCOL_TYPE == 2) {
 		if (confp) {
-			snprintf(udbuf, sizeof(udbuf), "%s/utils/hydracom dev %s ski %sdata/paths.dat rec %s res %s get", origdir, ttyname(serhandle), conference()->conf.CONF_PATH, path, buf2);
+			snprintf(udbuf, sizeof(udbuf), "%s/utils/hydracom dev %s ski %s/data/paths.dat rec %s/ res %s get", origdir, ttyname(serhandle), conference()->conf.CONF_PATH, path, buf2);
 		} else {
-			snprintf(udbuf, sizeof(udbuf), "%s/utils/hydracom dev %s rec %s res %s get", origdir, ttyname(serhandle), path, buf2);
+			snprintf(udbuf, sizeof(udbuf), "%s/utils/hydracom dev %s rec %s/ res %s get", origdir, ttyname(serhandle), path, buf2);
 		}
-		runstdio(udbuf, 0, 4);
+		runstdio(udbuf, -1, 4);
 		killflood();
 	} else if (protocol->PROTOCOL_TYPE == 3) {
 		if (confp) {
-			snprintf(udbuf, sizeof(udbuf), "%s/utils/smodem -t -X -p %s -e 0 -d %s @%sdata/paths.dat @%stmplist%d -z %s", origdir, ttyname(serhandle), path, conference()->conf.CONF_PATH, DDTMP, node, buf2);
+			snprintf(udbuf, sizeof(udbuf), "%s/utils/smodem -t -X -p %s -e 0 -d %s/ @%s/data/paths.dat @%s/tmplist%d -z %s", origdir, ttyname(serhandle), path, conference()->conf.CONF_PATH, DDTMP, node, buf2);
 		} else {
-			snprintf(udbuf, sizeof(udbuf), "%s/utils/smodem -t -X -p %s -x -e 0 -d %s -z %s", origdir, ttyname(serhandle), path, buf2);
+			snprintf(udbuf, sizeof(udbuf), "%s/utils/smodem -t -X -p %s -x -e 0 -d %s/ -z %s", origdir, ttyname(serhandle), path, buf2);
 		}
-		runstdio(udbuf, 0, 4);
+		runstdio(udbuf, -1, 4);
 		killflood();
 	} else if (protocol->PROTOCOL_TYPE == 4) {
 		checkforftp(2, path, buf2);
@@ -134,12 +133,12 @@ static void terminate_background_checker(void)
 
 		DDPut(sd[w4bgcheckstr]);
 		kill(bgrun, SIGUSR1);
-		while (1) {
+		for (;;) {
 			if (checkbgdone(0))
 				break;
 			sleep(1);
 			if (!ispid(bgrun)) {
-				checkbgdone(0);		// Just to be sure 
+				checkbgdone(0);
 
 				break;
 			}
@@ -205,7 +204,7 @@ int upload(int mode)
 				char logn[512];
 
 				snprintf(logn, sizeof(logn),
-					"%sdszlog.%d", DDTMP, node);
+					"%s/dszlog.%d", DDTMP, node);
 				unlink(logn);
 
 				if (checkforftp(1, currnode->MULTI_TEMPORARY, logn)) {
@@ -215,7 +214,7 @@ int upload(int mode)
 			}
 			ddprintf(sd[fnamelenstr], maxlen);
 
-			while (1) {
+			for (;;) {
 				DDPut(sd[proceedtransferstr]);
 				udbuf[0] = 0;
 				if (!(Prompt(udbuf, 1, 0)))
@@ -246,7 +245,6 @@ int upload(int mode)
 		}
 		recfiles(currnode->MULTI_TEMPORARY, 
 			 conference()->conf.CONF_PATH);
-
 		timenow = time(0);
 		timeleft = starttimeleft;
 		endtime = timenow + timeleft;
@@ -333,13 +331,14 @@ static int makelc(char *fn)
 
 	snprintf(buf, sizeof(buf), "%s/users/%d/lcfiles.dat", origdir,
 		 user.user_account_id);
-	fd = open(buf, O_WRONLY | O_CREAT, 0644);
-	if (fd < 0) {
+	fd = open(buf, O_WRONLY | O_CREAT, 0666);
+	if (fd == -1) {
 		unlink(fn);
 		return 0;
 	}
+	fsetperm(fd, 0666);
 	lseek(fd, 0, SEEK_END);
-	write(fd, &lc, sizeof(struct lcfile));
+	safe_write(fd, &lc, sizeof(struct lcfile));
 	snprintf(buf, sizeof(buf), "%s/users/%d/lcfiles/%s", origdir, 
 		user.user_account_id, fn);
 	close(fd);
@@ -352,7 +351,8 @@ int genstdiocmdline(char *dest, const char *src,
 {
 	char tbuffer[60];
 	char *s;
-	while (1) {
+
+	for (;;) {
 		if (*src == '%') {
 			if (*(src + 1) == 'A' || *(src + 1) == 'a') {
 				while (arc && *arc)
@@ -410,15 +410,93 @@ int genstdiocmdline(char *dest, const char *src,
 static int update_filelist(char *fname, char *desc)
 {
 	int fd;	
-	if ((fd = open(fname, O_RDWR | O_CREAT | O_APPEND, 0600)) == -1)
+	if ((fd = open(fname, O_RDWR | O_CREAT | O_APPEND, 0666)) == -1)
 		return -1;
-	if (write(fd, desc, strlen(desc)) != strlen(desc)) {
+	fsetperm(fd, 0666);
+	if (safe_write(fd, desc, strlen(desc)) != strlen(desc)) {
 		close(fd);
 		return -1;
 	}
 	return close(fd);
 }
 
+static void upload_ddput(int background_mode, const char *str)
+{
+	if (background_mode)
+		return;
+	DDPut(str);
+}
+
+static int check_archive(int background_checker)
+{
+	char arccmd[1024];
+	char arcbuf[4096];
+	char fname[PATH_MAX];
+	char tmpname[15] = "";
+	FILE *checkd;
+	int fd, arcok = 1;
+
+	if (strlcpy(fname, currnode->MULTI_TEMPORARY,
+			sizeof(fname)) >= sizeof(fname) ||
+		strlcat(fname, "/", sizeof(fname)) >= sizeof(fname) ||
+		strlcat(fname, finname, sizeof(fname)) >= sizeof(fname))
+		return -1;
+	genstdiocmdline(arccmd, arc->ARC_CMD_TEST, fname, 0);
+	strlcpy(tmpname, "/tmp/dd.XXXXXX", sizeof(tmpname));
+	if ((fd = mkstemp(tmpname)) == -1)
+		return -1;
+
+	upload_ddput(background_checker, sd[testingarcstr]);
+	if (arc->ARC_FLAGS & (1L << 4)) {
+		upload_ddput(background_checker, sd[testsepastr]);
+		runstdio(arccmd, fd, 0);
+		upload_ddput(background_checker, sd[testsepastr]);
+	} else {
+		runstdio(arccmd, fd, 3);
+	}
+	if ((checkd = fdopen(fd, "w+")) != NULL) {
+		int arcok = 1;
+		while (fgets(arcbuf, sizeof(arcbuf), checkd) && arcok) {
+			if (*arc->ARC_CORRUPTED1 && 
+				strstr(arcbuf, arc->ARC_CORRUPTED1))
+				arcok = 0;
+			if (*arc->ARC_CORRUPTED2 && 
+				strstr(arcbuf, arc->ARC_CORRUPTED2))
+				arcok = 0;
+			if (*arc->ARC_CORRUPTED3 && 
+				strstr(arcbuf, arc->ARC_CORRUPTED3))
+				arcok = 0;
+		}
+		fclose(checkd);
+	} else {
+		/* since there's no fclose() to close the descriptor,
+		 * we have to close it explicitly.
+		 */
+		close(fd);
+	}
+	unlink(tmpname);
+	return arcok ? 0 : -1;
+}
+
+static FILE *open_no_symlinks(const char *fname)
+{
+	int fd;
+	FILE *fp;
+
+#ifdef HAVE_O_NOFOLLOW
+	fd = open(fname, O_RDONLY | O_NOFOLLOW);
+#else
+	/* FIXME: how to prevent symlink attack if we don't have O_NOFOLLOW? */
+	fd = open(fname, O_RDONLY);
+#endif
+	if (fd == -1)
+		return NULL;
+	if ((fp = fdopen(fd, "r")) == NULL) 
+		close(fd);
+	
+	return fp;
+}
+
 int handleupload(char *upname)
 {
 	char linebuf[20 * 45];
@@ -437,17 +515,24 @@ int handleupload(char *upname)
 	FILE *founddiz = 0;
 	int lcn;
 	struct stat fst;
+	int background_checker;
+
+	background_checker = getpid() != daydream_pid;
 
 	strcpy(finname, upname);
 
 	stat(upname, &st);
-	ul_totulb += st.st_size;
-	ul_totulf++;
+	if (!background_checker) {
+		ul_totulb += st.st_size;
+		ul_totulf++;
+	}
 
 	currtime = time(0);
 	muntm = localtime(&currtime);
 
 	while (checkfilename(finname) == 0) {
+		if (background_checker)
+			return 0;
 		i = 1;
 		if (conference()->conf.CONF_ATTRIBUTES & (1L << 3)) {
 			ddprintf(sd[linvalidstr], finname, " ");
@@ -469,7 +554,7 @@ int handleupload(char *upname)
 
 	getarchiver(finname);
 
-	DDPut("\r                                                                          \r");
+	upload_ddput(background_checker, "\r                                                                          \r");
 
 	if (st.st_size >= 10000000) {
 		snprintf(ulsize, sizeof(ulsize), "%6dk", 
@@ -483,22 +568,25 @@ int handleupload(char *upname)
 	} else {
 		sprintf(daline, "%-12.12s ", finname);
 	}
-	DDPut(daline);
+	upload_ddput(background_checker, daline);
 	if ((conference()->conf.CONF_ATTRIBUTES & (1L << 9)) && dupecheck(finname)) {
 		ul_dupes++;
 		return 1;
 	}
 	if (conference()->conf.CONF_ATTRIBUTES & (1L << 3)) {
 		sprintf(&daline[35], "N--- %s %s", ulsize, ctime(&currtime));
-		DDPut(&daline[35]);
-		DDPut(sd[ullinestr]);
+		upload_ddput(background_checker, &daline[35]);
+		upload_ddput(background_checker, sd[ullinestr]);
 	} else {		
 		sprintf(&daline[13], "N--- %s %2.2d.%2.2d.%2.2d ", ulsize, muntm->tm_mday, muntm->tm_mon + 1, muntm->tm_year % 100);
-		DDPut(&daline[13]);
+		upload_ddput(background_checker, &daline[13]);
 	}
 
-	for (i = 0; i < 45 * 15; i++)
-		linebuf[i] = 0;
+	memset(linebuf, 0, sizeof(linebuf));
+
+	/* background checker accepts only packets with file_id.diz */
+	if ((!arc || !arc->ARC_EXTRACTFILEID[0]) && background_checker)
+		return 0;
 
 	if (arc && arc->ARC_EXTRACTFILEID[0]) {
 		struct dirent *dent;
@@ -506,19 +594,31 @@ int handleupload(char *upname)
 
 		char olddirri[1024];
 		char extdiz[1024];
-		DDPut(sd[checkdizstr]);
-
-		mkdir(".packtmp", 0700);
+		upload_ddput(background_checker, sd[checkdizstr]);
+		
+		/* FIXME: document this fiddling with UIDs
+		 * GIDs and setgid bits
+		 */
 		deldir(".packtmp");
+		mkdir(".packtmp", 0770);
+
+		if (chown(".packtmp", maincfg.CFG_BBSUID,
+			maincfg.CFG_ZIPGID) == -1) 
+			syslog(LOG_WARNING, "cannot change group: %s",
+				strerror(errno));
+		else if (chmod(".packtmp", S_ISGID | 0770) == -1) {
+			syslog(LOG_WARNING, "cannot set SETGID bit: %s",
+				strerror(errno));
+		}
 
 		getcwd(olddirri, 1024);
 		chdir(".packtmp");
 
-		snprintf(buf, sizeof(buf), "%s%s", 
+		snprintf(buf, sizeof(buf), "%s/%s", 
 			currnode->MULTI_TEMPORARY, finname);
 		genstdiocmdline(extdiz, arc->ARC_EXTRACTFILEID, buf, 0);
 
-		runstdio(extdiz, 0, 3);
+		runstdio(extdiz, -1, 3);
 
 		chdir(olddirri);
 
@@ -527,24 +627,34 @@ int handleupload(char *upname)
 				if (!strcasecmp("file_id.diz", dent->d_name)) {
 					snprintf(buf, sizeof(buf), 
 						".packtmp/%s", dent->d_name);
-					founddiz = fopen(buf, "r");
-					stat(buf, &fst);
-					break;
+					if ((founddiz = open_no_symlinks(buf)) != NULL) {
+						fstat(fileno(founddiz), &fst);
+						break;
+					}
 				}
 			}
 			closedir(dh);
 		}
-		DDPut("[27D                              [30D");
+
+		/* no file_id.diz found, so there's no point in
+		 * checking this file at background anymore.
+		 */
+		if (!founddiz && background_checker)
+			return 0;
+
+		upload_ddput(background_checker,
+			"[27D                              [30D");
 
 		if (founddiz) {
 			char kelabuf[256];
 			char dizbuf[45 * 20];
 			int dizcnt = 0;
 
-			DDPut(sd[gotdizstr]);
+			upload_ddput(background_checker, sd[gotdizstr]);
 
 			s = dizbuf;
-			while (dizcnt < 14 && fgetsnolf(kelabuf, 250, founddiz)) {
+			while (dizcnt < 14 && 
+				fgetsnolf(kelabuf, 250, founddiz)) {
 				if (!*kelabuf)
 					continue;
 				stripansi(kelabuf);
@@ -555,6 +665,14 @@ int handleupload(char *upname)
 			}
 			*s = 0;
 			fclose(founddiz);
+
+			/* we don't have controlling terminal, from which
+			 * we could ask additions to file_id.diz, so we
+			 * default to the quick mode.
+			 */
+			if (background_checker)
+				quickmode = 1;
+
 			if (checkcarrier() && !quickmode) {
 				DDPut(sd[dizwaitstr]);
 			      keke:
@@ -593,7 +711,6 @@ int handleupload(char *upname)
 						break;
 					default:
 						goto keke;
-						break;
 					}
 				if (sta != 1)
 					DDPut("\r                                                                      \r");
@@ -609,8 +726,7 @@ int handleupload(char *upname)
 		if (!(Prompt(s, 44, 0)))
 			return 0;
 		if (s[0] == 0 || linecnt == 14) {
-
-			while (1) {
+			for (;;) {
 				DDPut(sd[entrymenustr]);
 				buf[0] = 0;
 				if (!(Prompt(buf, 2, 0)))
@@ -660,7 +776,6 @@ int handleupload(char *upname)
 				} else if (!strcasecmp(buf, "d")) {
 					unlink(finname);
 					return 1;
-					break;
 				}
 			}
 		} else {
@@ -722,56 +837,29 @@ int handleupload(char *upname)
 			maincfg.CFG_HOLDDIR, finname);
 	} else {
 		if ((conference()->conf.CONF_ATTRIBUTES & (1L << 8)) == 0 && arc && *arc->ARC_CMD_TEST) {
-			char arcoutpname[80];
-			char arccmd[200];
-			char arcbuf[4000];
-			FILE *checkd;
-
-			snprintf(buf, sizeof(buf), "%s%s",
- 				currnode->MULTI_TEMPORARY, finname);
-			genstdiocmdline(arccmd, arc->ARC_CMD_TEST, buf, 0);
-			snprintf(arcoutpname, sizeof(arcoutpname), 
-				"%sarcoutp%d", DDTMP, node);
-			unlink(arcoutpname);
-
-			DDPut(sd[testingarcstr]);
-			if (arc->ARC_FLAGS & (1L << 4)) {
-				DDPut(sd[testsepastr]);
-				runstdio(arccmd, arcoutpname, 0);
-				DDPut(sd[testsepastr]);
-			} else {
-				runstdio(arccmd, arcoutpname, 3);
-			}
-			if ((checkd = fopen(arcoutpname, "r"))) {
-				int arcok = 1;
-				while (fgets(arcbuf, 4000, checkd) && arcok) {
-					if (*arc->ARC_CORRUPTED1 && strstr(arcbuf, arc->ARC_CORRUPTED1))
-						arcok = 0;
-					if (*arc->ARC_CORRUPTED2 && strstr(arcbuf, arc->ARC_CORRUPTED2))
-						arcok = 0;
-					if (*arc->ARC_CORRUPTED3 && strstr(arcbuf, arc->ARC_CORRUPTED3))
-						arcok = 0;
-				}
-				fclose(checkd);
-				if (arcok) {
-					if (conference()->conf.CONF_ATTRIBUTES & (1L << 3)) {
-						finalbuf[35] = 'P';
-					} else {
-						finalbuf[13] = 'P';
-					}
-					DDPut(sd[arcokstr]);
+			if (check_archive(background_checker) != -1) {
+				if (conference()->conf.CONF_ATTRIBUTES & (1L << 3)) {
+					finalbuf[35] = 'P';
 				} else {
-					if (arc->ARC_FLAGS & (1L << 3)) {
-						DDPut(sd[arcbrokenstr]);
-						unlink(finname);
-						return 1;
+					finalbuf[13] = 'P';
+				}
+				upload_ddput(background_checker, sd[arcokstr]);
+			} else {
+				if (arc->ARC_FLAGS & (1L << 3)) {
+					/* FIXME: bgchecker shouldn't
+					 * remove files?
+					 */
+					upload_ddput(background_checker,
+						sd[arcbrokenstr]);
+					unlink(finname);
+					return 1;
+				} else {
+					upload_ddput(background_checker,
+						sd[arcbroken2str]);
+					if (conference()->conf.CONF_ATTRIBUTES & (1L << 3)) {
+						finalbuf[35] = 'F';
 					} else {
-						DDPut(sd[arcbroken2str]);
-						if (conference()->conf.CONF_ATTRIBUTES & (1L << 3)) {
-							finalbuf[35] = 'F';
-						} else {
-							finalbuf[13] = 'F';
-						}
+						finalbuf[13] = 'F';
 					}
 				}
 			}
@@ -785,11 +873,17 @@ int handleupload(char *upname)
 		      askbag:
 			snprintf(upath, sizeof(upath), sd[uldeststr], 
 				conference()->conf.CONF_FILEAREAS);
-			DDPut(upath);
+			upload_ddput(background_checker, upath);
 			*db = 0;
-			ps = Prompt(db, 3, 0);
+			/* again, we don't have controlling terminal, so
+			 * let's select the default upload area.
+			 */
+			if (!background_checker) 
+				ps = Prompt(db, 3, 0);
+			else
+				ps = 0;
 			if (!ps || toupper(*db) == 'U' || !checkcarrier()) {
-				sprintf(daline, "%sdata/directory.%3.3d", conference()->conf.CONF_PATH, conference()->conf.CONF_UPLOADAREA);
+				sprintf(daline, "%s/data/directory.%3.3d", conference()->conf.CONF_PATH, conference()->conf.CONF_UPLOADAREA);
 				if (conference()->conf.CONF_ULPATH[0] == '@') {
 					getfreeulp(&conference()->conf.CONF_ULPATH[1], upath, 0);
 					snprintf(buf, sizeof(buf), "%s%s", 
@@ -806,14 +900,14 @@ int handleupload(char *upname)
 				ps = atoi(db);
 				if (ps < 1 || ps > conference()->conf.CONF_FILEAREAS)
 					goto askbag;
-				sprintf(daline, "%sdata/directory.%3.3d", conference()->conf.CONF_PATH, ps);
+				sprintf(daline, "%s/data/directory.%3.3d", conference()->conf.CONF_PATH, ps);
 				getpath(conference(), upath, ps);
 				snprintf(buf, sizeof(buf), "%s%s", 
 					upath, finname);
 			}
 		} else {
 			char upath[1024];
-			sprintf(daline, "%sdata/directory.%3.3d", conference()->conf.CONF_PATH, conference()->conf.CONF_UPLOADAREA);
+			sprintf(daline, "%s/data/directory.%3.3d", conference()->conf.CONF_PATH, conference()->conf.CONF_UPLOADAREA);
 			if (conference()->conf.CONF_ULPATH[0] == '@') {
 				getfreeulp(&conference()->conf.CONF_ULPATH[1], upath, 0);
 				snprintf(buf, sizeof(buf), "%s%s", 
@@ -828,7 +922,7 @@ int handleupload(char *upname)
 
 	runexamine(finname);
 
-	DDPut(sd[savingulstr]);
+	upload_ddput(background_checker, sd[savingulstr]);
 
 	if ((sta == 2 || sta == 4) && !founddiz && arc && *arc->ARC_ADDFILEID) {
 		FILE *fid;
@@ -845,22 +939,22 @@ int handleupload(char *upname)
 				lcn--;
 			}
 			fclose(fid);
-			snprintf(buu, sizeof(buu), "%s%s", 
+			snprintf(buu, sizeof(buu), "%s/%s", 
 				currnode->MULTI_TEMPORARY, finname);
 			genstdiocmdline(adddiz, arc->ARC_ADDFILEID, buu, 0);
-			runstdio(adddiz, 0, 3);
+			runstdio(adddiz, -1, 3);
 			unlink("file_id.diz");
 		}
 	}
 	       	
 	if (update_filelist(daline, finalbuf)) {
-		DDPut(sd[ulerrwritedescstr]);
+		upload_ddput(background_checker, sd[ulerrwritedescstr]);
 		return 1;
 	}
 	
 	newrename(finname, buf);
 
-	DDPut(sd[ulokstr]);
+	upload_ddput(background_checker, sd[ulokstr]);
 
 	if ((!(conference()->conf.CONF_ATTRIBUTES & (1L << 1))) && sta != 3) {
 		addulbytes(st.st_size);
@@ -869,15 +963,19 @@ int handleupload(char *upname)
 	}
 	ul_okf++;
 	ul_okb += st.st_size;
-
+	if (background_checker) {
+		ul_totulb += st.st_size;
+		ul_totulf++;
+	}
 	{
 		char lbuf[100];
 		int logfd;
 		struct DD_UploadLog ddl;
 		snprintf(lbuf, sizeof(lbuf), "%s/logfiles/uploadlog.dat", 
 			origdir);
-		logfd = open(lbuf, O_WRONLY | O_CREAT, 0664);
+		logfd = open(lbuf, O_WRONLY | O_CREAT, 0666);
 		if (logfd != -1) {
+			fsetperm(logfd, 0666);
 			memset((char *) &ddl, 0, sizeof(struct DD_UploadLog));
 			ddl.UL_SLOT = user.user_account_id;
 			strcpy(ddl.UL_FILENAME, finname);
@@ -887,7 +985,7 @@ int handleupload(char *upname)
 			ddl.UL_NODE = node;
 			ddl.UL_CONF = conference()->conf.CONF_NUMBER;
 			lseek(logfd, 0, SEEK_END);
-			write(logfd, &ddl, sizeof(struct DD_UploadLog));
+			safe_write(logfd, &ddl, sizeof(struct DD_UploadLog));
 			close(logfd);
 		}
 	}
@@ -901,7 +999,7 @@ static int getpath(conference_t *con, char *s, int b)
 	FILE *pf;
 	int i;
 
-	snprintf(buf, sizeof(buf), "%sdata/paths.dat", con->conf.CONF_PATH);
+	snprintf(buf, sizeof(buf), "%s/data/paths.dat", con->conf.CONF_PATH);
 	pf = fopen(buf, "r");
 	if (!pf) {
 		if (con->conf.CONF_ULPATH[0] == '@') {
@@ -933,7 +1031,7 @@ static int runexamine(char *filen)
 {
 	char exbuf[1024];
 
-	snprintf(exbuf, sizeof(exbuf), "%sdata/examine.dat", 
+	snprintf(exbuf, sizeof(exbuf), "%s/data/examine.dat", 
 		conference()->conf.CONF_PATH);
 	rundoorbatch(exbuf, filen);
 	return 1;
@@ -971,14 +1069,14 @@ int newrename(char *old, char *newf)
 		
 		oldfd = open(old, O_RDONLY);
 		if (oldfd != -1) {
-			newfd = open(newf, O_WRONLY | O_CREAT | O_TRUNC, 0644);
+			newfd = open(newf, O_WRONLY | O_CREAT | O_TRUNC, 0666);
 			if (newfd != -1) {
 				int cnt;
-
+				fsetperm(newfd, 0666);
 				rwbuf = (char *) xmalloc(60000);
 
 				while ((cnt = read(oldfd, rwbuf, 60000))) {
-					write(newfd, rwbuf, cnt);
+					safe_write(newfd, rwbuf, cnt);
 				}
 				free(rwbuf);
 			}
@@ -990,7 +1088,7 @@ int newrename(char *old, char *newf)
 	return 0;
 }
 
-int newcopy(char *old, char *newf)
+int newcopy(const char *old, const char *newf)
 {
 	int oldfd;
 	int newfd;
@@ -998,14 +1096,14 @@ int newcopy(char *old, char *newf)
 
 	oldfd = open(old, O_RDONLY);
 	if (oldfd != -1) {
-		newfd = open(newf, O_WRONLY | O_CREAT | O_TRUNC, 0644);
+		newfd = open(newf, O_WRONLY | O_CREAT | O_TRUNC, 0666);
 		if (newfd != -1) {
 			int cnt;
-
+			fsetperm(newfd, 0666);
 			rwbuf = (char *) xmalloc(60000);
 
 			while ((cnt = read(oldfd, rwbuf, 60000))) {
-				write(newfd, rwbuf, cnt);
+				safe_write(newfd, rwbuf, cnt);
 			}
 			free(rwbuf);
 		}
@@ -1045,7 +1143,7 @@ static int checkfilename(char *name)
 
 }
 
-int getarchiver(char *file)
+int getarchiver(const char *file)
 {
 	arc = arcs;
 
@@ -1067,7 +1165,7 @@ void cleantemp(void)
 
 	if ((dirfd = opendir(currnode->MULTI_TEMPORARY))) {
 		while ((ent = readdir(dirfd))) {
-			snprintf(tempbuf, sizeof(tempbuf), "%s%s", 
+			snprintf(tempbuf, sizeof(tempbuf), "%s/%s", 
 				currnode->MULTI_TEMPORARY, ent->d_name);
 			unlink(tempbuf);
 		}
@@ -1077,33 +1175,35 @@ void cleantemp(void)
 	}
 }
 
-void deldir(const char *dir)
+int deldir(const char *dir)
 {
 	DIR *dirfd;
 	struct dirent *ent;
 	char tempbuf[1024];
 
-	if ((dirfd = opendir(dir))) {
-		while ((ent = readdir(dirfd))) {
-			struct stat st;
+	if ((dirfd = opendir(dir)) == NULL)
+		return -1;
+		
+	while ((ent = readdir(dirfd))) {
+		struct stat st;
 
-			if ((ent->d_name[0] == '.' && ent->d_name[1] == 0) || 
-			    (ent->d_name[0] == '.' && ent->d_name[1] == '.' && 
-			     ent->d_name[2] == 0))
-				continue;
-			snprintf(tempbuf, sizeof(tempbuf), "%s/%s",
-				dir, ent->d_name);
-			stat(tempbuf, &st);
+		if ((ent->d_name[0] == '.' && ent->d_name[1] == 0) || 
+		    (ent->d_name[0] == '.' && ent->d_name[1] == '.' && 
+		     ent->d_name[2] == 0))
+			continue;
+		snprintf(tempbuf, sizeof(tempbuf), "%s/%s",
+			dir, ent->d_name);
+		stat(tempbuf, &st);
 
-			if (S_ISDIR(st.st_mode)) {
-				deldir(tempbuf);
-				rmdir(tempbuf);
-			} else {
-				unlink(tempbuf);
-			}
+		if (S_ISDIR(st.st_mode)) {
+			deldir(tempbuf);
+			rmdir(tempbuf);
+		} else {
+			unlink(tempbuf);
 		}
-		closedir(dirfd);
 	}
+	closedir(dirfd);
+	return 0;
 }
 
 int freespace(void)
@@ -1128,13 +1228,23 @@ int freespace(void)
 
 static uint64_t get_free_space(const char *path)
 {
+#ifdef HAVE_STATVFS 
+	struct statvfs freest;
+#else
 	struct statfs freest;
+#endif
 
+#ifdef HAVE_STATVFS
+	if (statvfs(path, &freest) == -1) {
+		syslog(LOG_WARNING, "cannot statfs(\"%s\"...): %m", path);
+		return 0;
+	}
+#else
 	if (statfs(path, &freest) == -1) {
 		syslog(LOG_WARNING, "cannot statfs(\"%s\"...): %m", path);
 		return 0;
 	}
-
+#endif
 	return ((uint64_t) freest.f_bsize * (uint64_t) freest.f_bavail);
 }
 
@@ -1202,22 +1312,24 @@ static int dupecheck(char *fname)
 			}
 		}
 
-		multibackspace(strlena(sd[dckstr]));
-		iterator_discard(iterator);
-		return 0;
+
 	}
+	multibackspace(strlena(sd[dckstr]));
 	iterator_discard(iterator);
 	
 	return 0;
 }
 
+/* prior to 2.14.6, MULTI_TEMPORARY was expected to end with 
+ * a trailing slash. Therefore we have to supply the slash.
+ */
 int maketmplist(void)
 {
 	char buf[1024];
 	FILE *fi;
 	struct DayDream_Multinode *cn;
 
-	snprintf(buf, sizeof(buf), "%stmplist%d", DDTMP, node);
+	snprintf(buf, sizeof(buf), "%s/tmplist%d", DDTMP, node);
 	fi = fopen(buf, "w");
 	if (fi) {
 		for (cn = nodes; cn->MULTI_NODE; cn++) {
@@ -1230,7 +1342,7 @@ int maketmplist(void)
 					j--;
 					if (i != node) {
 						fprintf(fi, cn->MULTI_TEMPORARY, i);
-						fputc('\n', fi);
+						fputs("/\n", fi);
 					}
 					i++;
 				}
@@ -1243,14 +1355,14 @@ int maketmplist(void)
 					j--;
 					if (i != node) {
 						fprintf(fi, cn->MULTI_TEMPORARY, i);
-						fputc('\n', fi);
+						fputs("/\n", fi);
 					}
 					i++;
 				}
 			} else if (cn->MULTI_NODE != 252) {
 				if (cn->MULTI_NODE != node) {
 					fprintf(fi, cn->MULTI_TEMPORARY, cn->MULTI_NODE);
-					fputc('\n', fi);
+					fputs("\n", fi);
 				}
 			}
 		}
@@ -1278,11 +1390,17 @@ static int checkbgdone(int mode)
 			return 0;
 		rere = read(efdii, &bi, sizeof(struct bgcheckinfo));
 		close(efdii);
-		tries++;
-		if (rere != sizeof(struct bgcheckinfo) && (tries < 50))
-			 goto doom;
+
+		if (rere != sizeof(struct bgcheckinfo)) {
+			usleep(100000);
+			if (++tries < 50)
+				goto doom;
+			log_error(__FILE__, __LINE__, 
+				"background checker failed");
+		} else 
+			user = bi.bguser;
+
 		unlink(buf);
-		user = bi.bguser;
 		return 1;
 	} else {
 		bi.bguser = user;
@@ -1292,11 +1410,15 @@ static int checkbgdone(int mode)
 		bi.okfiles = ul_okf;
 		bi.okbytes = ul_okb;
 
-		efdii = open(buf, O_WRONLY | O_CREAT, 0755);	/* If this fails -> trouble */
-		if (efdii < 0) {
+		efdii = open(buf, O_WRONLY | O_CREAT, 0666);	/* If this fails -> trouble */
+		if (efdii == -1) {
+			log_error(__FILE__, __LINE__,
+				"open() failed: %s (%d)",
+				strerror(errno), errno);
 			return 0;
 		}
-		write(efdii, &bi, sizeof(struct bgcheckinfo));
+		fsetperm(efdii, 0666);
+		safe_write(efdii, &bi, sizeof(struct bgcheckinfo));
 		close(efdii);
 	}
 	return 1;
@@ -1319,8 +1441,6 @@ int initbgchecker(void)
 
 	ul_totulf = ul_dupes = ul_totulb = ul_okf = ul_okb = 0;
 
-	signal(SIGUSR1, bghan);
-
 	bgrun = fork();
 	if (bgrun < 0) {
 		bgrun = 0;
@@ -1329,21 +1449,22 @@ int initbgchecker(void)
 	if (bgrun > 0)
 		return 0;
 
+	signal(SIGUSR1, bghan);
+
 	bgmode = 1;
 	bgdone = 0;
 	carrier = 0;
 
 	/* get rid of filedescriptors */
+	finalize_console();
 	close(serhandle);
-	close(conin);
-	close(conout);
 	close(dsockfd);
-	close(0);
-	close(1);
-	close(2);
-	
+	close(STDIN_FILENO);
+	close(STDOUT_FILENO);
+	close(STDERR_FILENO);
+
 	while (!bgdone) {
-		snprintf(buf, sizeof(buf), "%sdszlog.%d", DDTMP, node);
+		snprintf(buf, sizeof(buf), "%s/dszlog.%d", DDTMP, node);
 		log = fopen(buf, "r");
 		if (log) {
 			for (cnt = 0; cnt < lastf; cnt++) {
