@@ -76,70 +76,60 @@ struct flagstr fl[] =
 
 static void edit_new_users(void)
 {
-	struct userbase *ub = NULL;
+	struct userbase tmpuser;
 	int account_id = 0, retcode;
 	
 	for (;; account_id++) {
-		g_free(ub);
-		ub = NULL;
-		
-		ub = getubentbyid(account_id);
-		if (!ub)
+		if (getubentbyid(account_id, &tmpuser) == -1)
 			return;
-		
-		if ((ub->user_toggles & UBENT_STAT_MASK) != UBENT_STAT_NEW)
+		if ((tmpuser.user_toggles & UBENT_STAT_MASK) != UBENT_STAT_NEW)
 			continue;
 
-		retcode = uedask(ub);
-		if (retcode == 2) {
-			g_free(ub);
+		retcode = uedask(&tmpuser);
+		if (retcode == 2) 
 			return;
-		}
+
 		if (retcode == 0)
 			continue;
 		
-		if (!usereditor(ub))
+		if (!usereditor(&tmpuser))
 			continue;
 
-		if (writeubent(ub)) 
+		/* FIXME: do something more useful than just a naive panic */
+		if (writeubent(&tmpuser) == -1)  
 			panic("can't save userbase in edit_new_users()");
-		if (ub->user_account_id == user.user_account_id)
-			memcpy(&user, ub, sizeof(struct userbase));
+		if (tmpuser.user_account_id == user.user_account_id)
+			memcpy(&user, &tmpuser, sizeof(struct userbase));
 	}		
 }
 
 static void search_and_edit(const char *sstring)
 {
-	struct userbase *ub = NULL;
+	struct userbase tmpuser;
 	int account_id = 0, retcode;
 	
 	for (;; account_id++) {
-		g_free(ub);
-		ub = NULL;
-		
-		ub = getubentbyid(account_id);
-		if (!ub)
+		if (getubentbyid(account_id, &tmpuser) == -1)
 			return;
-		if ((ub->user_toggles & UBENT_STAT_MASK) == UBENT_STAT_DELETED)
+		if ((tmpuser.user_toggles & UBENT_STAT_MASK) == 
+			UBENT_STAT_DELETED)
 			continue;
-		if (!wildcmp(ub->user_realname, sstring) &&
-		    !wildcmp(ub->user_handle, sstring))
+		if (!wildcmp(tmpuser.user_realname, sstring) &&
+		    !wildcmp(tmpuser.user_handle, sstring))
 			continue;
 
-		retcode = uedask(ub);
-		if (retcode == 2) {
-			g_free(ub);
+		retcode = uedask(&tmpuser);
+		if (retcode == 2) 
 			return;
-		}
 		if (!retcode)
 			continue;
-		if (!usereditor(ub))
+		if (!usereditor(&tmpuser))
 			continue;
-		if (writeubent(ub))
+		/* FIXME: do something more useful than just a naive panic */
+		if (writeubent(&tmpuser) == -1)
 			panic("can't save userbase in search_and_edit()");
-		
-		if (ub->user_account_id == user.user_account_id)
-			memcpy(&user, ub, sizeof(struct userbase));
+		if (tmpuser.user_account_id == user.user_account_id)
+			memcpy(&user, &tmpuser, sizeof(struct userbase));
 	}	
 }	
 
@@ -196,7 +186,7 @@ static int uedask(struct userbase *auser)
 	
 	ddprintf("\r                                                      \r[0m%s [32m/ [0m%s [35m([36mYes[35m/[36mno[35m/[36mquit[35m)[0m: ", auser->user_realname, auser->user_handle);
 
-	while (1) {
+	for (;;) {
 		if ((hot = HotKey(0))) {
 			if (hot == 'q' || hot == 'Q') {
 				DDPut("Quit!");
@@ -364,7 +354,8 @@ int usereditor(struct userbase *euser)
 				return 0;
 			strupr(uebuf);
 			MDInit(&context);
-			MDUpdate(&context, uebuf, strlen(uebuf));
+			MDUpdate(&context, (unsigned char *) uebuf, 
+				strlen(uebuf));
 			MDFinal(euser->user_password, &context);
 		} else if (!strcasecmp(uebuf, "g")) {
 			uemove(19, 8);
@@ -490,9 +481,10 @@ int usereditor(struct userbase *euser)
 					continue;
 			}
 			if (seceditor(&secl)) {
-				secfd = open(sbuffer, O_WRONLY | O_CREAT, 0644);
+				secfd = open(sbuffer, O_WRONLY | O_CREAT, 0666);
 				if (secfd != -1) {
-					write(secfd, &secl, sizeof(struct DD_Seclevel));
+					fsetperm(secfd, 0666);
+					safe_write(secfd, &secl, sizeof(struct DD_Seclevel));
 					close(secfd);
 				}
 				getsec();
@@ -500,7 +492,7 @@ int usereditor(struct userbase *euser)
 			redraw = 1;
 		} else if (!strcasecmp(uebuf, "x")) {
 			DDPut("[A");
-			while (1) {
+			for (;;) {
 				DDPut("[33mSelect preset: [36m1[0m-[36m255[0m, [36mL[0mist or [36mQ[0m)uit:               [14D");
 				uebuf[0] = 0;
 				if (!(Prompt(uebuf, 3, 0)))
@@ -575,7 +567,7 @@ static int seceditor(struct DD_Seclevel *sech)
 			cf++;
 		}
 
-		while (1) {
+		for (;;) {
 			DDPut("[22;1H[0mSelect field to be altered ([36mSA[0mve, [36mAB[0mort or [36mVI[0mew):     [4D");
 			*uebuf = 0;
 			if (!(Prompt(uebuf, 3, 0)))
