@@ -2,61 +2,58 @@
 #include <stdio.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <dd.h>
 #include <math.h>
-
 #include <pwd.h>
 #include <grp.h>
-
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
 
-char *findtxt(char *, const char*);
-char *cfgcopy2(char *, int, const char *);
-char cfgchar2(char *);
-int cfgflag2(const char *, const char *);
-int cfgint2(const char *);
-int cfgconf2(char *);
-char *nextspace(char *);
-char *cfgcopy(char *, int);
-char *findstrinblock(char *);
-void cfgerror(char *);
+#include <dd.h>
 
-int mymkdir(char *);
-char * cfgfnadd3(unsigned short *, unsigned short *, unsigned short *, unsigned short *, char *);
-void removespaces(char *);
+static char *findtxt(char *, const char*);
+static char *cfgcopy2(char *, int, const char *);
+static char cfgchar2(char *);
+static int cfgflag2(const char *, const char *);
+static int cfgint2(const char *);
+static int cfgconf2(char *);
+static char *nextspace(char *);
+static char *cfgcopy(char *, int);
+static char *findstrinblock(const char *);
+static void cfgerror(const char *);
 
-char homedir[512];
-char group[512];
-char user[512];
-char perm[512];
-char olddir[1024];
+static int mymkdir(char *, mode_t);
+static char * cfgfnadd3(unsigned short *, unsigned short *, 
+	unsigned short *, unsigned short *, char *);
+static void removespaces(char *);
 
-char gerror[1024];
+static char homedir[512];
+static char group[512];
+static char zipgroup[512];
+static char user[512];
+static char olddir[1024];
 
-gid_t gid=-1;
-uid_t uid=-1;
-mode_t mode;
-mode_t dmode;
+static char gerror[1024];
+
+static gid_t zipgid = -1;
+static gid_t gid = -1;
+static uid_t uid = -1;
  
-char *src;
-char *ecfg(char *, const char *);
-void backtoold(void);
+static char *blockstart;
+static char *src;
+static char *ecfg(char *, const char *);
+static void backtoold(void);
 
-void backtoold(void)
+static void backtoold(void)
 {
 	chdir(olddir);
 }
 
-char *blockstart;
-
-
-char *endblock(void)
+static char *endblock(void)
 {
 	char *buffb;
 	buffb=blockstart;
-	while(1)
-	{
+	for (;;) {
 		while(*buffb!=10) {
 			if (buffb==0) return 0;
 			buffb++;
@@ -68,8 +65,13 @@ char *endblock(void)
 			return buffb;
 		}
 	}
+}
 
-
+static void remove_trailing_slash(char *pname)
+{
+	char *p;
+	for (p = pname + strlen(pname); 
+		p > pname && p[-1] == '/'; *--p = '\0');
 }
 
 int main(int argc, char *argv[])
@@ -88,27 +90,22 @@ int main(int argc, char *argv[])
 	struct DayDream_Multinode node;
 	struct DayDream_Protocol proto;
 	struct DD_Seclevel sec;
-	uint8_t  selcfg[2056];
+	uint8_t selcfg[2056];
 	int dodirs=1;
-		
+	struct passwd *pw;
+	struct group *gr;	
 	int i;
 	int j;
 	char *s;
 	
-	for (i=0;i<2056;i++) selcfg[i]=0;
-	
-	s=(char *)&maincfg;
-	for(i=0; i < sizeof(struct DayDream_MainConfig)-1 ; i++) {
-		s[i]=0;
-	}
+	memset(selcfg, 0, sizeof(selcfg));
+	memset(&maincfg, 0, sizeof(maincfg));
 	
 	if (argc < 2) {
 		printf("usage: ddcfg [configfile] <-d>\n\n");
 		exit(0);
 	}
 
-	umask(0600);
-	
 	infile = open(argv[1],O_RDONLY);
 	if (infile == -1) {
 		printf("Unknown configfile!\n\n");
@@ -132,22 +129,6 @@ int main(int argc, char *argv[])
 		cfgcopy(homedir,500);
 	}
 
-	src=ecfg(inmem,"FILEPERMISSION ");
-	if (!src) {
-		*perm=0;
-	} else {
-		cfgcopy(perm,500);
-		sscanf(perm,"%o",(unsigned int *)&mode);
-	}
-
-	src=ecfg(inmem,"DIRPERMISSION ");
-	if (!src) {
-		*perm=0;
-	} else {
-		cfgcopy(perm,500);
-		sscanf(perm,"%o",(unsigned int *)&dmode);
-	}
-
 	src=ecfg(inmem,"OWNERUSER ");
 	if (!src) {
 		*user=0;
@@ -162,35 +143,41 @@ int main(int argc, char *argv[])
 		cfgcopy(group,500);
 	}
 
+	src=ecfg(inmem,"ZIPGROUP ");
+	if (!src) {
+		*zipgroup=0;
+	} else {
+		cfgcopy(zipgroup,500);
+	}
+
 	if (*homedir) {
 		getcwd(olddir,1024);
 		chdir(homedir);
 		atexit(backtoold);
 	}
 
-	if (*user) {
-		struct passwd *pw;
-		setpwent();
-		while ((pw=getpwent())) {
-			if (!(strcmp(user,pw->pw_name))) {
-				uid=pw->pw_uid;
-				break;
-			}
-		}
-		endpwent();
-	}
+	if (*user == 0 || (pw = getpwnam(user)) == NULL) {
+		fputs("OWNERUSER missing.\n", stderr);
+		exit(1);
+	} else
+		uid = pw->pw_uid;
 
-	if (*group) {
-		struct group *gr;
-		setgrent();
-		while ((gr=getgrent())) {
-			if (!(strcmp(group,gr->gr_name))) {
-				gid=gr->gr_gid;
-				break;
-			}
-		}
-		endgrent();
-	}
+	if (*group == 0 || (gr = getgrnam(group)) == NULL) {
+		fputs("OWNERGROUP missing.\n", stderr);
+		exit(1);
+	} else
+		gid = gr->gr_gid;
+
+	if (*zipgroup == 0 || (gr = getgrnam(zipgroup)) == NULL) {
+		fputs("ZIPGROUP missing.\n", stderr);
+		exit(1);
+	} else if (gr->gr_gid == gid) {
+		fputs("ZIPGROUP equals to OWNERGROUP. This is a security risk.\n", stderr);
+		fputs("Please read \"docs/SECURITY\".\n", stderr);
+		exit(1);
+	} else
+		zipgid = gr->gr_gid;
+			
 
 	src=findtxt(inmem,"!DAYDREAM.DAT");
 
@@ -205,7 +192,8 @@ int main(int argc, char *argv[])
 	cfgcopy2(maincfg.CFG_SYSOPNAME,26,"SYSOP");
 	cfgcopy2(maincfg.CFG_SYSOPFIRST,26,"SYSOPFIRST");
 	cfgcopy2(maincfg.CFG_SYSOPLAST,26,"SYSOPLAST");
-	cfgcopy2(maincfg.CFG_CHATDLPATH,41,"CHATDLPATH");
+	cfgcopy2(maincfg.CFG_CHATDLPATH, 41, "CHATDLPATH");
+	remove_trailing_slash(maincfg.CFG_CHATDLPATH);
 	cfgcopy2(maincfg.CFG_SYSTEMPW,16,"SYSTEM_PW");
 	cfgcopy2(maincfg.CFG_PYTHON,54,"PYTHON");
 	cfgcopy2(maincfg.CFG_NEWUSERPW,16,"NEWUSER_PW");
@@ -217,17 +205,14 @@ int main(int argc, char *argv[])
 	if (cfgflag2("Y","ASK_REASON")) maincfg.CFG_FLAGS |= (1L << 0);
 	if (cfgflag2("H","NAMEMODE")) maincfg.CFG_FLAGS |= (1L << 1);
 	if (cfgflag2("O","LOCATION")) maincfg.CFG_FLAGS |= (1L << 2);
-	if (cfgflag2("Y","DISABLEWILDS")) maincfg.CFG_FLAGS |= (1L << 3);
-	if (cfgflag2("Y","QUESTIONS")) maincfg.CFG_FLAGS |= (1L << 4);
-	if (cfgflag2("Y","QUICKLINEED")) maincfg.CFG_FLAGS |= (1L << 6);
 	if (cfgflag2("Y","ASKHANDLE1ST")) maincfg.CFG_FLAGS |= (1L << 7);
+	if (cfgflag2("Y","ASKDISPLAY")) 
+		maincfg.CFG_FLAGS |= (1L << 16);
 	cfgcopy2(maincfg.CFG_COLORSYSOP,11,"SYSOP_COL");
 	cfgcopy2(maincfg.CFG_COLORUSER,11,"USER_COL");
-	cfgcopy2(maincfg.CFG_SHELLPW,16,"SHELL_PW");
 	cfgcopy2(maincfg.CFG_ALIENS,40,"ALIENS");
 	maincfg.CFG_LINEEDCHAR=cfgchar2("LINE_EDCOM");
 	cfgcopy2(maincfg.CFG_FSEDCOMMAND,70,"FSED");
-	maincfg.CFG_CHATMODE=cfgchar2("CHATMODE");
 	maincfg.CFG_LOCALSCREEN=cfgint2("SCREENLENGTH");
 	cfgcopy2(maincfg.CFG_FREEDLLINE,100,"FREEDLLINE");
 	maincfg.CFG_COSYSOPLEVEL=cfgint2("COSYSOPACCESS");
@@ -239,8 +224,19 @@ int main(int argc, char *argv[])
 	maincfg.CFG_LOCAL1ST=cfgint2("LOCALNODE1ST");
 	maincfg.CFG_LOCALMAX=cfgint2("LOCALNODEMAX");
 	if (cfgflag2("Y","ALLOW2LOGINS")) maincfg.CFG_FLAGS |= (1L << 8);
-	if (cfgflag2("Y","ASKNEWUSER")) maincfg.CFG_FLAGS |= (1L << 9);
-	
+
+	switch (cfgchar2("ASKNEWUSER")) {
+	case 'N':
+		break;
+	case 'Y':
+		maincfg.CFG_FLAGS |= (1L << 9);
+		break;
+	case 'D':
+		maincfg.CFG_FLAGS |= (1L << 17);
+		break;
+	}	
+	if (cfgflag2("Y","LOGINSTYLE")) maincfg.CFG_FLAGS |= (1L << 18);
+		
 	if (cfgflag2("Y","DEF_EXPERT")) maincfg.CFG_DEFAULTS |= (1L<<4);
 	switch(cfgchar2("DEF_MAILSCAN"))
 	{
@@ -282,14 +278,16 @@ int main(int argc, char *argv[])
 	maincfg.CFG_MAXFTPUSERS=cfgint2("MAXFTPUSERS");
 	maincfg.CFG_FLOODKILLTRIG=cfgint2("FLOODKILLTRIG");
 
-	maincfg.CFG_BBSUID=uid;
-	maincfg.CFG_BBSGID=gid;
+	maincfg.CFG_BBSUID = uid;
+	maincfg.CFG_BBSGID = gid;
+	maincfg.CFG_ZIPGID = zipgid;
 
-	outfile = open("data/daydream.dat",O_CREAT|O_TRUNC|O_RDWR,mode);
+	outfile = open("data/daydream.dat", O_CREAT|O_TRUNC|O_RDWR, 0640);
 	if (outfile == -1) {
 		perror("data/daydream.dat");
 		exit(0);
 	}
+	fchmod(outfile, 0640);
 		
 	write(outfile,&maincfg,sizeof(struct DayDream_MainConfig));
 	close(outfile);
@@ -302,11 +300,12 @@ int main(int argc, char *argv[])
 		exit(0);
 	}
 	
-	outfile = open("data/conferences.dat",O_CREAT|O_TRUNC|O_RDWR,mode);
+	outfile = open("data/conferences.dat", O_CREAT|O_TRUNC|O_RDWR, 0640);
 	if (outfile == -1) {
 		perror("data/conferences.dat");
 		exit(0);
 	}
+	fchmod(outfile, 0640);
 nextconf:
 	strcpy(gerror,"CONFERENCES.DAT");
 
@@ -320,16 +319,17 @@ nextconf:
 
 	cfgcopy2(conf.CONF_NAME,40,"CONF_NAME");
 	cfgcopy2(conf.CONF_PATH,40,"CONF_PATH");
+	remove_trailing_slash(conf.CONF_PATH);
 	if (dodirs) {
 		char tbu[600];
 
-		mymkdir(conf.CONF_PATH);
-		sprintf(tbu,"%sdata",conf.CONF_PATH);
-		mymkdir(tbu);
-		sprintf(tbu,"%smessages",conf.CONF_PATH);
-		mymkdir(tbu);
-		sprintf(tbu,"%sdisplay",conf.CONF_PATH);
-		mymkdir(tbu);
+		mymkdir(conf.CONF_PATH, 0750);
+		sprintf(tbu,"%s/data",conf.CONF_PATH);
+		mymkdir(tbu, 0770);
+		sprintf(tbu,"%s/messages",conf.CONF_PATH);
+		mymkdir(tbu, 0750);
+		sprintf(tbu,"%s/display",conf.CONF_PATH);
+		mymkdir(tbu, 0750);
 	}
 	conf.CONF_FILEAREAS=cfgint2("CONF_FILEAREAS");
 	conf.CONF_UPLOADAREA=cfgint2("CONF_UPLOADSTO");
@@ -337,7 +337,7 @@ nextconf:
 	conf.CONF_COMMENTAREA=cfgint2("CONF_COMMENTS");
 	cfgcopy2(conf.CONF_ULPATH,50,"CONF_ULPATH");
 	if (dodirs && *conf.CONF_ULPATH) {
-		mymkdir(conf.CONF_ULPATH);
+		mymkdir(conf.CONF_ULPATH, 0770);
 	}
 	cfgcopy2(conf.CONF_NEWSCANAREAS,30,"CONF_NSAREAS");
 	if (cfgflag2("Y","CONF_DEFFSCAN")) {
@@ -387,8 +387,8 @@ dobase:
 	if (dodirs) {
 		char tbu[600];
 
-		sprintf(tbu,"%smessages/base%3.3d",conf.CONF_PATH,base.MSGBASE_NUMBER);
-		mymkdir(tbu);
+		sprintf(tbu, "%s/messages/base%3.3d", conf.CONF_PATH, base.MSGBASE_NUMBER);
+		mymkdir(tbu, 0770);
 	}
 	base.MSGBASE_FN_FLAGS=cfgchar2("BASE_TYPE");
 	base.MSGBASE_MSGLIMIT=cfgint2("BASE_MAXMSGS");
@@ -426,11 +426,12 @@ confsdone:
 
 	chown("data/conferences.dat",uid,gid);
 	
-	outfile = open("data/selected.dat",O_CREAT|O_TRUNC|O_RDWR,mode);
+	outfile = open("data/selected.dat", O_CREAT|O_TRUNC|O_RDWR, 0640);
 	if (outfile == -1) {
 		perror("data/selected.dat");
 		exit(0);
 	}
+	fchmod(outfile, 0640);
 
 	write(outfile,&selcfg,2056);
 	close(outfile);
@@ -443,11 +444,13 @@ confsdone:
 		exit(0);
 	}
 
-	outfile = open("data/externalcommands.dat",O_CREAT|O_TRUNC|O_RDWR,mode);
+	outfile = open("data/externalcommands.dat", 
+		O_CREAT|O_TRUNC|O_RDWR, 0640);
 	if (outfile == -1) {
 		perror("data/externalcommands.dat");
 		exit(0);
 	}
+	fchmod(outfile, 0640);
 nextdoor:
 
 	strcpy(gerror,"EXTERNALCOMMANDS.DAT");
@@ -462,6 +465,7 @@ nextdoor:
 	cfgcopy2(extcom.EXT_COMMAND,87,"DOOR_EXECUTE");
 	extcom.EXT_CONF1=cfgconf2("DOOR_CONFS1");
 	extcom.EXT_CONF2=cfgconf2("DOOR_CONFS2");
+	cfgcopy2(extcom.EXT_PASSWD, 16, "DOOR_PASSWD");
 	
 	write(outfile,&extcom,sizeof(struct DD_ExternalCommand));
 	src=endblock();
@@ -481,11 +485,13 @@ nextdoor:
 		exit(0);
 	}
 
-	outfile = open("data/archivers.dat",O_CREAT|O_TRUNC|O_RDWR,mode);
+	outfile = open("data/archivers.dat", 
+		O_CREAT|O_TRUNC|O_RDWR, 0640);
 	if (outfile == -1) {
 		perror("data/archivers.dat");
 		exit(0);
 	}
+	fchmod(outfile, 0640);
 nextarc:
 
 	strcpy(gerror,"ARCHIVERS.DAT");
@@ -495,7 +501,6 @@ nextarc:
 	}
 	cfgcopy2(arc.ARC_NAME,21,"ARC_NAME");
 	sprintf(gerror,"ARCHIVERS.DAT: Archiver %s\n",arc.ARC_NAME);
-	if (cfgflag2("D","ARC_TYPE")) arc.ARC_FLAGS |= (1L<<0);
 	cfgcopy2(arc.ARC_PATTERN,20,"ARC_PATTERN");
 	cfgcopy2(arc.ARC_CMD_TEST,80,"ARC_TEST");
 	cfgcopy2(arc.ARC_CORRUPTED1,16,"ARC_FAIL1");
@@ -504,7 +509,6 @@ nextarc:
 	cfgcopy2(arc.ARC_EXTRACTFILEID,80,"ARC_FIDEXTRACT");
 	cfgcopy2(arc.ARC_ADDFILEID,80,"ARC_FIDADD");
 	cfgcopy2(arc.ARC_VIEW,80,"ARC_VIEW");
-	if (cfgflag2("Y","ARC_OFFLINE")) arc.ARC_FLAGS |= (1L<<1);
 	if (cfgflag2("Y","ARC_GETDATE")) arc.ARC_FLAGS |= (1L<<2);
 	if (cfgflag2("Y","ARC_DELETECOR")) arc.ARC_FLAGS |= (1L<<3);
 	if (cfgflag2("Y","ARC_DISPLAY")) arc.ARC_FLAGS |= (1L<<4);
@@ -528,11 +532,12 @@ nextarc:
 		exit(0);
 	}
 
-	outfile = open("data/display.dat",O_CREAT|O_TRUNC|O_RDWR,mode);
+	outfile = open("data/display.dat", O_CREAT|O_TRUNC|O_RDWR, 0640);
 	if (outfile == -1) {
 		perror("data/display.dat");
 		exit(0);
 	}
+	fchmod(outfile, 0640);
 nextdisp:
 
 	strcpy(gerror,"DISPLAY.DAT");
@@ -545,6 +550,7 @@ nextdisp:
 	sprintf(gerror,"DISPLAY.DAT, ID: %d",disp.DISPLAY_ID);
 
 	cfgcopy2(disp.DISPLAY_PATH,9,"DPL_PATH");
+	remove_trailing_slash(disp.DISPLAY_PATH);
 	if (cfgflag2("Y","DPL_ANSI")) disp.DISPLAY_ATTRIBUTES |= (1L<<0);
 	if (cfgflag2("Y","DPL_INCONV")) disp.DISPLAY_ATTRIBUTES |= (1L<<1);
 	if (cfgflag2("Y","DPL_OUTCONV")) disp.DISPLAY_ATTRIBUTES |= (1L<<2);
@@ -573,11 +579,12 @@ nextdisp:
 		exit(0);
 	}
 
-	outfile = open("data/multinode.dat",O_CREAT|O_TRUNC|O_RDWR,mode);
+	outfile = open("data/multinode.dat", O_CREAT|O_TRUNC|O_RDWR, 0640);
 	if (outfile == -1) {
 		perror("data/multinode.dat");
 		exit(0);
 	}
+	fchmod(outfile, 0640);
 nextnode:
 
 	strcpy(gerror,"MULTINODE.DAT");
@@ -609,6 +616,7 @@ nextnode:
 	if (cfgflag2("Y","MNODE_HIDECALL")) node.MULTI_OTHERFLAGS |= (1L<<4);
 	if (cfgflag2("Y","MNODE_NOPASSWD")) node.MULTI_OTHERFLAGS |= (1L<<5);
 	cfgcopy2(node.MULTI_TEMPORARY,33,"MNODE_TEMP");
+	remove_trailing_slash(node.MULTI_TEMPORARY);
 	cfgcopy2(node.MULTI_TTYNAME,20,"MNODE_TTYNAME");
 	node.MULTI_TTYTYPE=cfgint2("MNODE_TTYTYPE");
 	node.MULTI_TTYSPEED=cfgint2("MNODE_TTYSPEED");
@@ -631,11 +639,12 @@ nextnode:
 		exit(0);
 	}
 
-	outfile = open("data/security.dat",O_CREAT|O_TRUNC|O_RDWR,mode);
+	outfile = open("data/security.dat", O_CREAT|O_TRUNC|O_RDWR, 0640);
 	if (outfile == -1) {
 		perror("data/security.dat");
 		exit(0);
 	}
+	fchmod(outfile, 0640);
 nextsec:
 	strcpy(gerror,"SECURITY.DAT");
 
@@ -712,11 +721,12 @@ nextsec:
 		exit(0);
 	}
 
-	outfile = open("data/access.dat",O_CREAT|O_TRUNC|O_RDWR,mode);
+	outfile = open("data/access.dat", O_CREAT|O_TRUNC|O_RDWR, 0640);
 	if (outfile == -1) {
 		perror("data/access.dat");
 		exit(0);
 	}
+	fchmod(outfile, 0640);
 nextpreset:
 
 	s=(char *)&preset;
@@ -752,11 +762,12 @@ nextpreset:
 		exit(0);
 	}
 
-	outfile = open("data/protocols.dat",O_CREAT|O_TRUNC|O_RDWR,mode);
+	outfile = open("data/protocols.dat", O_CREAT|O_TRUNC|O_RDWR, 0640);
 	if (outfile == -1) {
 		perror("data/protocols.dat");
 		exit(0);
 	}
+	fchmod(outfile, 0640);
 nextproto:
 
 	strcpy(gerror,"PROTOCOLS.DAT");
@@ -783,42 +794,17 @@ nextproto:
 
 	chown("data/protocols.dat",uid,gid);
 
-	mymkdir("bulletins");
-	mymkdir("logfiles");
-	mymkdir("questionnaire");
-	mymkdir("users");
-	mymkdir("temp");
-	mymkdir("configs");
+	mymkdir("bulletins", 0750);
+	mymkdir("logfiles", 0770);
+	mymkdir("questionnaire", 0750);
+	mymkdir("users", 0770);
+	mymkdir("temp", 0770);
+	mymkdir("configs", 0750);
 
 	return 0;
 }
 
-int cfgconf()
-{
-	int confs=0;
-	int i;
-	
-	for (i=0; i != 32 ; i++)
-	{
-		if(*src=='X') confs |= (1L << i);
-		src++;
-	}
-	while(*src!=10 && *src!=13) src++;
-	src=nextspace(src);
-	return confs;
-}
-
-char cfgchar()
-{
-	char charru;
-	
-	charru=src[0];
-	while(*src!=10 && *src!=13) src++;
-	src=nextspace(src);
-	return charru;
-}
-
-int cfgconf2(char *id)
+static int cfgconf2(char *id)
 {
 	int confs=0;
 	int i;
@@ -834,7 +820,7 @@ int cfgconf2(char *id)
 	return confs;
 }
 
-char cfgchar2(char *id)
+static char cfgchar2(char *id)
 {
 	char charru;
 
@@ -845,19 +831,7 @@ char cfgchar2(char *id)
 	return charru;
 }
  
-int cfgflag(char *on)
-{
-	int res=0;
-	
-	if (src[0]==on[0]) res=1;
-
-	while(*src!=10 && *src!=13) src++;
-	src=nextspace(src);
-
-	return res;
-}
-
-int cfgflag2(const char *on, const char *id)
+static int cfgflag2(const char *on, const char *id)
 {
 	int res = 0;
 	
@@ -870,23 +844,7 @@ int cfgflag2(const char *on, const char *id)
 	return res;
 }
 
-int cfgint()
-{
-	char	intb[50];
-	int i;
-	for(i=0;i<50; i++) {
-		if (src[i]==10 || src[i]==13) break;
-		intb[i]=src[i];
-	}
-	intb[i]=0;
-	while(src[i]==' ') i++;
-	
-	src=nextspace(&src[i]);
-	return atoi(intb);
-
-}
-
-int cfgint2(const char *id)
+static int cfgint2(const char *id)
 {
 	char	intb[50];
 	int i;
@@ -903,10 +861,8 @@ int cfgint2(const char *id)
 
 }
 
-
-
-
-char * cfgfnadd3(unsigned short *zone, unsigned short *net, unsigned short *node, unsigned short *point, char *id)
+static char * cfgfnadd3(unsigned short *zone, unsigned short *net, 
+	unsigned short *node, unsigned short *point, char *id)
 {
 	char cb[1024];
 	char *t, *s;
@@ -945,7 +901,7 @@ char * cfgfnadd3(unsigned short *zone, unsigned short *net, unsigned short *node
 
 }
  
-char *cfgcopy(char *dest, int max)
+static char *cfgcopy(char *dest, int max)
 {
 	int i;
 
@@ -970,13 +926,13 @@ char *cfgcopy(char *dest, int max)
 	return 0;
 }
 
-void cfgerror(char *id)
+static void cfgerror(const char *id)
 {
-	fprintf(stderr,"Error in %s: %s missing!\n",gerror,id);
-	exit(-1);
+	fprintf(stderr, "Error in %s: %s missing!\n", gerror, id);
+	exit(1);
 }
 
-char *cfgcopy2(char *dest, int max, const char *ident)
+static char *cfgcopy2(char *dest, int max, const char *ident)
 {
 	int i;
 	src=findstrinblock(ident);
@@ -1001,7 +957,7 @@ char *cfgcopy2(char *dest, int max, const char *ident)
 	return 0;
 }
 
-void removespaces(char *strh)
+static void removespaces(char *strh)
 {
 	char *s;
 	s=strh;
@@ -1012,7 +968,7 @@ void removespaces(char *strh)
 	*(s+1)=0;
 }
 
-char *nextspace(char *s)
+static char *nextspace(char *s)
 {
 	while (*s!=' ') {
 		if (*s==10 && s[1] == '~') break;
@@ -1024,69 +980,75 @@ char *nextspace(char *s)
 	return s;
 }
 
-char *findstrinblock(char *text)
+static char *findstrinblock(const char *text)
 {
 	char *buffb;
 	
-	buffb=blockstart;
-	while(1) {
-		while(*buffb!=10) {
-			if (*buffb==0) return 0;
+	buffb = blockstart;
+	for (;;) {
+		while(*buffb != 10) {
+			if (*buffb == 0) 
+				return 0;
 			buffb++;
 		}
 		buffb++;
-		if (*buffb=='~' || *buffb=='+' || *buffb=='*') {
+		if (*buffb == '~' || *buffb == '+' || *buffb == '*') {
 			return 0;
 		}
-		if (!strncmp(text,buffb,strlen(text))) { 
-			while(*buffb!=' ') buffb++;
+		if (!strncmp(text, buffb, strlen(text))) { 
+			while(*buffb != ' ') 
+				buffb++;
 			return ++buffb;
 		}
 	}
 }
 
-char *findtxt(char *buffer, const char *text)
+static char *findtxt(char *buffer, const char *text)
 {
-	char *s;
+	const char *s;
 	char *buffb;
 nextline:
-	while(*buffer!=10) {
-		if (*buffer==0) return 0;
+	while (*buffer != 10) {
+		if (*buffer == 0) 
+			return 0;
 		buffer++;
 	}
 	
 	buffer++;
-	s=text;
-	buffb=buffer;
+	s = text;
+	buffb = buffer;
 	
-	for(;;)
-	{
-		if (*s==0) {
-			blockstart=buffer;
+	for(;;) {
+		if (*s == 0) {
+			blockstart = buffer;
 			return buffer;
 		}
-		if (*buffer!=*s) goto nextline;
-		s++; buffer++;
+		if (*buffer != *s) 
+			goto nextline;
+		s++; 
+		buffer++;
 	}
 }
 
-char *ecfg(char *hay, const char *need)
+static char *ecfg(char *hay, const char *need)
 {
-	char *s;
-	while(1)
-	{
-		s=need;
-		if (*hay==0) return 0;
-		if (*hay==';') { 
-			while(*hay!=10) {
-				if(*hay==0) return 0;
+	const char *s;
+	for (;;) {
+		s = need;
+		if (*hay == 0) 
+			return 0;
+		if (*hay == ';') { 
+			while (*hay != 10) {
+				if(*hay == 0) 
+					return 0;
 				hay++;
 			}
 			continue;
 		}
-		while (1) {
-			if (*s++==*hay++) {
-				if (*s==0) return hay;
+		for (;;) {
+			if (*s++ == *hay++) {
+				if (*s == 0) 
+				return hay;
 			} else {
 				break;
 			}
@@ -1094,7 +1056,7 @@ char *ecfg(char *hay, const char *need)
 	}
 }
 
-int mymkdir(char *dir)
+static int mymkdir(char *dir, mode_t mode)
 {
 	char *p, ch;
 
@@ -1108,11 +1070,15 @@ int mymkdir(char *dir)
 	ch = *p;
 	*p = 0;
 	
-	mymkdir(dir);
+	mymkdir(dir, mode);
 	
 	*p = ch;
-	if (!mkdir(dir, dmode))
+	if (!mkdir(dir, mode)) {
 		chown(dir, uid, gid);
+		if (chmod(dir, mode) == -1) 
+			fprintf(stderr, "cannot chmod(): %s\n", 
+				strerror(errno));
+	}
 	
 	return 1;
 }
