@@ -20,83 +20,83 @@ struct ddtopwbase {
 	time_t ddw_start;
 };
  
-struct userbase *ub;
-struct userbase *ub2;
+static struct userbase *ub;
+static struct userbase *ub2;
 
-int includesysop=1;
+static int includesysop=1;
 
-int users;
+static int users;
 
-int *ulers;
-int *ulfers;
-uint64_t totuploads=0;
-int totfuploads=0;
+static int *ulers;
+static int *ulfers;
+static uint64_t totuploads=0;
+static int totfuploads=0;
 
-int *dlers;
-int *dlfers;
-uint64_t totdnloads=0;
-int totfdnloads=0;
+static int *dlers;
+static int *dlfers;
+static uint64_t totdnloads=0;
+static int totfdnloads=0;
 
-int *callers;
-int totcalls=0;
+static int *callers;
+static int totcalls=0;
 
-int *msgers;
-int totmsgs=0;
+static int *msgers;
+static int totmsgs=0;
 
-int *wulers;
-int *wulfers;
-uint64_t wtotuploads=0;
-int wtotfuploads=0;
+static int *wulers;
+static int *wulfers;
+static uint64_t wtotuploads=0;
+static int wtotfuploads=0;
 
-int *wdlers;
-int *wdlfers;
-uint64_t wtotdnloads=0;
-int wtotfdnloads=0;
+static int *wdlers;
+static int *wdlfers;
+static uint64_t wtotdnloads=0;
+static int wtotfdnloads=0;
 
-int *wcallers;
-int wtotcalls=0;
+static int *wcallers;
+static int wtotcalls=0;
 
-int *wmsgers;
-int wtotmsgs=0;
+static int *wmsgers;
+static int wtotmsgs=0;
 
-int namemode=0;
-int orgmode=0;
+static int namemode=0;
+static int orgmode=0;
 
-char *script=0;
+static char *script=0;
 
-FILE *scriptfd;
+static FILE *scriptfd;
 
-time_t wtstart;
-time_t currdate;
-time_t wtend;
+static time_t wtstart;
+static time_t currdate;
+static time_t wtend;
 
-struct ddtopwent *wtb;
-struct ddtopwent *wtb2;
+static struct ddtopwent *wtb;
+static struct ddtopwent *wtb2;
 
-int wtusers;
+static int wtusers;
 
-void stripansi(char *);
-char *strspa(char *src, char *dest);
-char * fgetsnolf(char *buf, int n, FILE *fh);
-char *procc(char *s, char *foo);
-char * StyleNum(char *bufferi, uint64_t num);
-void showhelp(void);
-void makeweekbase(void);
-void readwt();
-int enddate(int da);
-void makeuplist(int *);
-void makeupflist(int *);
-void makewuplist(int *);
-void makewupflist(int *);
-void makednlist(int *);
-void makednflist(int *);
-void makewdnlist(int *);
-void makewdnflist(int *);
-void makeclist(int *);
-void makemlist(int *);
-void makewclist(int *);
-void makewmlist(int *);
-void mkmsg(char *, char *, char *, char *);
+static void stripansi(char *);
+static char *strspa(char *src, char *dest);
+static char *fgetsnolf(char *buf, int n, FILE *fh);
+static char *procc(char *s, char *foo);
+static char *StyleNum(char *bufferi, uint64_t num);
+static void showhelp(void);
+static void makeweekbase(void);
+static void readwt(int);
+static int enddate(int);
+static void makeuplist(int *);
+static void makeupflist(int *);
+static void makewuplist(int *);
+static void makewupflist(int *);
+static void makednlist(int *);
+static void makednflist(int *);
+static void makewdnlist(int *);
+static void makewdnflist(int *);
+static void makeclist(int *);
+static void makemlist(int *);
+static void makewclist(int *);
+static void makewmlist(int *);
+static void mkmsg(char *, char *, char *, char *);
 
 int main(int argc, char *argv[])
 {
@@ -334,13 +334,12 @@ int main(int argc, char *argv[])
 		}	
 	}
 	fclose(scriptfd);
-	if (wtend < currdate) {
+	if (wtend < currdate) 
 		makeweekbase();
-	}
-	exit(0);
+	return 0;
 }
 
-void mkmsg(char *d, char *s, char *tx, char *gf)
+static void mkmsg(char *d, char *s, char *tx, char *gf)
 {
 	while(*s)
 	{
@@ -362,7 +361,7 @@ void mkmsg(char *d, char *s, char *tx, char *gf)
 	*d=0;
 }
 
-void readwt(int fd)
+static void readwt(int fd)
 {
 	struct stat st;
 	fstat(fd,&st);
@@ -374,7 +373,7 @@ void readwt(int fd)
 	wtusers=(st.st_size-sizeof(time_t))/sizeof(struct ddtopwent);
 }
 
-char *procc(char *s, char *foo)
+static char *procc(char *s, char *foo)
 {
 	char intb[80];
 	char fom[120];
@@ -770,7 +769,7 @@ char *procc(char *s, char *foo)
 	return s;
 }
 
-char * StyleNum(char *bufferi, uint64_t num)
+static char *StyleNum(char *bufferi, uint64_t num)
 {
 	char tempbuf[60];
 	char *tp;
@@ -805,7 +804,7 @@ char * StyleNum(char *bufferi, uint64_t num)
 	return(bufferi);
 }
 
-void stripansi(char *tempmem)
+static void stripansi(char *tempmem)
 {
 
 	int go=1;
@@ -840,7 +839,7 @@ gooff:;
 	tempmem[t]=0;
 }
 
-char *strspa(char *src, char *dest)
+static char *strspa(char *src, char *dest)
 {
 	int go=1;
 
@@ -857,7 +856,7 @@ char *strspa(char *src, char *dest)
 	return src;
 }
 
-char * fgetsnolf(char *buf, int n, FILE *fh)
+static char *fgetsnolf(char *buf, int n, FILE *fh)
 {
 	char *hih;
 	char *s;
@@ -876,7 +875,7 @@ char * fgetsnolf(char *buf, int n, FILE *fh)
 	return hih;
 }
 
-void showhelp(void)
+static void showhelp(void)
 {
 	printf( "DDTop V1.0\n\n"
 		"Usage:\n"
@@ -886,7 +885,7 @@ void showhelp(void)
 		" -e - exclude sysop\n");
 }
 
-int cmpupf(const void *e1, const void *e2)
+static int cmpupf(const void *e1, const void *e2)
 {
 	struct userbase *ent1, *ent2;
 	ent1=(struct userbase *)e1;
@@ -896,7 +895,7 @@ int cmpupf(const void *e1, const void *e2)
 	return -1;
 }
 
-void makeupflist(int *ul)
+static void makeupflist(int *ul)
 {
 	struct userbase *u;
 	int i;
@@ -915,7 +914,7 @@ void makeupflist(int *ul)
 	}
 }
 
-int cmpupb(const void *e1, const void *e2)
+static int cmpupb(const void *e1, const void *e2)
 {
 	struct userbase *ent1, *ent2;
 	ent1=(struct userbase *)e1;
@@ -925,7 +924,7 @@ int cmpupb(const void *e1, const void *e2)
 	return -1;
 }
 
-void makeuplist(int *ulers)
+static void makeuplist(int *ulers)
 {
 	struct userbase *u;
 	int i;
@@ -945,7 +944,7 @@ void makeuplist(int *ulers)
 }
 
 
-int wcmpupf(const void *e1, const void *e2)
+static int wcmpupf(const void *e1, const void *e2)
 {
 	struct ddtopwent *ent1, *ent2;
 	ent1=(struct ddtopwent *)e1;
@@ -956,7 +955,7 @@ int wcmpupf(const void *e1, const void *e2)
 	return -1;
 }
 
-void makewupflist(int *ul)
+static void makewupflist(int *ul)
 {
 	struct ddtopwent *u;
 	int i;
@@ -977,7 +976,7 @@ void makewupflist(int *ul)
 	}
 }
 
-int wcmpupb(const void *e1, const void *e2)
+static int wcmpupb(const void *e1, const void *e2)
 {
 	struct ddtopwent *ent1, *ent2;
 	ent1=(struct ddtopwent *)e1;
@@ -988,7 +987,7 @@ int wcmpupb(const void *e1, const void *e2)
 	return -1;
 }
 
-void makewuplist(int *ul)
+static void makewuplist(int *ul)
 {
 	struct ddtopwent *u;
 	int i;
@@ -1009,7 +1008,7 @@ void makewuplist(int *ul)
 	}
 }
 
-int wcmpdlf(const void *e1, const void *e2)
+static int wcmpdlf(const void *e1, const void *e2)
 {
 	struct ddtopwent *ent1, *ent2;
 	ent1=(struct ddtopwent *)e1;
@@ -1020,7 +1019,7 @@ int wcmpdlf(const void *e1, const void *e2)
 	return -1;
 }
 
-void makewdnflist(int *ul)
+static void makewdnflist(int *ul)
 {
 	struct ddtopwent *u;
 	int i;
@@ -1041,7 +1040,7 @@ void makewdnflist(int *ul)
 	}
 }
 
-int wcmpdlb(const void *e1, const void *e2)
+static int wcmpdlb(const void *e1, const void *e2)
 {
 	struct ddtopwent *ent1, *ent2;
 	ent1=(struct ddtopwent *)e1;
@@ -1052,7 +1051,7 @@ int wcmpdlb(const void *e1, const void *e2)
 	return -1;
 }
 
-void makewdnlist(int *ul)
+static void makewdnlist(int *ul)
 {
 	struct ddtopwent *u;
 	int i;
@@ -1073,7 +1072,7 @@ void makewdnlist(int *ul)
 	}
 }
 
-int wcmpcall(const void *e1, const void *e2)
+static int wcmpcall(const void *e1, const void *e2)
 {
 	struct ddtopwent *ent1, *ent2;
 	ent1=(struct ddtopwent *)e1;
@@ -1084,7 +1083,7 @@ int wcmpcall(const void *e1, const void *e2)
 	return -1;
 }
 
-void makewclist(int *ul)
+static void makewclist(int *ul)
 {
 	struct ddtopwent *u;
 	int i;
@@ -1105,7 +1104,7 @@ void makewclist(int *ul)
 	}
 }
 
-int wcmpmsg(const void *e1, const void *e2)
+static int wcmpmsg(const void *e1, const void *e2)
 {
 	struct ddtopwent *ent1, *ent2;
 	ent1=(struct ddtopwent *)e1;
@@ -1116,7 +1115,7 @@ int wcmpmsg(const void *e1, const void *e2)
 	return -1;
 }
 
-void makewmlist(int *ul)
+static void makewmlist(int *ul)
 {
 	struct ddtopwent *u;
 	int i;
@@ -1138,7 +1137,7 @@ void makewmlist(int *ul)
 }
 
 
-int cmpdnf(const void *e1, const void *e2)
+static int cmpdnf(const void *e1, const void *e2)
 {
 	struct userbase *ent1, *ent2;
 	ent1=(struct userbase *)e1;
@@ -1148,7 +1147,7 @@ int cmpdnf(const void *e1, const void *e2)
 	return -1;
 }
 
-void makednflist(int *ul)
+static void makednflist(int *ul)
 {
 	struct userbase *u;
 	int i;
@@ -1167,7 +1166,7 @@ void makednflist(int *ul)
 	}
 }
 
-int cmpdnb(const void *e1, const void *e2)
+static int cmpdnb(const void *e1, const void *e2)
 {
 	struct userbase *ent1, *ent2;
 	ent1=(struct userbase *)e1;
@@ -1177,7 +1176,7 @@ int cmpdnb(const void *e1, const void *e2)
 	return -1;
 }
 
-void makednlist(int *ul)
+static void makednlist(int *ul)
 {
 	struct userbase *u;
 	int i;
@@ -1196,7 +1195,7 @@ void makednlist(int *ul)
 	}
 }
 
-int cmpcall(const void *e1, const void *e2)
+static int cmpcall(const void *e1, const void *e2)
 {
 	struct userbase *ent1, *ent2;
 	ent1=(struct userbase *)e1;
@@ -1206,7 +1205,7 @@ int cmpcall(const void *e1, const void *e2)
 	return -1;
 }
 
-void makeclist(int *ul)
+static void makeclist(int *ul)
 {
 	struct userbase *u;
 	int i;
@@ -1225,7 +1224,7 @@ void makeclist(int *ul)
 	}
 }
 
-int cmpm(const void *e1, const void *e2)
+static int cmpm(const void *e1, const void *e2)
 {
 	struct userbase *ent1, *ent2;
 	ent1=(struct userbase *)e1;
@@ -1235,7 +1234,7 @@ int cmpm(const void *e1, const void *e2)
 	return -1;
 }
 
-void makemlist(int *ul)
+static void makemlist(int *ul)
 {
 	struct userbase *u;
 	int i;
@@ -1254,7 +1253,7 @@ void makemlist(int *ul)
 	}
 }
 
-void makeweekbase(void)
+static void makeweekbase(void)
 {
 	int wfd;
 	time_t t;
@@ -1285,7 +1284,7 @@ void makeweekbase(void)
 	close(wfd);
 }
 
-int enddate(int da)
+static int enddate(int da)
 {
 	int i;
 	da-=(252460800/(60*60*24));
