@@ -1,7 +1,6 @@
-#include <dd.h>
-#include <ddlib.h>
 #include <fcntl.h>
 #include <stdlib.h>
+#include <sys/types.h>
 #include <unistd.h>
 #include <string.h>
 #include <sys/stat.h>
@@ -10,14 +9,20 @@
 #include <ctype.h>
 
 #include <ddcommon.h>
+#include <dd.h>
+#include <ddlib.h>
 
-char *cfg;
-struct DayDream_MainConfig maincfg;
+enum {
+	MAILTYPE_ECHO = 0x01,
+	MAILTYPE_NET  = 0x02
+};
 
-struct DayDream_Conference *confs;
-struct DayDream_Conference *tc;
-struct DayDream_MsgBase *tb;
+static char *cfg;
+static struct DayDream_MainConfig maincfg;
 
+static struct DayDream_Conference *confs;
+static struct DayDream_Conference *tc;
+static struct DayDream_MsgBase *tb;
 
 static const char *months[] = {
       "Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"
@@ -58,10 +63,10 @@ struct basei {
 	int basen;
 };
 
-unsigned char ctable[256];
-int usetab=0;
+static unsigned char ctable[256];
+static int usetab=0;
 
-struct List *outlist;
+static struct List *outlist;
 
 struct ftsheader {
 	unsigned short origNode;
@@ -103,44 +108,43 @@ struct ftsmsg {
 };
 
 
-char incoming[1024];
-char outgoing[1024];
-char unpdir[1024];
-char olddir[1024];
-char unpcmd[1024];
+static char incoming[1024];
+static char outgoing[1024];
+static char unpdir[1024];
+static char olddir[1024];
+static char unpcmd[1024];
 
-struct basei bad;
-struct basei netm;
-int highest, lowest;
-
-struct List *NewList(void);
-void die(void);
-int parsecfg(void);
-char *ExamineCfg(char *, const char *);
-char *Fnd(char *, char *);
-time_t ftndatetodd(char *);
-char *strtoaka(char *s, struct aka *ak);
-struct onodeinfo *findni(struct aka *ak);
-int export(void);
-void inbound(void);
-void closeout(void);
-int procnmsg(struct DayDream_Message *);
-int procemsg(struct DayDream_Message *);
-int wildcmp (char *, char *);
-void ftndate(time_t, char *);
-int writenetom(char *, struct onodeinfo *, struct ftsmsg *, struct DayDream_Message *);
-void makeseenby(char *, char *);
-void cpytolaina(char *, char *);
-int checkarea(struct aka *, int, int);
-void handleipkt(char *);
-int procpkt(char *);
-int locatetag(char *);
-void getmsgptrs(void);
-int setmsgptrs(void);
-void makeftsh(struct onodeinfo *);
-void AddTail(struct List *, struct Node *);
-void convcpy(char *, char *, int);
+static struct basei bad;
+static struct basei netm;
+static int highest, lowest;
 
+static struct List *NewList(void);
+static void die(void);
+static int parsecfg(void);
+static char *ExamineCfg(char *, const char *);
+static time_t ftndatetodd(char *);
+static char *strtoaka(char *s, struct aka *ak);
+static struct onodeinfo *findni(struct aka *ak);
+static int export(void);
+static void inbound(void);
+static void closeout(void);
+static int procnmsg(struct DayDream_Message *);
+static int procemsg(struct DayDream_Message *);
+static int wildcmp (char *, char *);
+static void ftndate(time_t, char *);
+static int writenetom(char *, struct onodeinfo *, 
+	struct ftsmsg *, struct DayDream_Message *);
+static void makeseenby(char *, char *);
+static void cpytolaina(char *, char *);
+static int checkarea(struct aka *, int, int);
+static void handleipkt(char *);
+static int procpkt(char *);
+static int locatetag(char *);
+static void getmsgptrs(void);
+static int setmsgptrs(void);
+static void makeftsh(struct onodeinfo *);
+static void AddTail(struct List *, struct Node *);
+static void convcpy(char *, char *, int);
 
 int main(int argc, char *argv[])
 {
@@ -148,6 +152,8 @@ int main(int argc, char *argv[])
 	int cfgfd;
 	struct stat st;
 	
+	init_common_library(argc, argv);
+
 	umask(~(0775));
 
 	sprintf(buf,"%s/configs/tosser.cfg",getenv("DAYDREAM"));
@@ -157,8 +163,8 @@ int main(int argc, char *argv[])
 	}
 	cfg=(char *)malloc(st.st_size+2);
 	memset(cfg,0,st.st_size+2);
-	cfgfd=open(buf,O_RDONLY);
-	if (cfgfd < 0) {
+	cfgfd = open(buf, O_RDONLY);
+	if (cfgfd == -1) {
 		perror("Configfile error");
 		exit(0);
 	}
@@ -166,8 +172,8 @@ int main(int argc, char *argv[])
 	close(cfgfd);
 
 	sprintf(buf,"%s/data/daydream.dat",getenv("DAYDREAM"));
-	cfgfd=open(buf,O_RDONLY);
-	if (cfgfd < 0) {
+	cfgfd = open(buf, O_RDONLY);
+	if (cfgfd == -1) {
 		perror("Can't open DD datafiles");
 		exit(0);
 	}
@@ -198,7 +204,7 @@ int main(int argc, char *argv[])
 	return 0;
 }
 
-void deldir(char *dir)
+static void deldir(char *dir)
 {
 	DIR *dirfd;
 	struct dirent *ent;
@@ -225,111 +231,86 @@ void deldir(char *dir)
 	} 
 }
 
-void die(void)
+static void die(void)
 {
 	chdir(olddir);
 	deldir(unpdir);
 	rmdir(unpdir);
 }
 
-int export(void)
+static int process_mail(const char *filename, int type)
+{
+	struct DayDream_Message msg;
+	int fd, retvalue;
+	off_t ptr;
+
+	fd = open(filename, O_RDWR);
+	if (fd == -1)
+		return -1;
+	ptr = 0;
+	while (pread(fd, &msg, sizeof(struct DayDream_Message), ptr) ==
+		sizeof(struct DayDream_Message)) {
+		if ((msg.MSG_FLAGS & (1L << 2)) == 0)
+			goto next_message;
+
+		if (type == MAILTYPE_ECHO)
+			retvalue = procemsg(&msg);
+		else if (type == MAILTYPE_NET)
+			retvalue = procnmsg(&msg);
+		else 
+			abort();	
+			
+		if (!retvalue) {
+			fputs("warning: cannot process echomail.\n", stderr);
+			goto next_message;
+		}
+		msg.MSG_FLAGS &= ~(1L << 2);
+		if (pwrite(fd, &msg, sizeof(struct DayDream_Message), ptr) !=
+			sizeof(struct DayDream_Message)) {
+			fputs("error: short write, cannot process mail.\n", stderr);
+			break;
+		}
+next_message:
+		ptr += sizeof(struct DayDream_Message);		
+	}
+	close(fd);
+	return 0;
+}
+
+static int export(void)
 {
 	confs=(struct DayDream_Conference *)dd_getconfdata();
 	if (!confs) return 0;
 	tc=confs;
 
-	while(1)
-	{
-		int newcnt;
-		struct stat st;
+	while(1) {
 		char mailbuf[1024];
 		int bcnt;
 		
-		if (tc->CONF_NUMBER==255) break;
+		if (tc->CONF_NUMBER == 255) 
+			break;
 
-		(struct DayDream_Conference *)tb=tc+1;
-		for(bcnt=tc->CONF_MSGBASES;bcnt;bcnt--,tb++)
-		{
-			int basefd;
-			newcnt=0;
-			if (toupper(tb->MSGBASE_FN_FLAGS)=='E') {
-				struct DayDream_Message msg;
-				int ols;
-				
-				if (!*tb->MSGBASE_FN_TAG) continue;
-
-					
-				sprintf(mailbuf,"%smessages/base%3.3d/msgbase.dat",tc->CONF_PATH,tb->MSGBASE_NUMBER);
-				basefd=open(mailbuf,O_RDWR);
-				if (basefd==-1) {
-					perror(mailbuf);
+		/* FIXME: Very suspicious code */
+		tb = (struct DayDream_MsgBase *) tc + 1;
+		for(bcnt = tc->CONF_MSGBASES; bcnt; bcnt--, tb++) {
+			sprintf(mailbuf, "%s/messages/base%3.3d/msgbase.dat",
+				tc->CONF_PATH, tb->MSGBASE_NUMBER);
+			
+			if (toupper(tb->MSGBASE_FN_FLAGS) == 'E') {
+				/* Fido tag required */
+				if (!*tb->MSGBASE_FN_TAG) 
 					continue;
-				}
-				fstat(basefd,&st);
-
-				ols = dd_lseek(basefd, -sizeof(struct DayDream_Message), SEEK_END);
-				while(1) {
-					int naama;
-				       
-					if (!read(basefd,&msg,sizeof(struct DayDream_Message))) break;
-					if (!(msg.MSG_FLAGS & (1L<<2))) break;
-					naama=lseek(basefd,-sizeof(struct DayDream_Message)*2,SEEK_CUR);
-					if (naama==-1) break;
-					if (naama==ols) break;
-					ols=naama;
-				}
-				
-				while(read(basefd,&msg,sizeof(struct DayDream_Message)))
-				{
-					if (procemsg(&msg)) {
-						lseek(basefd,-sizeof(struct DayDream_Message),SEEK_CUR);
-						msg.MSG_FLAGS &= ~(1L<<2);
-						write(basefd,&msg,sizeof(struct DayDream_Message));
-					}
-				}
-				close(basefd);
-			} else if (toupper(tb->MSGBASE_FN_FLAGS)=='N') {
-				struct DayDream_Message msg;
-				int ols;
-				
-				sprintf(mailbuf,"%smessages/base%3.3d/msgbase.dat",tc->CONF_PATH,tb->MSGBASE_NUMBER);
-				basefd=open(mailbuf,O_RDWR);
-				if (basefd==-1) continue;
-					
-				fstat(basefd,&st);
-
-				ols = dd_lseek(basefd, -sizeof(struct DayDream_Message), SEEK_END);
-				while(1) {
-					int naama;
-				       
-					if (!read(basefd,&msg,sizeof(struct DayDream_Message))) break;
-					if (!(msg.MSG_FLAGS & (1L<<2))) break;
-					naama=lseek(basefd,-sizeof(struct DayDream_Message)*2,SEEK_CUR);
-					if (naama==-1) break;
-					if (naama==ols) break;
-					ols=naama;
-				}
-				
-				while(read(basefd,&msg,sizeof(struct DayDream_Message)))
-				{
-					if (procnmsg(&msg)) {
-						lseek(basefd,-sizeof(struct DayDream_Message),SEEK_CUR);
-						msg.MSG_FLAGS &= ~(1L<<2);
-						write(basefd,&msg,sizeof(struct DayDream_Message));
-					}
-				}
-				close(basefd);
-				
-				
-			}
+				process_mail(mailbuf, MAILTYPE_ECHO);
+			} else if (toupper(tb->MSGBASE_FN_FLAGS) == 'N') 
+				process_mail(mailbuf, MAILTYPE_NET);
 		}
-		(struct DayDream_MsgBase *)tc=tb;
-	
+		/* FIXME: Very suspicious code */
+		tc = (struct DayDream_Conference *) tb;
 	}
 	return 1;
 }
 
-int routeadd(struct aka *sak, struct aka *dak)
+static int routeadd(struct aka *sak, struct aka *dak)
 {
 	char buf1[512];
 	char buf2[512];
@@ -354,7 +335,7 @@ int routeadd(struct aka *sak, struct aka *dak)
 	return 1;
 }
 
-int wildcmp (char *nam, char *pat)
+static int wildcmp (char *nam, char *pat)
 {
 	register char *p;              /* Thu Jan 16 14:50:30 1992 */
 
@@ -382,7 +363,7 @@ int wildcmp (char *nam, char *pat)
 	return 0;
 }
 
-int procnmsg(struct DayDream_Message *msg)
+static int procnmsg(struct DayDream_Message *msg)
 {
 	struct ftsmsg me;
 	struct aka ak;
@@ -396,7 +377,7 @@ int procnmsg(struct DayDream_Message *msg)
 	struct onodeinfo *on;
 	if (!(msg->MSG_FLAGS & (1L<<2))) return 1;
 	
-	sprintf(msgn,"%smessages/base%3.3d/msg%5.5d",tc->CONF_PATH,tb->MSGBASE_NUMBER,msg->MSG_NUMBER);
+	sprintf(msgn,"%s/messages/base%3.3d/msg%5.5d",tc->CONF_PATH,tb->MSGBASE_NUMBER,msg->MSG_NUMBER);
 	if (stat(msgn,&st)==-1) return 0;
 	
 	memset(&me,0,sizeof(struct ftsmsg));
@@ -452,7 +433,7 @@ int procnmsg(struct DayDream_Message *msg)
 	return 1;
 }
 
-int writenetom(char *msgname, struct onodeinfo *on, struct ftsmsg *me, struct DayDream_Message *msg) 
+static int writenetom(char *msgname, struct onodeinfo *on, struct ftsmsg *me, struct DayDream_Message *msg) 
 {
 	
 	FILE *of;
@@ -499,7 +480,7 @@ int writenetom(char *msgname, struct onodeinfo *on, struct ftsmsg *me, struct Da
 	return 1;
 }
 
-int procemsg(struct DayDream_Message *msg)
+static int procemsg(struct DayDream_Message *msg)
 {
 	struct ftsmsg me;
 	struct aka ak;
@@ -511,7 +492,7 @@ int procemsg(struct DayDream_Message *msg)
 	
 	if (!(msg->MSG_FLAGS & (1L<<2))) return 0;
 
-	sprintf(buf,"%smessages/base%3.3d/msg%5.5d",tc->CONF_PATH,tb->MSGBASE_NUMBER,msg->MSG_NUMBER);
+	sprintf(buf,"%s/messages/base%3.3d/msg%5.5d",tc->CONF_PATH,tb->MSGBASE_NUMBER,msg->MSG_NUMBER);
 	if (stat(buf,&st)==-1) {
 		perror(buf);
 		return 0;
@@ -641,7 +622,7 @@ int procemsg(struct DayDream_Message *msg)
 	return 1;
 }
 
-void closeout(void)
+static void closeout(void)
 {
 	struct onodeinfo *myo;
 	const char *days[] = { "su", "mo", "tu", "we", "th", "fr", "sa" };
@@ -737,7 +718,7 @@ void closeout(void)
 	}
 }
 
-void makeseenby(char *s, char *buf)
+static void makeseenby(char *s, char *buf)
 {
 	unsigned short ams[1024];
 	unsigned short ams2[1024];
@@ -897,7 +878,7 @@ void makeseenby(char *s, char *buf)
 	strcat(buf,"\r");
 }
 
-void inbound(void)
+static void inbound(void)
 {
 	struct dirent *de;
 	DIR *dh;
@@ -926,7 +907,7 @@ void inbound(void)
 	}
 }
 
-struct onodeinfo *findni(struct aka *ak)
+static struct onodeinfo *findni(struct aka *ak)
 {
 	struct onodeinfo *myo;
 	
@@ -944,7 +925,7 @@ struct onodeinfo *findni(struct aka *ak)
 	return 0;
 }
 
-int checkarea(struct aka *ak, int conf, int base)
+static int checkarea(struct aka *ak, int conf, int base)
 {
 	
 	char buf[1024];
@@ -964,7 +945,7 @@ int checkarea(struct aka *ak, int conf, int base)
 	return 0;	
 }
 
-void handleipkt(char *pk)
+static void handleipkt(char *pk)
 {
 	char pbuf[1024];
 	struct dirent *de;
@@ -987,7 +968,7 @@ void handleipkt(char *pk)
 	}
 }
 
-int procpkt(char *pn)
+static int procpkt(char *pn)
 {
 	int pfd;
 	int seekpoint;
@@ -1089,7 +1070,7 @@ int procpkt(char *pn)
 		msg.MSG_NUMBER=highest;
 		setmsgptrs();
 		msg.MSG_FLAGS |= (1L<<2);
-		sprintf(qbuf,"%smessages/base%3.3d/msgbase.dat",tc->CONF_PATH,tb->MSGBASE_NUMBER);
+		sprintf(qbuf,"%s/messages/base%3.3d/msgbase.dat",tc->CONF_PATH,tb->MSGBASE_NUMBER);
 
 		if ((msgfd=open(qbuf,O_RDWR|O_CREAT,0664)) < 0) {
 			printf("Fatal write error!\n");
@@ -1099,7 +1080,7 @@ int procpkt(char *pn)
 		write(msgfd,&msg,sizeof(struct DayDream_Message));
 		close(msgfd);
 
-		sprintf(qbuf,"%smessages/base%3.3d/msg%5.5d",tc->CONF_PATH,tb->MSGBASE_NUMBER,highest);
+		sprintf(qbuf,"%s/messages/base%3.3d/msg%5.5d",tc->CONF_PATH,tb->MSGBASE_NUMBER,highest);
 		msgf=fopen(qbuf,"w");
 		
 		if (!msgf) {
@@ -1135,7 +1116,7 @@ int procpkt(char *pn)
 
 }
 
-void convcpy(char *dest, char *src, int n)
+static void convcpy(char *dest, char *src, int n)
 {
 	if (usetab) {
 		while(n && *src) *dest++=ctable[(unsigned char)*src++];
@@ -1145,13 +1126,13 @@ void convcpy(char *dest, char *src, int n)
 	}
 }
 
-void getmsgptrs(void) 
+static void getmsgptrs(void) 
 {
 	int msgfd;
 	struct DayDream_MsgPointers ptrs;
 	
 	char gmpbuf[1024];
-	sprintf(gmpbuf,"%smessages/base%3.3d/msgbase.ptr",tc->CONF_PATH,tb->MSGBASE_NUMBER);
+	sprintf(gmpbuf,"%s/messages/base%3.3d/msgbase.ptr",tc->CONF_PATH,tb->MSGBASE_NUMBER);
 
 	if ((msgfd=open(gmpbuf,O_RDONLY)) < 0) {
 		highest=0;
@@ -1165,13 +1146,13 @@ void getmsgptrs(void)
 	lowest=ptrs.msp_low;	
 }
 
-int setmsgptrs(void)
+static int setmsgptrs(void)
 {
 	int msgfd;
 	struct DayDream_MsgPointers ptrs;
 	
 	char gmpbuf[1024];
-	sprintf(gmpbuf,"%smessages/base%3.3d/msgbase.ptr",tc->CONF_PATH,tb->MSGBASE_NUMBER);
+	sprintf(gmpbuf,"%s/messages/base%3.3d/msgbase.ptr",tc->CONF_PATH,tb->MSGBASE_NUMBER);
 
 	if ((msgfd=open(gmpbuf,O_RDWR|O_CREAT,0664)) < 0) {
 		return 0;
@@ -1184,7 +1165,7 @@ int setmsgptrs(void)
 	return 1;
 }
 
-int locatetag(char *s)
+static int locatetag(char *s)
 {
 	char bu[1024];
 	char *t;
@@ -1202,7 +1183,8 @@ int locatetag(char *s)
 		int bcnt;
 		if (mconf->CONF_NUMBER==255) break;
 		
-		(struct DayDream_Conference *)mbase=mconf+1;
+		/* FIXME: Very suspicious code */
+		mbase = (struct DayDream_MsgBase *) mconf + 1;
 		bcnt=mconf->CONF_MSGBASES;
 		for(bcnt=mconf->CONF_MSGBASES;bcnt;bcnt--,mbase++)
 		{
@@ -1212,7 +1194,8 @@ int locatetag(char *s)
 				return 1;
 			}
 		}	
-		(struct DayDream_MsgBase *)mconf=mbase;
+		/* FIXME: Very suspicious code */
+		mconf = (struct DayDream_Conference *) mbase;
 	}
 	tc=dd_getconf(bad.confn);
 	tb=dd_getbase(bad.confn,bad.basen);
@@ -1223,7 +1206,7 @@ int locatetag(char *s)
 	return 1;
 }
 
-void getbasei(char *s, struct basei *b)
+static void getbasei(char *s, struct basei *b)
 {
 	char bu[1024];
 	char *t;
@@ -1240,7 +1223,7 @@ void getbasei(char *s, struct basei *b)
 	b->basen=atoi(bu);
 }
 
-time_t ftndatetodd(char *t)
+static time_t ftndatetodd(char *t)
 {
 	struct tm tm;
 	char foob[256];
@@ -1300,7 +1283,7 @@ time_t ftndatetodd(char *t)
 	return mktime(&tm);
 }
 
-int parsecfg(void)
+static int parsecfg(void)
 {
 	char *s;
 	struct basei bad;
@@ -1419,7 +1402,7 @@ int parsecfg(void)
 	return 1;
 }
 
-void makeftsh(struct onodeinfo *on)
+static void makeftsh(struct onodeinfo *on)
 {
 	struct ftsheader mh;
 	struct tm *tm;
@@ -1449,14 +1432,13 @@ void makeftsh(struct onodeinfo *on)
 	write(on->NI_FHandle,&mh,sizeof(struct ftsheader));
 }
 
-
-void cpytolaina(char *s, char *t)
+static void cpytolaina(char *s, char *t)
 {
 	while(*t!='\"' && *t) *s++=*t++;
 	*s=0;
 }
 
-char *strtoaka(char *s, struct aka *ak)
+static char *strtoaka(char *s, struct aka *ak)
 {
 	char *t;
 	char cb[1024];
@@ -1490,38 +1472,35 @@ char *strtoaka(char *s, struct aka *ak)
 	return s;
 }
 
-char *ExamineCfg(char *hay, const char *need)
+static char *ExamineCfg(char *hay, const char *need)
 {
-	char *s;
-	while(1)
-	{
-		s=need;
-		if (*hay==0) return 0;
-		if (*hay==';') { 
-			while(*hay!=10) {
-				if(*hay==0) return 0;
+	const char *s;
+	for (;;) {
+		s = need;
+		if (*hay == 0) return 0;
+		if (*hay == ';') { 
+			while(*hay != 10) {
+				if(*hay == 0) 
+					return 0;
 				hay++;
 			}
 			continue;
 		}
-		while (1) {
-			if (*s++==*hay) {
+		for (;;) {
+			if (*s++ == *hay) {
 				hay++;
-				if (*s==0) return hay;
+				if (*s == 0) 
+					return hay;
 			} else {
-				if ((s-1)==need) hay++;
+				if ((s-1) == need) 
+					hay++;
 				break;
 			}
 		}
 	}
 }
 
-char * Fnd(char *needle, char *haystack)
-{
-	return strstr(haystack, needle);
-}
-
-struct List * NewList(void)
+static struct List * NewList(void)
 {
 	struct List *myl;
 	
@@ -1532,7 +1511,7 @@ struct List * NewList(void)
 	return myl;
 }  
 
-void AddTail(struct List *myl, struct Node *myn)
+static void AddTail(struct List *myl, struct Node *myn)
 {
 	struct Node *tnod;
 	
@@ -1543,21 +1522,7 @@ void AddTail(struct List *myl, struct Node *myn)
 	myn->ln_Pred=tnod;
 }
 
-struct Node *RemTail(struct List *myl)
-{
-	struct Node *tnod;
-	struct Node *pnod;
-	
-	if (myl->lh_TailPred == (struct Node *)&myl->lh_Head) return 0;
-	
-	tnod=myl->lh_TailPred;
-	pnod=tnod->ln_Pred;
-	pnod->ln_Succ=(struct Node *)&myl->lh_Tail;
-	myl->lh_TailPred=pnod;
-	return tnod;
-}
-
-void ftndate(time_t t, char *buf)
+static void ftndate(time_t t, char *buf)
 {
         struct tm *ptm;
                 
@@ -1566,5 +1531,3 @@ void ftndate(time_t t, char *buf)
                 months[ptm->tm_mon],ptm->tm_year%100,
                 ptm->tm_hour,ptm->tm_min,ptm->tm_sec);
 }    
-
-                                                                    
